@@ -7,7 +7,7 @@ using a Metropolis-Hastings Monte Carlo algorithm
 	AIC model test   R O U T I N E S
 
 	Peter Beerli 2001, Seattle
-	beerli@csit.fsu.edu
+	beerli@scs.fsu.edu
 
 	Copyright 2001-2002 Peter Beerli and Joseph Felsenstein
 
@@ -15,7 +15,7 @@ using a Metropolis-Hastings Monte Carlo algorithm
 	and is copyrighted. Of course, we do not guarantee that the software
 	works and are not responsible for any damage you may cause or have.
 
-$Id: aic.c 175 2005-12-02 14:51:23Z beerli $ 
+$Id: aic.c 633 2007-03-06 04:14:07Z beerli $ 
 -------------------------------------------------------*/
 
 #include "migration.h"
@@ -32,44 +32,44 @@ $Id: aic.c 175 2005-12-02 14:51:23Z beerli $
 #endif
 
 
-void print_progressheader_aic(boolean progress, FILE *file, MYREAL mleaic, MYREAL numparam);
+static void print_progressheader_aic(boolean progress, FILE *file, MYREAL mleaic, MYREAL numparam);
 
-void aic_score (aic_fmt ** aicvec, long *aicnum, nr_fmt * nr,
+static void aic_score (aic_fmt ** aicvec, long *aicnum, nr_fmt * nr,
                 long zero, long which, char *temppattern, MYREAL *param0,
-                char migtype);
+                int migtype);
 
-boolean legal_pattern (char *matrix, long numpop);
+static boolean legal_pattern (char *matrix, long numpop);
 
 void fast_aic_score (aic_fmt ** aicvec, long *aicnum, nr_fmt * nr,
                      long zero, long which, char *temppattern,
-                     MYREAL *param0, char migtype);
+                     MYREAL *param0, int migtype);
 
-void add_aiclist (char migtype, long numparam, long freeparam, long remainnum, long zero, long m,
+static void add_aiclist (int migtype, long numparam, long freeparam, long remainnum, long zero, long m,
                   MYREAL *param0, char *temppattern, char *custm2,
                   long *aicnum, aic_fmt ** aicvec, nr_fmt * nr);
 
-void add_aicvec(MYREAL aic, aic_fmt **aicvec, long *aicnum, nr_fmt *nr, long numparam, long remainnum);
+static void add_aicvec(MYREAL aic, aic_fmt **aicvec, long *aicnum, nr_fmt *nr, long numparam, long remainnum);
 
-void aic_print_driver(aic_fmt *aicvec, MYREAL mleaic, long aicnum, nr_fmt *nr, world_fmt *world);
+static void aic_print_driver(aic_fmt *aicvec, MYREAL mleaic, long aicnum, nr_fmt *nr, world_fmt *world);
 
-void aic_score_driver(aic_struct *aic, MYREAL mle, MYREAL mleaic, nr_fmt *nr, world_fmt *world, char *temppattern);
+static void aic_score_driver(aic_struct *aic, MYREAL mle, MYREAL mleaic, nr_fmt *nr, world_fmt *world, char *temppattern);
 
-void akaike_information (world_fmt * world, long *Gmax);
+//void akaike_information (world_fmt * world, long *Gmax);
 
-void print_header_aic (nr_fmt * nr, MYREAL mleaic);
+static void print_header_aic (nr_fmt * nr, MYREAL mleaic);
 
-void print_aicfile(aic_fmt **aicvec, long *aicnum, nr_fmt *nr);
+static void print_aicfile(aic_fmt **aicvec, long *aicnum, nr_fmt *nr);
 
-void print_progress_aic(boolean add
+static void print_progress_aic(boolean add
 						, aic_fmt **aicvec, long *aicnum, nr_fmt *nr, long numparam, long freeparam);
 
-long find_paramnum(world_fmt *world, char *connect);
+//static long find_paramnum(world_fmt *world, char *connect);
 
 ///
 /// changes a linear matrix of ddd mm mm mm
 /// to a diagonal matrix dmm mdm mmd
 /// it destroys pattern
-char * reshuffle (char *pattern, char *origpattern, long numpop)
+static char * reshuffle (char *pattern, char *origpattern, long numpop)
 {
     long space = 0;
     long i, j, z = numpop;
@@ -169,7 +169,7 @@ void aic_score_driver(aic_struct *aic, MYREAL mle, MYREAL mleaic, nr_fmt *nr, wo
     }
     memcpy (world->options->custm2, savecustm, sizeof (char) * nr->numpop2);
     memcpy (world->options->custm, savecustm2, sizeof (char) * nr->numpop2);
-    free(savecustm); // savecustom2 is also freed, because it's part of savecustm
+    myfree(savecustm); // savecustom2 is also freed, because it's part of savecustm
 }
 
 ///
@@ -179,12 +179,17 @@ void aic_score_driver(aic_struct *aic, MYREAL mle, MYREAL mleaic, nr_fmt *nr, wo
 /// aic value, number of parameters, MLE of pattern, Prob of standard LRT, Prob of weighted chisquare
 void aic_print_driver(aic_fmt *aicvec, MYREAL mleaic, long aicnum, nr_fmt *nr, world_fmt *world)
 {
-    long i;
-    char *temppattern;
-    boolean mldone=FALSE;
-    char dashes[101] = "----------------------------------------------------------------------------------------------------";
-    temppattern =  (char *) mycalloc (world->numpop2 + 1 + world->numpop, sizeof (char));
-    FPRINTF (world->outfile,
+  //#ifdef MYREAL == float
+  //const MYREAL eps = FLT_EPSILON ;
+  //#else
+  const MYREAL eps = DBL_EPSILON ;
+  //#endif
+  long i;
+  char *temppattern;
+  boolean mldone=FALSE;
+  char dashes[101] = "----------------------------------------------------------------------------------------------------";
+  temppattern =  (char *) mycalloc (world->numpop2 + 1 + world->numpop, sizeof (char));
+  FPRINTF (world->outfile,
              "%-*.*s  AIC     #param   Log(L)   LRT        Prob    Probc\n",
              (int) MAX (18, nr->partsize + nr->numpop),
              (int) (nr->partsize + nr->numpop), "Pattern");
@@ -201,7 +206,7 @@ void aic_print_driver(aic_fmt *aicvec, MYREAL mleaic, long aicnum, nr_fmt *nr, w
                  reshuffle (temppattern, aicvec[i].pattern, nr->numpop),
                  aicvec[i].aic, aicvec[i].numparam, aicvec[i].mle,
                  aicvec[i].lrt, aicvec[i].prob, aicvec[i].probcorr);
-        if ((aicvec[i].aic - mleaic > DBL_EPSILON) && !mldone)
+        if ((aicvec[i].aic - mleaic > eps) && !mldone)
         {
             mldone = TRUE;
             FPRINTF (world->outfile, "%-*.*s %53.53s\n",
@@ -209,10 +214,10 @@ void aic_print_driver(aic_fmt *aicvec, MYREAL mleaic, long aicnum, nr_fmt *nr, w
                      (int) (nr->partsize + nr->numpop),dashes,
 					 dashes);
         }
-        free (aicvec[i].pattern);
+        myfree(aicvec[i].pattern);
     }
     FPRINTF (world->outfile, "\n\n\n");
-    free(temppattern);
+    myfree(temppattern);
 }
 
 ///
@@ -267,16 +272,16 @@ akaike_information (world_fmt * world, long *Gmax)
 	
     aic_score_driver(&aic, mle, mleaic, nr, world, temppattern);
 	
-    qsort ((void *) aic.aicvec, aic.aicnum, sizeof (aic_fmt), aiccmp);
+    qsort ((void *) aic.aicvec, (unsigned long) aic.aicnum, sizeof (aic_fmt), aiccmp);
 	
     aic_print_driver(aic.aicvec, mleaic, aic.aicnum, nr, world);
 	
-    free (aic.aicvec);
+    myfree(aic.aicvec);
     (void) fflush (world->outfile);
-    free (aic.param0);
-    free (temppattern);
+    myfree(aic.param0);
+    myfree(temppattern);
     destroy_nr (nr, world);
-    free(nr);
+    myfree(nr);
 }
 
 ///
@@ -285,8 +290,14 @@ akaike_information (world_fmt * world, long *Gmax)
 /// [this is different to the "branch-bound" algorithm]
 void fast_aic_score (aic_fmt ** aicvec, long *aicnum, nr_fmt * nr,
                 long zero, long which, char *temppattern,
-                MYREAL *param0, char migtype)
+                MYREAL *param0, int migtype)
 {
+  //#ifdef MYREAL == float
+  //MYREAL numbermax = FLT_MAX ;
+  //#else
+  const MYREAL numbermax = DBL_MAX ;
+  //#endif
+
     long m, ii;
     MYREAL likes = 0;
     MYREAL normd = 0;
@@ -294,7 +305,7 @@ void fast_aic_score (aic_fmt ** aicvec, long *aicnum, nr_fmt * nr,
     MYREAL borderaic = (*aicvec)[0].aic + nr->world->options->aicmod * nr->numpop2;
     char savecustm2;
     long remainnum = 0;
-    boolean legal;
+    //    boolean mylegal;
     char *custm2 = nr->world->options->custm2;
     char *scustm2;
     long numparam;
@@ -310,7 +321,7 @@ void fast_aic_score (aic_fmt ** aicvec, long *aicnum, nr_fmt * nr,
     best = (aic_fmt *) mycalloc (nr->partsize, sizeof (aic_fmt));
     for (m = nr->numpop; m < nr->numpop2; m++)
     {
-        best[m].aic = DBL_MAX;
+        best[m].aic = numbermax;
         best[m].numparam = m;
         if (scustm2[m] == migtype)
             continue;
@@ -318,7 +329,7 @@ void fast_aic_score (aic_fmt ** aicvec, long *aicnum, nr_fmt * nr,
         custm2[m] = migtype;
         memcpy (nr->world->param0, param0, sizeof (MYREAL) * nr->numpop2);
         resynchronize_param (nr->world);
-        if ((legal = legal_pattern (nr->world->options->custm2, nr->numpop)))
+        if ((legal_pattern (nr->world->options->custm2, nr->numpop)))
         {
             (void) do_profiles (nr->world, nr, &likes, &normd, PROFILE,
                          nr->world->rep, nr->world->repkind);
@@ -349,8 +360,8 @@ void fast_aic_score (aic_fmt ** aicvec, long *aicnum, nr_fmt * nr,
                             temppattern, param0, migtype);
         }
     }
-    free (best);
-    free(scustm2);
+    myfree(best);
+    myfree(scustm2);
 }
 
 /// add model and AIC score to the list of models for the final printing
@@ -441,12 +452,12 @@ print_progress_aic(boolean add
                      probcorr);
     }
     (void) fflush (stdout);
-    free(temppattern);
+    myfree(temppattern);
 }
 
 /// add the AIC models and scores to the result list
 void
-add_aiclist (char migtype, long numparam, long freeparam, long remainnum, long zero, long m,
+add_aiclist (int migtype, long numparam, long freeparam, long remainnum, long zero, long m,
              MYREAL *param0, char *temppattern, char *custm2,
              long *aicnum, aic_fmt ** aicvec, nr_fmt * nr)
 {
@@ -474,14 +485,14 @@ add_aiclist (char migtype, long numparam, long freeparam, long remainnum, long z
 void
 aic_score (aic_fmt ** aicvec, long *aicnum, nr_fmt * nr,
            long zero, long which, char *temppattern, MYREAL *param0,
-           char migtype)
+           int migtype)
 {
     long m;
     MYREAL likes = 0;
     MYREAL normd = 0;
     char savecustm2;
     long remainnum = 0;
-    boolean legal;
+    //    boolean mylegal;
     char *custm2 = nr->world->options->custm2;
     long numparam = 0;
     long freeparam;
@@ -507,7 +518,7 @@ aic_score (aic_fmt ** aicvec, long *aicnum, nr_fmt * nr,
         memcpy (nr->world->param0, param0, sizeof (MYREAL) * nr->numpop2);
         resynchronize_param (nr->world);
 		
-        if ((legal = legal_pattern (nr->world->options->custm2, nr->numpop)))
+        if ((legal_pattern (nr->world->options->custm2, nr->numpop)))
         {
             (void) do_profiles (nr->world, nr, &likes, &normd, PROFILE,
                          nr->world->rep, nr->world->repkind);
@@ -529,8 +540,9 @@ aic_score (aic_fmt ** aicvec, long *aicnum, nr_fmt * nr,
     }
 }
 
+/* not used
 /// check how many parameter are set to zero or are set to m (average)
-boolean
+static boolean
 check_numparam (long which, long migtype, worldoption_fmt * options,
                 long *numparam, long *remainnum)
 {
@@ -550,6 +562,7 @@ check_numparam (long which, long migtype, worldoption_fmt * options,
     }
     return rc;
 }
+end not used */
 
 ///
 /// check whether the migration pattern in matrix is legal or not
@@ -559,6 +572,11 @@ check_numparam (long which, long migtype, worldoption_fmt * options,
 boolean
 legal_pattern (char *matrix, long numpop)
 {
+  //#ifdef MYREAL == float
+  //const MYREAL eps = FLT_EPSILON ;
+  //#else
+  const MYREAL eps = DBL_EPSILON ;
+  //#endif
     long from=0, to=0, i;
     MYREAL summ = -1;
     long oldto;
@@ -573,14 +591,14 @@ legal_pattern (char *matrix, long numpop)
         m2mm (i, numpop, &from, &to);
         if (oldto != to)
         {
-            if (summ < DBL_EPSILON)
+            if (summ < eps)
                 return FALSE;
             oldto = to;
             summ = 0;
         }
         summ += (MYREAL) (matrix[i] != '0') + (MYREAL) (matrix[mm2m (to, from, numpop)] != '0');
     }
-    if (summ < DBL_EPSILON)
+    if (summ < eps)
         return FALSE;
     return TRUE;
 }
@@ -590,7 +608,7 @@ long find_paramnum(world_fmt *world, char *connect)
     long cc;
     long tm;
     long mm;
-    long mmm;
+    //    long mmm;
     long ms;
     long mss;
     long total;
@@ -639,16 +657,16 @@ long find_paramnum(world_fmt *world, char *connect)
         cc = scan_connect (temp, 0, world->numpop2, 'c');
         tm = scan_connect (temp, 0, world->numpop, 'm');
         mm = scan_connect (temp, world->numpop, world->numpop2, 'm');
-        mmm = scan_connect (temp, world->numpop, world->numpop2, 'M');
+	//        mmm = scan_connect (temp, world->numpop, world->numpop2, 'M');
         ms = scan_connect (temp, world->numpop, world->numpop2, 's')/2;
         mss = scan_connect (temp, world->numpop, world->numpop2, 'S')/2;
         total -= cc + (tm > 0 ? tm-1 : 0);
         total -= (mm > 0 ? mm-1 : 0 );
         total -= ms;
         total -= mss;
-        free(temp);
-        free(temp1);
-        //free(temp2);
+        myfree(temp);
+        myfree(temp1);
+        //myfree(temp2);
 	}
     return total;
 }
