@@ -10,7 +10,7 @@ using a Metropolis-Hastings Monte Carlo algorithm
  
  
  Peter Beerli 1997, Seattle
- beerli@csit.fsu.edu
+ beerli@scs.fsu.edu
  
  Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
 Copyright 2003-2004 Peter Beerli, Tallahassee FL
@@ -19,7 +19,7 @@ Copyright 2003-2004 Peter Beerli, Tallahassee FL
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
- $Id: broyden.c 205 2005-12-26 19:25:22Z beerli $
+ $Id: broyden.c 753 2007-06-15 18:17:57Z beerli $
 -------------------------------------------------------*/
 /*! \file broyden.c
 Basic routines for maximum likelihood parameter estimation.
@@ -67,7 +67,7 @@ MYREAL probG2 (MYREAL *param, MYREAL *lparam, MYREAL *sm, MYREAL *kt,
                long numpop);
 void probG3 (MYREAL *apg, MYREAL *apg0r, timearchive_fmt * tyme, long numpop,
              MYREAL *apgmax, MYREAL *param, MYREAL *lparam, MYREAL *sm);
-MYINLINE  MYREAL probG4 (MYREAL *fullparam, MYREAL *data, long numpop, long numpop2);
+MYINLINE  MYREAL probG4 (MYREAL *fullparam, MYREAL *data, int stop);
 MYREAL sum_mig (MYREAL *param, long msta, long msto);
 void calc_cov (MYREAL **dd, MYREAL *d, MYREAL *param, long n);
 boolean is_singular (MYREAL **dd, long n);
@@ -166,7 +166,7 @@ calc_line_new (helper_fmt * helper, MYREAL a, MYREAL b, MYREAL c,
     {
         mid = (*fpsi) (xmid, helper);
         quadratic_constants (&xguess, low, mid, high, xlow, xmid, xhigh);
-        if (MYISNAN (xguess))
+        if (MYISNAN ((float) xguess))
             return 1.;
         if (xguess < xmid)
         {
@@ -216,7 +216,7 @@ calc_line (helper_fmt * helper, MYREAL a, MYREAL b, MYREAL c,
     /* a < b < c AND psia > psib < psic */
     MYREAL d, psib, psic;
     d = c;
-    if (fabs (c - b) > fabs (b - a))
+    if ((fabs (c - b)) > (fabs (b - a)))
     {
         c = b + QQ * (c - b);
     }
@@ -227,7 +227,7 @@ calc_line (helper_fmt * helper, MYREAL a, MYREAL b, MYREAL c,
     }
     psib = (*fpsi) (b, helper);
     psic = (*fpsi) (c, helper);
-    while (fabs (d - a) > EPSILON * (fabs (b) + fabs (c)))
+    while ((fabs (d - a)) > (EPSILON * (fabs (b) + fabs (c))))
     {
         if (psic < psib)
         {
@@ -284,14 +284,14 @@ calc_hessian (MYREAL **hess, long n, MYREAL *delta, MYREAL *gama)
         dd[i] = dd[0] + n * i;
         dtg += delta[i] * gama[i];
     }
-    if (dtg != 0.0)
+    if (dtg > 0.0 || dtg < 0.0)
         dtg = 1. / dtg;
     else
     {
         reset_hess (hess, n);
-        free (temp);
-        free (dd[0]);
-        free (dd);
+        myfree(temp);
+        myfree(dd[0]);
+        myfree(dd);
         return;
     }
     for (i = 0; i < n; i++)
@@ -332,9 +332,9 @@ calc_hessian (MYREAL **hess, long n, MYREAL *delta, MYREAL *gama)
             hess[i][j] += delta[i] * delta[j] * t - dd[i][j];
         }
     }
-    free (temp);
-    free (dd[0]);
-    free (dd);
+    myfree(temp);
+    myfree(dd[0]);
+    myfree(dd);
 }
 
 MYREAL
@@ -427,7 +427,7 @@ create_nr (nr_fmt * nr, world_fmt * world, long G,
     for (i = 0; i < nr->partsize; ++i)
     {
         if (nr->param[i] > 0.0)
-            nr->lparam[i] = log (nr->param[i]);
+            nr->lparam[i] = LOG (nr->param[i]);
         else
             nr->lparam[i] = -DBL_MAX;
     }
@@ -480,7 +480,7 @@ reuse_nr (nr_fmt * nr, world_fmt * world, long G, long profilenum,
     for (i = 0; i < nr->partsize; ++i)
     {
         if (nr->param[i] > 0.0)
-            nr->lparam[i] = log (nr->param[i]);
+            nr->lparam[i] = LOG (nr->param[i]);
         else
             nr->lparam[i] = -DBL_MAX;
     }
@@ -490,13 +490,13 @@ reuse_nr (nr_fmt * nr, world_fmt * world, long G, long profilenum,
     /*
      for (j = world->repstop - 1; j >= 0; j--)
      {
-      free (nr->apg0[j][0]);
-      free (nr->apg[j][0]);
-      free (nr->apg0[j]);
-      free (nr->apg[j]);
+      myfree(nr->apg0[j][0]);
+      myfree(nr->apg[j][0]);
+      myfree(nr->apg0[j]);
+      myfree(nr->apg[j]);
      }
-     free (nr->apg0);
-     free (nr->apg);
+     myfree(nr->apg0);
+     myfree(nr->apg);
      alloc_apg (&nr->apg0, world->repstop, world->loci, G);
      alloc_apg (&nr->apg, world->repstop, world->loci, G);*/
 }
@@ -519,18 +519,20 @@ void
 destroy_nr (nr_fmt * nr, world_fmt * world)
 {
     // the large holding vectors are freed and all related points will go away, too.
-    free (nr->parts);
-    free (nr->datalike);
-    free (nr->apg_max); 
-    free (nr->profiles);
+    myfree(nr->parts);
+    myfree(nr->datalike);
+    myfree(nr->apg_max); 
+    myfree(nr->profiles);
 
     if (nr->categs != 0)
     {
-        free (nr->rate); // probcat is also freed, was part of rate-array
+        myfree(nr->rate); // probcat is also freed, was part of rate-array
     }
-    free (nr);
+    myfree(nr);
 }
 
+///
+/// calculate the likelihood of the parameter estimates
 #ifndef ALTIVEC
 MYREAL
 calc_locus_like (nr_fmt * nr, MYREAL *param, MYREAL *lparam, long locus)
@@ -539,6 +541,7 @@ calc_locus_like (nr_fmt * nr, MYREAL *param, MYREAL *lparam, long locus)
     const long numpop = nr->world->numpop;
     const long numpop2 = nr->world->numpop2;
     int msta, msto;
+    int stop = (int) (numpop2 + numpop + numpop);
     MYREAL gsum = 0;
     MYREAL ***apg0;
     MYREAL apgmax = -DBL_MAX;
@@ -553,6 +556,7 @@ calc_locus_like (nr_fmt * nr, MYREAL *param, MYREAL *lparam, long locus)
     MYREAL *localparam;
     MYREAL *sm;
     MYREAL mu_rate = nr->world->options->mu_rates[locus];
+    MYREAL inv_mu_rate = 1./ mu_rate;
     MYREAL lmu_rate = nr->world->options->lmu_rates[locus];
     MYREAL normaldev;
 #ifdef LONGSUM
@@ -562,9 +566,12 @@ calc_locus_like (nr_fmt * nr, MYREAL *param, MYREAL *lparam, long locus)
     MYREAL *rtimes;
 #endif /*LONGSUM*/
 #ifndef LONGSUM
-
-    locallparam = (MYREAL *) mycalloc ((numpop2 + numpop + numpop),
-                                     sizeof (MYREAL));
+    // experiment with unrolling the loop in probG4 by 3, the 3 added here guarantees
+    // that enough elements are present so that the loop can gracefully finish
+#ifdef FAST_TRY
+    stop += 3 - (stop % 3);
+#endif
+    locallparam = (MYREAL *) mycalloc (stop, sizeof (MYREAL));
 #else /*LONGSUM*/
 
     locallparam = (MYREAL *) mycalloc ((numpop2 + numpop + numpop + 9 * numpop),
@@ -595,7 +602,7 @@ calc_locus_like (nr_fmt * nr, MYREAL *param, MYREAL *lparam, long locus)
         msto = nr->mend[r];
         for (j = msta; j < msto; j++)
         {
-            sm[r] -= geo[j] * param[j] / mu_rate; //minus, so that we can loop over all in probG4
+            sm[r] -= geo[j] * param[j] * inv_mu_rate; //minus, so that we can loop over all in probG4
             locallparam[j] += lgeo[j] - lmu_rate;
         }
     }
@@ -610,11 +617,13 @@ calc_locus_like (nr_fmt * nr, MYREAL *param, MYREAL *lparam, long locus)
         tl = &(tyme.tl[0]);
         copies = tl->copies - 1;
         gsum += copies;
-
+#ifdef INTEGRATEDLIKE
+	tl->lcopies = ln_copies(copies);
+#endif
         if (copies > 0)
         {
 #ifndef LONGSUM
-            apg[0] = tl->lcopies + probG4 (locallparam, tl->data, numpop, numpop2) - apg0r[0] + normaldev;
+            apg[0] = tl->lcopies + probG4 (locallparam, tl->data, stop) - apg0r[0] + normaldev;
 #else /*LONGSUM*/
 
             apg[0] = tl->lcopies + probG4_longsum(locallparam,tl->longsum,tl->longsumlen, numpop,numpop2)
@@ -631,9 +640,13 @@ calc_locus_like (nr_fmt * nr, MYREAL *param, MYREAL *lparam, long locus)
         {
             tl = &(tyme.tl[g]);
             gsum += tl->copies;
+#ifdef INTEGRATEDLIKE
+	tl->lcopies = ln_copies(tl->copies);
+#endif
+
 #ifndef LONGSUM
 
-            apg[g] = tl->lcopies + probG4 (locallparam, tl->data, numpop, numpop2) - apg0r[g]
+            apg[g] = tl->lcopies + probG4 (locallparam, tl->data, stop) - apg0r[g]
                      +normaldev;
 #else /*LONGSUM*/
 
@@ -664,8 +677,8 @@ calc_locus_like (nr_fmt * nr, MYREAL *param, MYREAL *lparam, long locus)
         }
     }    // replicates
     nr->apg_max[locus] = apgmax;
-    nr->llike = apgmax + log (nr->PGC[locus]) - log (gsum);
-    free (locallparam);
+    nr->llike = apgmax + LOG (nr->PGC[locus]) - LOG (gsum);
+    myfree(locallparam);
     return nr->llike;
 }
 #else /*ALTIVEC*/
@@ -691,6 +704,7 @@ calc_locus_like (nr_fmt * nr, MYREAL *param, MYREAL *lparam, long locus)
     MYREAL *localparam;
     MYREAL *sm;
     MYREAL mu_rate = nr->world->options->mu_rates[locus];
+    MYREAL inv_mu_rate = 1./ mu_rate;
     MYREAL lmu_rate = nr->world->options->lmu_rates[locus];
     MYREAL normaldev;
     long size4 = numpop2 + numpop + numpop;
@@ -719,7 +733,7 @@ calc_locus_like (nr_fmt * nr, MYREAL *param, MYREAL *lparam, long locus)
         msto = nr->mend[r];
         for (j = msta; j < msto; j++)
         {
-            sm[r] -= geo[j] * param[j] / mu_rate; //minus, so that we can loop over all in probG4
+            sm[r] -= geo[j] * param[j] * inv_mu_rate; //minus, so that we can loop over all in probG4
             locallparam[j] += lgeo[j] - lmu_rate;
         }
     }
@@ -774,10 +788,10 @@ calc_locus_like (nr_fmt * nr, MYREAL *param, MYREAL *lparam, long locus)
         }
     }    // replicates
     nr->apg_max[locus] = apgmax;
-    nr->llike = apgmax + log (nr->PGC[locus]) - log (gsum);
-    free (locallparam);
-    free(vlocal);
-    free(vdata);
+    nr->llike = apgmax + LOG (nr->PGC[locus]) - LOG (gsum);
+    myfree(locallparam);
+    myfree(vlocal);
+    myfree(vdata);
     return nr->llike;
 }
 #endif /*ALTIVEC*/
@@ -835,7 +849,7 @@ ln_copies (long n)
     case 20:
         return 2.9957322735539909934;
     default:
-        return log ((MYREAL) n);
+        return LOG ((MYREAL) n);
     }
 }
 
@@ -855,7 +869,7 @@ force_sametheta (MYREAL *param, worldoption_fmt * wopt, long numpop)
         }
     }
     summ /= n;
-    logsumm = log (summ);
+    logsumm = LOG (summ);
     for (i = 0; i < numpop; i++)
     {
         if (custm2[i] == 'm')
@@ -882,7 +896,7 @@ force_samemigration (MYREAL *param, worldoption_fmt * wopt, long numpop)
         }
     }
     summ /= n;
-    logsumm = log (summ);
+    logsumm = LOG (summ);
     for (i = numpop; i < numpop2; i++)
     {
         if (custm2[i] == 'm')
@@ -943,15 +957,15 @@ param_all_adjust (MYREAL *xv, nr_fmt *nr)
 	for (i = 0; i < constn; i++)
     {
         if (consts[i] < numpop)
-            param[consts[i]] = log (wopt->thetag[consts[i]]);
+            param[consts[i]] = LOG (wopt->thetag[consts[i]]);
         if (consts[i] >= numpop)
         {
             if (consts[i] >= numpop * numpop)
-                param[consts[i]] = log (wopt->alphavalue);
+                param[consts[i]] = LOG (wopt->alphavalue);
             else
             {
                 m2mm (consts[i], numpop, &from, &to);
-                param[consts[i]] = log (wopt->mg[to*(numpop-1)+from]);
+                param[consts[i]] = LOG (wopt->mg[to*(numpop-1)+from]);
             }
         }
     }
@@ -964,14 +978,14 @@ param_all_adjust (MYREAL *xv, nr_fmt *nr)
             for(z=0;z<nr->profilenum;z++)
             {
                 which_profile = nr->profiles[z];
-                if(which_profile == syms[i][0])
+                if(which_profile == (long) syms[i][0])
                 {
                     param[syms[i][1]] = param[syms[i][0]];
                     break;
                 }
                 else
                 {
-                    if(which_profile == syms[i][1])
+                    if(which_profile == (long) syms[i][1])
                     {
                         param[syms[i][0]] = param[syms[i][1]];
                         break;
@@ -979,7 +993,7 @@ param_all_adjust (MYREAL *xv, nr_fmt *nr)
                     else
                     {
                         mm = (EXP (param[syms[i][0]]) + EXP (param[syms[i][1]]))  * HALF;
-                        param[syms[i][0]] = param[syms[i][1]] = log (mm);
+                        param[syms[i][0]] = param[syms[i][1]] = LOG (mm);
                         break;
                     }
                 }
@@ -988,7 +1002,7 @@ param_all_adjust (MYREAL *xv, nr_fmt *nr)
         else
         {
             mm = (EXP (param[syms[i][0]]) + EXP (param[syms[i][1]])) * HALF;
-            param[syms[i][0]] = param[syms[i][1]] = log (mm);
+            param[syms[i][0]] = param[syms[i][1]] = LOG (mm);
         }
     }
     for (i = 0; i < sym2n; i++)
@@ -998,7 +1012,7 @@ param_all_adjust (MYREAL *xv, nr_fmt *nr)
             for(z=0;z<nr->profilenum;z++)
             {
                 which_profile = nr->profiles[z];
-                if(which_profile == sym2s[i][0])
+                if(which_profile == (long) sym2s[i][0])
                 {
                     //printf(".1.\n");
                     mm = param[sym2s[i][0]] + param[sym2s[i][2]];
@@ -1008,7 +1022,7 @@ param_all_adjust (MYREAL *xv, nr_fmt *nr)
                 }
                 else
                 {
-                    if(which_profile == sym2s[i][1])
+                    if(which_profile == (long) sym2s[i][1])
                     {
                         //printf(".2.\n");
                         mm = param[sym2s[i][1]] + param[sym2s[i][3]];
@@ -1019,7 +1033,7 @@ param_all_adjust (MYREAL *xv, nr_fmt *nr)
                     {
                         mm = (EXP (param[sym2s[i][0]] + param[sym2s[i][2]]) +
                               EXP (param[sym2s[i][1]] + param[sym2s[i][3]])) * HALF;
-                        param[sym2s[i][0]] = (lmm = log (mm)) - param[sym2s[i][2]];
+                        param[sym2s[i][0]] = (lmm = LOG (mm)) - param[sym2s[i][2]];
                         param[sym2s[i][1]] = lmm - param[sym2s[i][3]];
                         break;
                     }
@@ -1030,7 +1044,7 @@ param_all_adjust (MYREAL *xv, nr_fmt *nr)
         {
             mm = (EXP (param[sym2s[i][0]] + param[sym2s[i][2]]) +
                   EXP (param[sym2s[i][1]] + param[sym2s[i][3]])) * HALF;
-            param[sym2s[i][0]] = (lmm = log (mm)) - param[sym2s[i][2]];
+            param[sym2s[i][0]] = (lmm = LOG (mm)) - param[sym2s[i][2]];
             param[sym2s[i][1]] = lmm - param[sym2s[i][3]];
         }
     }
@@ -1123,15 +1137,15 @@ check_matrix_arbitrary (MYREAL *param, worldoption_fmt * wopt, long numpop, long
     for (i = 0; i < constn; i++)
     {
         if (consts[i] < numpop)
-            param[consts[i]] = log (wopt->thetag[consts[i]]);
+            param[consts[i]] = LOG (wopt->thetag[consts[i]]);
         if (consts[i] >= numpop)
         {
             if (consts[i] >= numpop * numpop)
-                param[consts[i]] = log (wopt->alphavalue);
+                param[consts[i]] = LOG (wopt->alphavalue);
             else
             {
                 m2mm (consts[i], numpop, &from, &to);
-                param[consts[i]] = log (wopt->mg[to*(numpop-1)+from]);
+                param[consts[i]] = LOG (wopt->mg[to*(numpop-1)+from]);
             }
         }
     }
@@ -1147,7 +1161,7 @@ check_matrix_arbitrary (MYREAL *param, worldoption_fmt * wopt, long numpop, long
             else
             {
                 mm = (EXP (param[syms[i][0]]) + EXP (param[syms[i][1]])) * HALF;
-                param[syms[i][0]] = param[syms[i][1]] = log (mm);
+                param[syms[i][0]] = param[syms[i][1]] = LOG (mm);
             }
         }
     }
@@ -1165,7 +1179,7 @@ check_matrix_arbitrary (MYREAL *param, worldoption_fmt * wopt, long numpop, long
                       EXP (param[sym2s[i][1]] + param[sym2s[i][3]])) * HALF;
             }
         }
-        param[sym2s[i][0]] = (lmm = log (mm)) - param[sym2s[i][2]];
+        param[sym2s[i][0]] = (lmm = LOG (mm)) - param[sym2s[i][2]];
         param[sym2s[i][1]] = lmm - param[sym2s[i][3]];
     }
 }
@@ -1254,7 +1268,7 @@ gradient (MYREAL *d, nr_fmt * nr, long locus)
             }
         }
     }
-    free(normalgrad);
+    myfree(normalgrad);
 }
 
 #ifdef LONGSUM
@@ -1410,7 +1424,7 @@ gradient_longsum (MYREAL *d, nr_fmt * nr, long locus)
             }
         }
     }
-    free(treeparts);
+    myfree(treeparts);
 }
 #endif /*LONGSUM*/
 /* calculates the derivatives for the different
@@ -1687,6 +1701,7 @@ probG (MYREAL *param, MYREAL *lparam, tarchive_fmt * tl, nr_fmt * nr,
     MYREAL *geo = nr->world->data->geo;
     MYREAL *lgeo = nr->world->data->lgeo;
     MYREAL mu_rate = nr->world->options->mu_rates[locus];
+    MYREAL inv_mu_rate = 1./ mu_rate;
     MYREAL lmu_rate = nr->world->options->lmu_rates[locus];
     for (i = 0; i < numpop; i++)
     {
@@ -1701,18 +1716,24 @@ probG (MYREAL *param, MYREAL *lparam, tarchive_fmt * tl, nr_fmt * nr,
             if (param[j] > 0.0)
             {
                 result += tl->mindex[j] * (lgeo[j] + lparam[j] - lmu_rate);
-                sm += geo[j] * param[j] / mu_rate;
+                sm += geo[j] * param[j] * inv_mu_rate;
             }
         }
-        result += -tl->km[i] * sm - tl->kt[i] / (param[i] * mu_rate);
+        result += -tl->km[i] * sm - tl->kt[i] * (1./param[i]) * inv_mu_rate;
     }
     return result;
 }
 
 MYINLINE  MYREAL
-probG4 (MYREAL *fullparam, MYREAL *data, long numpop, long numpop2)
+probG4 (MYREAL *fullparam, MYREAL *data, int stop)
 {
     int i;
+    //    int i1;
+    //    int i2;
+    //int stop = (int) (numpop2 + numpop + numpop);
+    //MYREAL r1;
+    //MYREAL r2;
+    //MYREAL r3;
     MYREAL result = 0.;
     // fullparam is a linearized
     //     log(param) [size: numpop * numpop] [Log(2/theta), Log(M)] for example 3pop:  3 log(2/theta), 6 log(M)
@@ -1728,10 +1749,24 @@ probG4 (MYREAL *fullparam, MYREAL *data, long numpop, long numpop2)
     //
     //  calculation: p * log(2/theta) + mindex * log(M)
     // - kt * 1/theta - km * sm
-    for (i = 0; i < numpop2 + 2 * numpop; i++)
+    for (i = 0; i < stop; i++)
     {
         result += fullparam[i] * data[i];
     }
+    // stop is at least 3:
+    // population in dataset  stop
+    //            1            3
+    //            2            8
+    //            3           15
+    //            4           24
+    //for (i = 0, i1= 1, i2 = 2; i2 < stop ; i++, i1++, i2++)
+    //{
+    //  r1 = fullparam[i] * data[i];
+    //  r2 = fullparam[i1] * data[i1];
+    //  r3 = fullparam[i2] * data[i2];
+    //  result += r1 + r2 + r3;
+    //}
+
     return result;
 }
 
@@ -1780,7 +1815,7 @@ probG_longsum (MYREAL *param, MYREAL *lparam, tarchive_fmt * tl, nr_fmt * nr,
             result -= tint->interval * tint->lineages[tint->to] * sm[pop];
         }
     }
-    free(sm);
+    myfree(sm);
     return result;
 }
 
@@ -1874,8 +1909,8 @@ probG4_longsum (MYREAL *fullparam, longsum_fmt *data, long longsumlen, long nump
         }
     }
     // FPRINTF(stdout,"\n");
-    //free(rates);
-    //  free(rtimes);
+    //myfree(rates);
+    //  myfree(rtimes);
     return result;
 }
 #endif /*LONGSUM*/
@@ -2008,7 +2043,7 @@ is_singular (MYREAL **dd, long n)
         {
             temp += dd[i][j];
         }
-        if (temp == 0.0)
+        if (!(temp > 0.0 || temp < 0.0))
         {
             singular = TRUE;
             break;
@@ -2021,7 +2056,7 @@ is_singular (MYREAL **dd, long n)
         {
             temp += dd[i][j];
         }
-        if (temp == 0.0)
+        if (!(temp < 0.0 || temp > 0.0))
         {
             singular = TRUE;
             break;
@@ -2135,7 +2170,7 @@ log_param0 (MYREAL *param, MYREAL *lparam, long nn)
     for (i = 0; i < nn; i++)
     {
         if (param[i] > 0)
-            lparam[i] = log (param[i]);
+            lparam[i] = LOG (param[i]);
         else
             lparam[i] = -30; //-DBL_MAX;
     }
@@ -2176,8 +2211,11 @@ create_apg0 (MYREAL *apg0, nr_fmt * nr, timearchive_fmt * tyme, long locus)
 #ifdef LONGSUM
             apg0[g] =probG_longsum (tyme->param0, tyme->lparam0, &tyme->tl[g], nr, locus);
 #else /*LONGSUM*/
-
+	    //#ifdef INTEGRATEDLIKE
+	    //apg0[g] = 0;
+	    //#else
             apg0[g] =  probG (tyme->param0, tyme->lparam0, &tyme->tl[g], nr, locus);
+	    //#endif /*INTEGRATEDLIKE*/
 #endif /*LONGSUM*/
 
         }
@@ -2411,7 +2449,7 @@ for (r = nr->repstart; r < nr->repstop; r++)
    }
 }    // replicates
 nr->apg_max[locus] = apgmax;
-nr->llike = apgmax + log (nr->PGC[locus]) - log (gsum);
-free (locallparam);
+nr->llike = apgmax + LOG (nr->PGC[locus]) - LOG (gsum);
+myfree(locallparam);
 return nr->llike;
 }*/
