@@ -14,7 +14,7 @@
  
  
  Peter Beerli 1996-1998, Seattle
- beerli@csit.fsu.edu
+ beerli@scs.fsu.edu
  
  
     Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
@@ -25,7 +25,7 @@ Copyright 2003-2004 Peter Beerli, Tallahassee FL
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: combroyden2.c 196 2005-12-18 15:43:16Z beerli $
+$Id: combroyden2.c 753 2007-06-15 18:17:57Z beerli $
  
 -------------------------------------------------------*/
 
@@ -213,9 +213,14 @@ estimateParameter (long rep, long G, world_fmt * world, option_fmt * options,
     set_replicates (world, world->repkind, world->rep, &nr->repstart,
                     &nr->repstop);
     prepare_broyden (kind, world, &multilocus);
-    SETUPPARAM0 (world, nr, world->repkind, nr->repstart, nr->repstop,
+#ifndef INTEGRATEDLIKE
+    //    if(world->options->integrated_like)
+      SETUPPARAM0 (world, nr, world->repkind, nr->repstart, nr->repstop,
                  world->loci, kind, multilocus);
-
+#else
+      SETUPPARAM0 (world, nr, world->repkind, nr->repstart, nr->repstop,
+                 world->loci, kind, multilocus);
+#endif
     doublevec1d (&param, nr->partsize + 1);
 
 #ifdef MPI
@@ -228,12 +233,10 @@ estimateParameter (long rep, long G, world_fmt * world, option_fmt * options,
         calc_meanparam (world, world->numpop2, repstart, repstop);
         memcpy (param, world->param0, sizeof (MYREAL) * nr->numpop2);
     }
-//    else
   //    {
     //    expected_param(param,world);
      // }
 #endif
-
     memcpy (param, world->param0, sizeof (MYREAL) * nr->numpop2);
 
 #ifdef LONGSUM
@@ -251,8 +254,9 @@ estimateParameter (long rep, long G, world_fmt * world, option_fmt * options,
             trials = do_profiles (world, nr, &likes, &normd, kind, rep, repkind);
     }
     else
+      {
         trials = maximize (&param, world, nr, dd, kind, repkind);
-
+      }
     if (kind == SINGLELOCUS && world->options->progress)
     {
         print_menu_chain (type, chain, world->atl[rep][world->locus].T,
@@ -278,7 +282,7 @@ estimateParameter (long rep, long G, world_fmt * world, option_fmt * options,
 
     world->trials = trials;
     world->normd = nr->normd;
-    free (param);
+    myfree(param);
     destroy_nr (nr, world);
     return trials;
 }
@@ -385,9 +389,9 @@ phi (MYREAL t, helper_fmt *b)
   set_gamma_param (nr->param, nr->oparam, nr->lparam, nr->olparam, t, nr);
   ll = calc_locus_like (nr, nr->param, nr->lparam, locus);
 #ifdef LAGUERRE
-  ll = -t / beta + (alpha - 1.) * log (t) + ll  + weight;
+  ll = -t / beta + (alpha - 1.) * LOG (t) + ll  + weight;
 #else
-  ll = EXP (-t / beta + (alpha - 1.) * log (t) + ll  + weight);
+  ll = EXP (-t / beta + (alpha - 1.) * LOG (t) + ll  + weight);
 #endif
   return ll;
 }
@@ -539,7 +543,7 @@ set_gamma_param (MYREAL *param, MYREAL *oparam,
     long i;
     MYREAL logtheta;
     param[0] = theta;
-    lparam[0] = logtheta = log (theta);
+    lparam[0] = logtheta = LOG (theta);
     for (i = 1; i < nr->numpop; i++)
     {
         lparam[i] = olparam[i] + logtheta - olparam[0];
@@ -688,7 +692,7 @@ calc_loci_param (nr_fmt * nr, MYREAL *lparam, MYREAL *olparam, MYREAL *dv,
         {
             lparam[nr->numpop2] = 9.903487553;
         }
-        initgammacat (nr->categs, EXP (lparam[nr->numpop2]),1./* EXP (lparam[0])*/,
+        initgammacat (nr->categs, EXP (lparam[nr->numpop2]),1.0 /*EXP (lparam[0])*/,
                       nr->rate, nr->probcat);
         //}
     }
@@ -741,6 +745,12 @@ line_search (MYREAL *dv, MYREAL *gxv,
                              MYREAL *lparam), helper_fmt * thishelper,
              MYREAL oldL, MYREAL *tempparam, MYREAL *templparam)
 {
+  //#ifdef MYREAL == float
+  //const MYREAL eps = FLT_EPSILON ;
+  //#else
+  const MYREAL eps = DBL_EPSILON ;
+  //#endif
+
     long trials = 0;
     MYREAL locallamda = STARTLAMBDA, ql;
     MYREAL a, b, c;
@@ -768,7 +778,7 @@ line_search (MYREAL *dv, MYREAL *gxv,
     {
         //            printf ("*%3li> %f %f %f / %f %f %f\n", trials, a, b, c, la, lb, lc);
       denom = (2. * (b * la - c * la - a * lb + c * lb +  a * lc - b * lc));
-      if((fabs(denom)-DBL_EPSILON)>DBL_EPSILON)
+      if((fabs(denom)-eps)>eps)
 	{
 	  nom = (c * c * (lb - la) + b * b * (la - lc) +
 		 a * a * (lc - lb));
@@ -1002,8 +1012,8 @@ do_profiles (world_fmt * world, nr_fmt * nr, MYREAL *likes,
        world->options->gamma : 0);
       doublevec2d(&hess,z,z);
     trials = maximize (&world->param0, world, nr, hess, kind, repkind);
-    free(hess[0]);
-    free(hess);
+    myfree(hess[0]);
+    myfree(hess);
     return trials;
 }
 
@@ -1028,11 +1038,11 @@ correct_values (world_fmt * world, nr_fmt * nr)
                     nsum++;
                 }
             }
-            if (nsum > 0 && ssum != 0.0)
+            if (nsum > 0 && ssum > 0.0)
                 world->param0[pop] = ssum / (MYREAL) nsum;
             else
             {
-                if (ssum == 0.0)
+	      if (!(ssum > 0.0))
                     world->param0[pop] = SMALLEST_THETA;
                 else
                     world->param0[pop] = BIGGEST_THETA;
@@ -1163,7 +1173,7 @@ profile_setup (long profilenum, long *profiles, MYREAL *param, MYREAL *value,
     {
         *nnn -= profilenum;
         for (i = 0; i < profilenum; i++)
-            param[profiles[i]] = log (value[i]);
+            param[profiles[i]] = LOG (value[i]);
     }
     if (*nnn == 0)
         *nnn = 1;
@@ -1293,71 +1303,78 @@ long
 maximize (MYREAL **thisparam, world_fmt * world, nr_fmt * nr, MYREAL **hess, long analystype,
           long repkind)
 {
-    boolean stop;
-    long repstart, repstop;
-    //  long Gmax;
-    MYREAL two = 2.;
-    MYREAL newL;
-    MYREAL oldL;
-    MYREAL lamda;
-    long trials;
-    MYREAL normd = 1000000;
-    MYREAL normd20 = DBL_MAX;
-    worldoption_fmt *wopt = world->options;
-    long nnn = nr->partsize; // keeps track of profile or standard
-    long nn;   // keeps track of the full parameter array
-    
-    MYREAL *param;
-    //    MYREAL **hess;
-    MYREAL *delta;
-    MYREAL *gama;
-    MYREAL *dv = NULL;
-    MYREAL *gxv;
-    MYREAL *gxv0;
-    MYREAL *xv0;
-    MYREAL *xv;
-    MYREAL *expxv;
-    MYREAL *temp;
-    MYREAL *tempparam;  //just pointer into temp
-    MYREAL *templparam;  //just pointer into temp
-    helper_fmt helper;
-
-    setdoublevec1d (&param, *thisparam, nnn);
-    //  long reset_z = 0;
-    helper.boolgamma = world->locus == world->loci ? wopt->gamma : FALSE;
-    if (helper.boolgamma)
-        param[nnn - 1] = world->options->alphavalue;
-    //doublevec2d (&hess, nnn, nnn);
-    reset_hess (hess, nnn);
-    doublevec1d (&delta, nnn);
-    doublevec1d (&gama, nnn);
-    doublevec1d (&xv, nnn + 1);
-    doublevec1d (&temp, 2 * (nnn + 1));
-    tempparam = temp;
-    templparam = temp + nnn + 1;
-    helper.xv = xv;
-    //  expxv = xv + nnn;
-    //memcpy(expxv,param,sizeof(MYREAL)*nnn);
-    doublevec1d (&gxv, nnn);
-    doublevec1d (&gxv0, nnn);
-    setdoublevec1d (&expxv, param, nnn);
-    helper.expxv = expxv;
-    set_logparam (xv, expxv, nnn);
-    setdoublevec1d (&xv0, xv, nnn);
-    helper.analystype = analystype;
-    set_replicates (world, world->repkind, world->rep, &repstart, &repstop);
-    nr->repstart = repstart;
-    nr->repstop = repstop;
-    nr->normd = normd;
-    prepare_broyden (analystype, world, &helper.multilocus);
-    nnn = multilocus_gamma_adjust (helper.multilocus, helper.boolgamma,
-                                   world, repstart, repstop);
+  //#ifdef MYREAL == float
+  //const MYREAL eps = FLT_EPSILON ;
+  //const MYREAL numbermax = FLT_MAX ;
+  //#else
+  const MYREAL eps = DBL_EPSILON ;
+  const MYREAL numbermax = DBL_MAX ;
+  //#endif
+  boolean stop;
+  long repstart, repstop;
+  //  long Gmax;
+  MYREAL two = 2.;
+  MYREAL newL;
+  MYREAL oldL;
+  MYREAL lamda;
+  long trials;
+  MYREAL normd = 1000000;
+  MYREAL normd20 = DBL_MAX;
+  worldoption_fmt *wopt = world->options;
+  long nnn = nr->partsize; // keeps track of profile or standard
+  long nn;   // keeps track of the full parameter array
+  
+  MYREAL *param;
+  //    MYREAL **hess;
+  MYREAL *delta;
+  MYREAL *gama;
+  MYREAL *dv = NULL;
+  MYREAL *gxv;
+  MYREAL *gxv0;
+  MYREAL *xv0;
+  MYREAL *xv;
+  MYREAL *expxv;
+  MYREAL *temp;
+  MYREAL *tempparam;  //just pointer into temp
+  MYREAL *templparam;  //just pointer into temp
+  helper_fmt helper;
+  
+  setdoublevec1d (&param, *thisparam, nnn);
+  //  long reset_z = 0;
+  helper.boolgamma = world->locus == world->loci ? wopt->gamma : FALSE;
+  if (helper.boolgamma)
+    param[nnn - 1] = world->options->alphavalue;
+  //doublevec2d (&hess, nnn, nnn);
+  reset_hess (hess, nnn);
+  doublevec1d (&delta, nnn);
+  doublevec1d (&gama, nnn);
+  doublevec1d (&xv, nnn + 1);
+  doublevec1d (&temp, 2 * (nnn + 1));
+  tempparam = temp;
+  templparam = temp + nnn + 1;
+  helper.xv = xv;
+  //  expxv = xv + nnn;
+  //memcpy(expxv,param,sizeof(MYREAL)*nnn);
+  doublevec1d (&gxv, nnn);
+  doublevec1d (&gxv0, nnn);
+  setdoublevec1d (&expxv, param, nnn);
+  helper.expxv = expxv;
+  set_logparam (xv, expxv, nnn);
+  setdoublevec1d (&xv0, xv, nnn);
+  helper.analystype = analystype;
+  set_replicates (world, world->repkind, world->rep, &repstart, &repstop);
+  nr->repstart = repstart;
+  nr->repstop = repstop;
+  nr->normd = normd;
+  prepare_broyden (analystype, world, &helper.multilocus);
+  nnn = multilocus_gamma_adjust (helper.multilocus, helper.boolgamma,
+				 world, repstart, repstop);
 #ifdef LONGSUM
-
-    nnn=nr->partsize;
+  
+  nnn=nr->partsize;
 #endif
-
-    nn = nnn;
+  
+  nn = nnn;
     //if(helper.boolgamma)
     //{
     // save_a = nr->world->options->custm[nr->world->numpop2];
@@ -1398,8 +1415,8 @@ maximize (MYREAL **thisparam, world_fmt * world, nr_fmt * nr, MYREAL **hess, lon
      //       lamda = golden_section (psilo, &helper, tempparam, templparam);
                 //line_search (dv, gxv, psilo, &helper, oldL, tempparam, templparam);
         helper.lamda = lamda = 1.;
-        while ((newL <= oldL || MYISNAN (newL) || newL <= -DBL_MAX) &&
-                fabs (lamda) > 10. * DBL_EPSILON)
+        while ((newL < oldL || MYISNAN ((float) newL) || newL <= -numbermax) &&
+                fabs (lamda) > 10. * eps)
         {
             calc_loci_param (nr, xv, xv0, dv, lamda, nn);
             set_expparam (expxv, xv, nn);
@@ -1419,9 +1436,17 @@ maximize (MYREAL **thisparam, world_fmt * world, nr_fmt * nr, MYREAL **hess, lon
         }
 #endif
         if (world->options->verbose && newL > oldL)
-            report_broyden (newL, normd, trials, helper.boolgamma,
-                            expxv[0], expxv[nr->numpop2],
-                            world->options->logfile);
+            {
+	      if(helper.boolgamma)
+		report_broyden (newL, normd, trials, helper.boolgamma,
+				expxv[0], expxv[nr->numpop2],
+				world->options->logfile);
+	      else
+		report_broyden (newL, normd, trials, helper.boolgamma,
+				expxv[0], -1.0,
+				world->options->logfile);
+
+	    }
         stop = guard_broyden (newL, oldL, trials, normd, &normd20);
         if (normd < EPSILON || stop)
             break;   // stopping criteria
@@ -1435,7 +1460,7 @@ maximize (MYREAL **thisparam, world_fmt * world, nr_fmt * nr, MYREAL **hess, lon
         //  lamda=0;
         //  reset_z = 0;
         //}
-        if (fabs (lamda) <= DBL_EPSILON * 10.)
+        if (fabs (lamda) <= eps * 10.)
         {
             reset_hess (hess, nnn);
             two = -two;
@@ -1463,18 +1488,18 @@ maximize (MYREAL **thisparam, world_fmt * world, nr_fmt * nr, MYREAL **hess, lon
     finish_broyden (newL, normd, trials, world, nr, expxv, nn,
                     analystype, repkind);
     memcpy ((*thisparam), expxv, sizeof (MYREAL) * nn);
-    free(param);
-    //free (hess[0]);
-    //free (hess);
-    free (delta);
-    free (gama);
-    free (dv);
-    free (gxv);
-    free (gxv0);
-    free (xv0);
-    free (xv);
-    free (expxv);
-    free (temp);
+    myfree(param);
+    //myfree(hess[0]);
+    //myfree(hess);
+    myfree(delta);
+    myfree(gama);
+    myfree(dv);
+    myfree(gxv);
+    myfree(gxv0);
+    myfree(xv0);
+    myfree(xv);
+    myfree(expxv);
+    myfree(temp);
     fflush (stderr);
     return trials;
 }
@@ -1492,6 +1517,15 @@ finish_broyden (MYREAL newL, MYREAL normd, long trials,
 
     //if (world->options->verbose)
     //    FPRINTF (stdout, "\n");
+
+    //#ifdef INTEGRATEDLIKE
+    //if(analystype==MULTILOCUS && world->loci==1)
+    //  {
+    //	analystype=SINGLELOCUS;
+    //	world->locus=0;
+    //   }
+    //#endif
+
     switch (analystype)
     {
     case MULTILOCUS:
@@ -1508,6 +1542,7 @@ finish_broyden (MYREAL newL, MYREAL normd, long trials,
         world->param_like = newL;
         if (repkind != SINGLECHAIN)
         {
+	  //	  printf("%i> single locus combine replicates into rep=%li\n",myID,world->repstop);
             tyme[world->repstop][world->locus].param_like = newL;
             tyme[world->repstop][world->locus].normd = normd;
             tyme[world->repstop][world->locus].trials = trials;
@@ -1516,20 +1551,22 @@ finish_broyden (MYREAL newL, MYREAL normd, long trials,
         }
         else
         {
+	  //printf("%i> more loci put replicate into rep=%li\n",myID,world->rep);
             tyme[world->rep][world->locus].param_like = newL;
             tyme[world->rep][world->locus].normd = normd;
             tyme[world->rep][world->locus].trials = trials;
             memmove (tyme[world->rep][world->locus].param, expxv,
                      sizeof (MYREAL) * nn);
         }
-        convergence_check (world, world->options->verbose);
+	if(world->options->gelman)
+	  convergence_check (world, world->options->verbose);
         if (world->loci == 1)
         {
             if ((!world->options->gelman &&
                     world->param_like < world->options->lcepsilon &&
                     world->options->plotnow && !world->options->simulation) ||
                     (world->options->plotnow && world->options->gelman &&
-                     world->gelmanmaxR < GELMAN_MYSTIC_VALUE))
+                     world->convergence->gelmanmeanmaxR[1] < GELMAN_MYSTIC_VALUE))
             {
                 if ((world->options->replicate && repkind != SINGLECHAIN) ||
                         (!world->options->replicate && repkind == SINGLECHAIN))
@@ -1593,7 +1630,7 @@ set_logparam (MYREAL *loga, MYREAL *a, long size)
 {
     long i;
     for (i = 0; i < size; i++)
-        loga[i] = log (a[i]);
+        loga[i] = LOG (a[i]);
 }
 
 /// fills the helper structure with parameters and pointers to world and nr
@@ -1611,8 +1648,8 @@ fill_helper (helper_fmt * helper, MYREAL *param, MYREAL *lparam,
         alpha = param[nr->numpop2];
         if (alpha <= 0.0)
             error ("no no! alpha=0\n");
-        denom = LGAMMA (alpha) + log (theta1 / alpha) * alpha;
-        //  denom = LGAMMA (alpha) + log (1. / alpha) * alpha;
+        denom = LGAMMA (alpha) + LOG (theta1 / alpha) * alpha;
+        //  denom = LGAMMA (alpha) + LOG (1. / alpha) * alpha;
     }
     if (nr->lparam != lparam)
         memcpy (nr->lparam, lparam, sizeof (MYREAL) * nr->partsize);
