@@ -12,17 +12,17 @@
  destroys data.
  
  Peter Beerli 1996, Seattle
- beerli@csit.fsu.edu
+ beerli@scs.fsu.edu
  
 Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
-Copyright 2003-2004 Peter Beerli, Tallahassee FL
+Copyright 2003-2007 Peter Beerli, Tallahassee FL
  
  This software is distributed free of charge for non-commercial use
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: data.c 228 2006-01-19 19:13:57Z beerli $
+$Id: data.c 714 2007-05-22 17:06:46Z beerli $
  
 -------------------------------------------------------*/
 /*! \file data.c
@@ -38,7 +38,11 @@ Data manipulation routines
 #include "tools.h"
 #include "migrate_mpi.h"
 #include "data.h"
-extern long number_genomes (char type);
+#include "sequence.h"
+#ifdef PRETTY
+#include "pretty.h"
+#endif
+extern long number_genomes (int type);
 
 #ifdef DMALLOC_FUNC_CHECK
 #include <dmalloc.h>
@@ -50,6 +54,7 @@ void print_data (world_fmt * world, option_fmt * options, data_fmt * data);
 long find_missing(data_fmt *data, long pop, long locus);
 void print_data_summary (FILE * file, world_fmt * world, option_fmt * options,
                          data_fmt * data);
+short findAllele (data_fmt * data, char s[], long locus);
 void free_datapart (data_fmt * data, option_fmt * options, long locus);
 /*private functions */
 void init_data_structure1 (data_fmt ** data, option_fmt * options);
@@ -72,11 +77,13 @@ void finish_read_seq (FILE * infile, data_fmt * data, option_fmt * options,
                       long pop, long baseread);
 void print_alleledata (world_fmt * world, data_fmt * data,
                        option_fmt * options);
+void print_microdata (world_fmt * world, data_fmt * data,
+                       option_fmt * options);
 void print_seqdata (world_fmt * world, option_fmt * options, data_fmt * data);
 
 void print_header (FILE * outfile, long pop, world_fmt * world,
                    option_fmt * options, data_fmt * data);
-void create_alleles (data_fmt * data);
+void create_alleles (data_fmt * data, option_fmt *options);
 void addAllele (data_fmt * data, char s[], long locus, long *z);
 void set_numind (data_fmt * data);
 void print_seq_pop (long locus, long pop, world_fmt * world,
@@ -87,6 +94,7 @@ void print_locus_head (long locus, world_fmt * world, option_fmt * options,
                        data_fmt * data);
 void find_delimiter (char *title, char *dlm);
 void read_geofile (data_fmt * data, option_fmt * options, long numpop);
+void read_datefile (data_fmt * data, option_fmt * options, long numpop);
 void read_uep_fromfile (FILE * uepfile, long tips, long nmlength, int **uep,
                         long *uepsites, long datatype);
 void read_uepfile (data_fmt * data, option_fmt * options, long numpop);
@@ -123,11 +131,11 @@ destroy_data (data_fmt * data)
   // free data from init_data_structure3
   for (locus = 0; locus < loci; locus++)
     {
-      free(data->allele[locus]);
+      myfree(data->allele[locus]);
     }
-  free(data->allele);
-  free(data->maxalleles);
-  free(data->skiploci);
+  myfree(data->allele);
+  myfree(data->maxalleles);
+  myfree(data->skiploci);
 
   // free data from init_data_structure2
   for(pop=0; pop < numpop ; pop++)
@@ -140,34 +148,46 @@ destroy_data (data_fmt * data)
 	}
       for (ind = 0; ind < indalloc; ind++)
 	{
-	  free(data->indnames[pop][ind]);
-	  free(data->yy[pop][ind]);
+	  myfree(data->indnames[pop][ind]);
+	  myfree(data->yy[pop][ind]);
 	}
-      free(data->indnames[pop]);
-      free(data->yy[pop]);
+      myfree(data->indnames[pop]);
+      myfree(data->yy[pop]);
     }
-  free(data->indnames);
-  free(data->yy);
+  myfree(data->indnames);
+  myfree(data->yy);
 
   // data->yy were already freed in free_datapart()
 
   // free data from init_structure_1
-  free (data->popnames[0]);
-  free (data->numind[0]);
-  free (data->numalleles[0]);
-  free (data->popnames);
-  free (data->numind);
-  free (data->numalleles);
-  free (data->seq->sites);
-  free (data->seq);
-  free (data->geo);
-  free (data->lgeo);
+  myfree(data->popnames[0]);
+  myfree(data->numind[0]);
+  myfree(data->numalleles[0]);
+  myfree(data->popnames);
+  myfree(data->numind);
+  myfree(data->numalleles);
+  myfree(data->seq->sites);
+  myfree(data->seq);
+  myfree(data->geo);
+  myfree(data->lgeo);
   if(data->ogeo != NULL)
     {
-      free (data->ogeo[0]);
-      free (data->ogeo);
+      myfree(data->ogeo[0]);
+      myfree(data->ogeo);
     }
-  free (data);
+#ifdef TESTINGDATE
+  // free sampledates
+  for(locus=0;locus<data->loci;locus++)
+    {
+      for(pop=0;pop < data->numpop; pop++)
+	{
+	  myfree(data->sampledates[pop][locus]);
+	}
+    }
+  myfree(data->sampledates[0]);
+  myfree(data->sampledates);
+#endif
+  myfree(data);
 }
  
 
@@ -207,9 +227,9 @@ get_data (FILE * infile, data_fmt * data, option_fmt * options)
     {
         read_popheader (infile, data, pop, genomes);
         if (options->progress)
-            fprintf (stdout, "Reading %s ...\n", data->popnames[pop]);
+            fprintf (stdout, "Reading (%li) %s ...\n", pop, data->popnames[pop]);
         if (options->writelog)
-            fprintf (options->logfile, "Reading %s ...\n", data->popnames[pop]);
+            fprintf (options->logfile, "Reading (%li) %s ...\n", pop, data->popnames[pop]);
         init_data_structure2 (&data, options, pop);
         read_popdata (infile, data, pop, options);
     }
@@ -218,7 +238,9 @@ get_data (FILE * infile, data_fmt * data, option_fmt * options)
 
     read_uepfile (data, options, data->numpop);
 #endif
-
+#ifdef TESTINGDATE
+    read_datefile(data, options, data->numpop);
+#endif
     if (options->progress)
         fprintf (stdout, "\n\n");
     init_data_structure3 (data);
@@ -226,19 +248,26 @@ get_data (FILE * infile, data_fmt * data, option_fmt * options)
     switch (options->datatype)
     {
     case 'a':
-        create_alleles (data);
+        create_alleles (data, options);
         break;
     case 'b':
-        create_alleles (data);
+        create_alleles (data, options);
         for (pop = 0; pop < data->loci; pop++)
             data->maxalleles[pop] = XBROWN_SIZE;
         break;
     case 'm':
-        create_alleles (data);
+        create_alleles (data, options);
         for (pop = 0; pop < data->loci; pop++)
             data->maxalleles[pop] = options->micro_stepnum;
         break;
     }
+    if(options->murates_fromdata)
+      {
+	if (strchr (SEQUENCETYPES, options->datatype))
+	  {
+	    find_rates_fromdata(data, options);
+	  }
+      }
 }
 
 /* private functions ========================================== */
@@ -323,6 +352,16 @@ init_data_structure2 (data_fmt ** data, option_fmt * options, long pop)
 }
 
 
+short
+findAllele (data_fmt * data, char s[], long locus)
+{
+    short found = 0;
+    while ((strcmp (s, data->allele[locus][found])
+            && data->allele[locus][found][0] != '\0'))
+        found++;
+    return found;
+}
+
 void
 free_datapart (data_fmt * data, option_fmt * options, long locus)
 {
@@ -334,14 +373,14 @@ free_datapart (data_fmt * data, option_fmt * options, long locus)
         {
             if (!strchr (SEQUENCETYPES, options->datatype))
             {
-                free (data->yy[pop][ind][locus][0]);
-                free (data->yy[pop][ind][locus][1]);
-                free (data->yy[pop][ind][locus]);
+                myfree(data->yy[pop][ind][locus][0]);
+                myfree(data->yy[pop][ind][locus][1]);
+                myfree(data->yy[pop][ind][locus]);
             }
             else
             {
-                free (data->yy[pop][ind][locus][0]);
-                free (data->yy[pop][ind][locus]);
+                myfree(data->yy[pop][ind][locus][0]);
+                myfree(data->yy[pop][ind][locus]);
             }
         }
     }
@@ -378,23 +417,27 @@ void read_header (FILE * infile, data_fmt * data, option_fmt * options)
     char input[LINESIZE], *p;
     char title[LINESIZE];
     strcpy(title,"\0");
-    FGETS (input, sizeof (input), infile);
-    if ((p = (char *) strpbrk (input, CRLF)) != NULL)
-        *p = '\0';
+    input[0] = '#';
+    while(input[0]=='#')
+      {
+	FGETS (input, sizeof (input), infile);
+	if ((p = (char *) strpbrk (input, CRLF)) != NULL)
+	  *p = '\0';
+      }
     switch (lowercase (input[0]))
     {
     case 'a':
         sscanf (input, "%1s%ld%ld%[^\n]", &options->datatype, &(data->numpop),
                 &(data->loci), title);
         find_delimiter (title, &data->dlm);
-	if(!(title[0]==' ' || title[0] == '\0'))
+	if(!(title[0] == '\0'))
 	  strcpy(options->title,title);
         break;
     case 'b':
     case 'm':
         sscanf (input, "%1s%ld%ld%1s%[^\n]", &options->datatype,
                 &(data->numpop), &(data->loci), &data->dlm, title);
-	if(!(title[0]==' ' || title[0] == '\0'))
+	if(!(title[0] == '\0'))
 	  strcpy(options->title,title);
         break;
     case 's':
@@ -403,7 +446,7 @@ void read_header (FILE * infile, data_fmt * data, option_fmt * options)
     case 'f':
         sscanf (input, "%1s%ld%ld%[^\n]", &options->datatype, &(data->numpop),
                 &(data->loci), title);
-	if(!(title[0]==' ' || title[0] == '\0'))
+	if(!(title[0] == '\0'))
 	  strcpy(options->title,title);
         break;
     case 'g':   /* fall through if a menu change forces to analyze data
@@ -413,6 +456,11 @@ void read_header (FILE * infile, data_fmt * data, option_fmt * options)
         else
             memmove (input, input + 1, (strlen (input) - 1) * sizeof (char));
     default:
+      if(input[0]== '<')
+	{
+	  usererror ("This data file may contain an XML or HTML tag,\nand cannot be read properly, check the data formatting section in the manual!");
+	  exit(-1);
+	}
         switch (options->datatype)
         {
         case 'a':
@@ -499,7 +547,7 @@ read_sites (data_fmt * data)
             usererror ("will crash anyway, so I stop now\n");
         }
     }
-    free (input);
+    myfree(input);
 
 }
 */
@@ -514,11 +562,17 @@ read_sites (data_fmt * data)
 {
     long locus;
     char *input;
-    input = (char *) mycalloc(LINESIZE,sizeof(char));
+    input = (char *) mycalloc(data->loci * 20 ,sizeof(char));
     
     for (locus = 0; locus < data->loci-1; locus++)
     {
         read_word(data->infile, input);
+	if(input[0] == '#')
+	  {
+	    FGETS(input,LINESIZE,data->infile);
+	    locus--;
+	    continue;
+	  }
         data->seq->sites[locus] = atoi (input);
         if (data->seq->sites[locus] == 0)
         {
@@ -528,10 +582,12 @@ read_sites (data_fmt * data)
             usererror ("will crash anyway, so I stop now\n");
         }
     }
-        FGETS(input,LINESIZE,data->infile);
-        data->seq->sites[locus] = atoi (input);
+    FGETS(input,LINESIZE,data->infile);
+    while(input[0]=='#')
+      FGETS(input,LINESIZE,data->infile);
+    data->seq->sites[locus] = atoi (input);
 
-    free (input);
+    myfree(input);
 
 }
 
@@ -608,10 +664,12 @@ read_popheader (FILE * infile, data_fmt * data, long pop, long genomes)
 void
 read_popheader (FILE * infile, data_fmt * data, long pop, long genomes)
 {
-    long lo;
-    boolean havepopname=FALSE;
-    long locus;
-    char *input;
+    boolean havepopname = FALSE;
+    long    minlength   = 0;
+    long    lo;
+    long    locus;
+    char   *input;
+
     input = (char *) mycalloc(LINESIZE,sizeof(char));
 
     // allows that sequence data can have different numbers of individuals for different loci
@@ -620,50 +678,75 @@ read_popheader (FILE * infile, data_fmt * data, long pop, long genomes)
     if(data->loci>1)
     {
         read_word(data->infile, input);
+	while(input[0]=='#')
+	  {
+	    FGETS(input,LINESIZE,data->infile);
+	    read_word(data->infile, input);
+	  }
         data->numind[pop][0] = atol(input);
         data->numalleles[pop][0] = data->numind[pop][0] * genomes;
-        for(locus=1; locus< data->loci; locus++)
+        for(locus=1; locus < data->loci; locus++)
         {
             read_word(infile, input);
-            if(isdigit(input[0]))
-            {
+	    while(input[0]=='#')
+	      {
+		FGETS(input,LINESIZE,data->infile);
+		read_word(data->infile, input);
+	      }
+            if(isdigit(input[0]) && havepopname == FALSE )
+	      {
                 data->numind[pop][locus] = atol(input);
                 data->numalleles[pop][locus] = data->numind[pop][locus]*genomes;
-            }
+	      }
             else
-            {
+	      {
                 unread_word(infile, input);
                 FGETS(input,LINESIZE,infile);
                 havepopname=TRUE;
-                strncpy(data->popnames[pop],input,MIN(strlen(input),80));
+		if(input!=NULL)
+		  {
+		    minlength = strlen(input);
+		    minlength = MIN(minlength,80);
+		    strncpy(data->popnames[pop],input,minlength);
+		  }
                 break;
-            }
+	      }
         }
         if(!havepopname)
-        {
+	  {
             read_word(infile, input);
-            strncpy(data->popnames[pop],input,MIN(strlen(input),80));
-        }
-
+	    if(input!=NULL)
+	      {
+		minlength = strlen(input);
+		minlength = MIN(minlength,80);
+		strncpy(data->popnames[pop],input,minlength);
+	      }
+	    //            strncpy(data->popnames[pop],input,MIN(strlen(input),80));
+	  }
+	
         // fills numind for additional locus in case the numind was not specified
-        for(lo=locus; lo< data->loci; lo++)
-        {
+        for(lo=locus; lo < data->loci; lo++)
+	  {
             data->numind[pop][lo] = data->numind[pop][locus-1];
             data->numalleles[pop][lo] = data->numind[pop][lo] * genomes;
-        }
+	  }
     }
     else
-    {
+      {
         // only one locus so we can use old scheme [see below]
         FGETS(input,LINESIZE,infile);
+	while(input[0]=='#')
+	  {
+	    FGETS(input,LINESIZE,data->infile);
+	  }
         sscanf (input, "%ld%[^\n]", &(data->numind[pop][0]),
                 data->popnames[pop]);
         data->numalleles[pop][0] = data->numind[pop][0]*genomes;
-    }
-
+      }
+    
     translate (data->popnames[pop], ' ', '_');
     translate (data->popnames[pop], '\t', '_');
-    free(input);
+    myfree(input);
 }
 
 
@@ -672,9 +755,15 @@ read_indname (FILE * file, data_fmt * data, long pop, long ind, long nmlength)
 {
     long i = 0;
     char ch;
+    char input[LINESIZE];
     while (i < nmlength)
     {
         ch = getc (file);
+	while(ch =='#')
+	  {
+	    FGETS(input,LINESIZE,data->infile);
+	    ch = getc (file);
+	  }
         if(!strchr("\r\n",ch))
             data->indnames[pop][ind][i++] = ch;
         if(strchr("\t",ch))
@@ -773,10 +862,10 @@ read_microalleles (FILE * infile, data_fmt * data, long pop, long ind)
         strcpy (data->yy[pop][ind][locus][0], a1);
         strcpy (data->yy[pop][ind][locus][1], a2);
     }
-    free (a);
-    free (a1);
-    free (a2);
-    free (isave);
+    myfree(a);
+    myfree(a1);
+    myfree(a2);
+    myfree(isave);
 }
 
 void
@@ -815,8 +904,8 @@ read_alleles (FILE * infile, data_fmt * data, long pop, long ind)
             data->yy[pop][ind][locus][1][1] = '\0';
         }
     }
-    free (a);
-    free (isave);
+    myfree(a);
+    myfree(isave);
 }
 
 long
@@ -828,6 +917,10 @@ read_ind_seq (FILE * infile, data_fmt * data, option_fmt * options,
     j = (options->interleaved) ? baseread : 0;
     charstate = getc (infile);
     ungetc ((int) charstate, infile);
+    if(options->printdata)
+      {
+    	fprintf(stdout,"%s|",data->indnames[pop][ind]);
+      }
     while (j < data->seq->sites[locus]
             && !(options->interleaved && strchr(CRLF,charstate)))
     {
@@ -848,12 +941,15 @@ read_ind_seq (FILE * infile, data_fmt * data, option_fmt * options,
                 || (charstate >= '0' && charstate <= '9') || charstate == '\\')
             continue;
         charstate = uppercase (charstate);
-        //      printf("%c",charstate);
+	if(options->printdata)
+	{
+	  fprintf(stdout, "%c",charstate);
+	}
         if ((strchr ("ABCDGHKMNRSTUVWXY?O-", (int) charstate)) == NULL)
         {
             printf
             ("ERROR: BAD BASE: %c AT POSITION %5ld OF INDIVIDUUM %3li in POPULATION %ld\n",
-             charstate, j, ind, pop);
+             charstate, j, ind+1, pop+1);
             printf
             ("Last complete sequences was in population %li, individual %li and locus %li:\n%s",
              pop + 1, ind - 1, locus, data->indnames[pop][ind - 1]);
@@ -873,6 +969,10 @@ read_ind_seq (FILE * infile, data_fmt * data, option_fmt * options,
             ungetc((int) charstate, infile);
     }
 #endif
+    if(options->printdata)
+     {
+       fprintf(stdout,"\n");
+     }
     return j;
 }
 
@@ -889,7 +989,7 @@ read_distance_fromfile (FILE * dfile, long tips, long nmlength, MYREAL **m)
         FGETS (input, LONGLINESIZE, dfile); //reads header line with
         for (i = 0; i < tips; i++) // of individuals
         {
-            //reads header line with
+            //reads the population label
             FGETS (input, nmlength + 1, dfile);
             for (j = 0; j < tips; j++)
             {
@@ -898,6 +998,11 @@ read_distance_fromfile (FILE * dfile, long tips, long nmlength, MYREAL **m)
 #else
                 fscanf (dfile, "%lf", &m[i][j]);
 #endif
+		if((i!=j) && (m[i][j] < EPSILON))
+		  {
+		    warning("Reading geofile: adjusting dist[%li][%li]=%f to %f\n",i,j,m[i][j],EPSILON);
+		    m[i][j] = EPSILON;
+		  }
             }
             // reads the last \n from the
             // data matrix
@@ -1027,7 +1132,7 @@ print_data_summary (FILE * file, world_fmt * world, option_fmt * options,
     long *totalmiss;
     total = (long *) mycalloc(data->loci,sizeof(long));
     totalmiss = (long *) mycalloc(data->loci,sizeof(long));
-    fprintf (file, "Summary of data:\n");
+    fprintf (file, "\nSummary of data:\n");
     fprintf (file, "---------------\n");
     switch (options->datatype)
     {
@@ -1118,8 +1223,8 @@ print_data_summary (FILE * file, world_fmt * world, option_fmt * options,
           }
       }    
     fprintf(file,"\n");
-    free(total);
-    free(totalmiss);
+    myfree(total);
+    myfree(totalmiss);
     fflush (file);
 }
 
@@ -1131,9 +1236,14 @@ print_data (world_fmt * world, option_fmt * options, data_fmt * data)
         switch (options->datatype)
         {
         case 'a':
+	  //	  if(options->dlm=='\0')
+          //  print_alleledata (world, data, options);
+	  //else
+	  print_microdata (world, data, options);
+	  break;
         case 'b':
         case 'm':
-            print_alleledata (world, data, options);
+            print_microdata (world, data, options);
             break;
         case 's':
         case 'n':
@@ -1145,8 +1255,182 @@ print_data (world_fmt * world, option_fmt * options, data_fmt * data)
     }
 }
 
+///
+/// calculate allele frequency spectrum and print
+/// allele population1 .. populationN All
+void print_spectra(world_fmt * world, option_fmt * options,data_fmt * data)
+{
+  long found;
+  long locus;
+  long a;
+  long pop;
+  long ind;
+  MYREAL **total;
+  MYREAL allfreq;
+  long *maxalleles;
+  long *maxallelepop;
+  MYREAL ***freq;
+  char *thisallele;
+  char *thatallele;
+  FILE *outfile = world->outfile;
+  if (strchr (SEQUENCETYPES, options->datatype))
+    return; /*we do not calculate allele frequencies for DNA sequences*/
+  // find total number of alleles
+  maxalleles = (long *) mycalloc(data->loci, sizeof(long));
+  maxallelepop = (long *) mycalloc(data->numpop, sizeof(long));
+  for (locus = 0; locus < data->loci; locus++)
+    {
+      maxalleles[locus] = findAllele(data,"\0",locus);
+      //qsort((void *) data->allele[locus], maxalleles[locus], sizeof(char *), strcmp);
+    }
+
+  // create bins for each population and all
+  freq = (MYREAL ***) mycalloc(data->numpop, sizeof(MYREAL **));
+  doublevec2d(&total,data->numpop,data->loci);
+  for (pop = 0; pop < data->numpop; pop++)
+    {
+      freq[pop] = (MYREAL **) mycalloc(data->loci, sizeof(MYREAL *));
+      for (locus = 0; locus < data->loci; locus++)
+	{
+	  freq[pop][locus] = (MYREAL *) mycalloc(maxalleles[locus], sizeof(MYREAL));
+	}
+    }
+  
+
+  // calculate spectrum
+  for (pop = 0; pop < data->numpop; pop++)
+    {
+      for (ind = 0; ind < data->numind[pop][0]; ind++)
+        {
+	  for (locus = 0; locus < data->loci; locus++)
+	    {
+	      thisallele = data->yy[pop][ind][locus][0];
+	      found = findAllele(data, thisallele, locus);
+	      thatallele = data->yy[pop][ind][locus][1];
+	      if(!strcmp(thisallele,thatallele))
+		{
+		     freq[pop][locus][found] += 2.0 ;
+		     if(!strchr(thisallele,'?'))
+			total[pop][locus] += 2.0 ;
+		}
+	      else
+		{
+		  freq[pop][locus][found] += 1.0 ;
+		  if(!strchr(thisallele,'?'))
+		     total[pop][locus] += 1.0 ;
+		  found = findAllele(data, thatallele, locus);
+		  freq[pop][locus][found] += 1.0 ;
+		  if(!strchr(thatallele,'?'))
+		    total[pop][locus] += 1.0 ;
+		}
+	    }
+	}
+    }
+  for (pop = 0; pop < data->numpop; pop++)
+    {
+      for (locus = 0; locus < data->loci; locus++)
+	{
+	  for(a=0; a < maxalleles[locus]; a++)
+	    {
+	      freq[pop][locus][a] /= (MYREAL) total[pop][locus];
+	    }
+	}
+    }
+  // print in ascii
+  fprintf(outfile,"Allele frequency spectra\n");
+  fprintf(outfile,"========================\n\n");
+  for (locus = 0; locus < data->loci; locus++)
+    {
+      memset(maxallelepop,'\0',sizeof(long)*data->numpop);
+      fprintf(outfile,"Locus %li\n", locus);
+      fprintf(outfile,"Allele  ");
+      for (pop = 0; pop < data->numpop; pop++)
+	fprintf(outfile,"Pop%-2li  ",pop+1);
+      fprintf(outfile,"All\n-------");
+      for (pop = 0; pop < data->numpop+1; pop++)
+	fprintf(outfile,"-------");
+      fprintf(outfile, "\n");
+      for(a=0; a < maxalleles[locus]; a++)
+	{
+	  allfreq = 0.0;
+	  fprintf(outfile,"%6s ",data->allele[locus][a]);
+	  for (pop = 0; pop < data->numpop; pop++)
+	    {
+	      if(freq[pop][locus][a]>0.0)
+		{
+		  maxallelepop[pop] += 1;
+		  fprintf(outfile," %1.3f ",freq[pop][locus][a]);
+		  allfreq += freq[pop][locus][a];
+		}
+	      else
+		{
+		  fprintf(outfile,"   -   ");
+		}
+	    }
+	  fprintf(outfile," %1.3f\n", (MYREAL) allfreq/data->numpop);
+	}
+      fprintf(outfile,"Total ");
+      for (pop = 0; pop < data->numpop; pop++)
+	fprintf(outfile," %5li ",maxallelepop[pop]);
+      fprintf(outfile," %5li\n\n", maxalleles[locus]);
+    }
+  // printd in PDF
+#ifdef PRETTY
+  pdf_print_spectra(data, freq, maxalleles);
+#endif
+  fflush(outfile);
+  // cleanup
+  for (pop = 0; pop < data->numpop; pop++)
+    {
+      for (locus = 0; locus < data->loci; locus++)
+	{
+	  myfree(freq[pop][locus]);
+	}
+      myfree(freq[pop]);
+    }
+  myfree(freq);
+  myfree(maxalleles);
+  myfree(maxallelepop);
+  free_doublevec2d(total);
+}
+
 void
 print_alleledata (world_fmt * world, data_fmt * data, option_fmt * options)
+{
+    long i, pop, ind, locus, mult80;
+    for (pop = 0; pop < data->numpop; pop++)
+    {
+        print_header (world->outfile, pop, world, options, data);
+        for (ind = 0; ind < data->numind[pop][0]; ind++)
+        {
+            fprintf (world->outfile, "%-*.*s ", (int) options->nmlength,
+                     (int) options->nmlength, data->indnames[pop][ind]);
+            mult80 = options->nmlength;
+            for (locus = 0; locus < data->loci; locus++)
+            {
+                mult80 +=
+                    1 + (long) (strlen (data->yy[pop][ind][locus][0]));
+                if (mult80 >= 80)
+                {
+                    mult80 = 0;
+                    fprintf (world->outfile, "\n");
+                    for (i = 0; i < options->nmlength; i++)
+                        FPRINTF(world->outfile, " ");
+                }
+                fprintf (world->outfile, " %c.%c",
+                         data->yy[pop][ind][locus][0][0],
+                         data->yy[pop][ind][locus][0][1]);
+            }
+            fprintf (world->outfile, "\n");
+        }
+        fprintf (world->outfile, "\n");
+    }
+    fprintf (world->outfile, "\n\n");
+    fflush (world->outfile);
+}
+
+void
+print_microdata (world_fmt * world, data_fmt * data, option_fmt * options)
 {
     long i, pop, ind, locus, mult80;
     for (pop = 0; pop < data->numpop; pop++)
@@ -1232,8 +1516,11 @@ print_header (FILE * outfile, long pop, world_fmt * world,
 
 
 void
-create_alleles (data_fmt * data)
+create_alleles (data_fmt * data, option_fmt *options)
 {
+  long n=0;
+  MYREAL  mean=0.;
+  MYREAL  delta=0.;
     long locus, pop, ind;
     long z;
     char a1[DEFAULT_ALLELENMLENGTH];
@@ -1260,7 +1547,26 @@ create_alleles (data_fmt * data)
         }
         data->maxalleles[locus] = z + 1;
         /* + 1: for all the unencountered alleles */
+	if(options->murates_fromdata)
+	  {
+	    if(options->mu_rates==NULL)
+	      {
+		options->mu_rates = (MYREAL * ) mycalloc(data->loci,sizeof(MYREAL));
+	      }
+	    options->mu_rates[locus] = z;
+	    n = n + 1;
+	    delta = options->mu_rates[locus] - mean;
+	    mean += delta/n;
+	  }
     }
+    if(options->murates_fromdata)
+      {
+	options->muloci = data->loci;
+	for (locus=0; locus < data->loci; locus++)
+	  {
+	    options->mu_rates[locus] /= mean;
+	  }
+      }
 }
 
 void
@@ -1344,7 +1650,7 @@ print_locus_head (long locus, world_fmt * world, option_fmt * options,
              (int) options->nmlength, (int) options->nmlength, head);
     fprintf (world->outfile, " --------40 --------50 --------60\n");
 
-    free (head);
+    myfree(head);
 }
 
 void
@@ -1375,13 +1681,81 @@ read_geofile (data_fmt * data, option_fmt * options, long numpop)
                 {
                     data->geo[mm2m (i, j, numpop)] =   1. / data->ogeo[i][j];
                     data->lgeo[mm2m (i, j, numpop)] =  data->ogeo[i][j] > 0.0 ?
-                                                       log (1. / data->ogeo[i][j]) : -DBL_MAX;
+                                                       LOG (1. / data->ogeo[i][j]) : -DBL_MAX;
                 }
             }
         }
     }
 }
 
+///
+/// read the file with the tip dates and returns the oldest date
+MYREAL
+read_date_fromfile (FILE * datefile, data_fmt *data, option_fmt *options, long nmlength)
+{
+    char input[LINESIZE];
+    long pop;
+    long locus;
+    long ind;
+    MYREAL oldest = 0. ;
+    MYREAL temp;
+    if (datefile != NULL)
+    {
+        FGETS (input, LINESIZE, datefile); //title line 
+	fprintf(stdout,"\n%i> Tip dates from file %s\n----------------------------------------\n", myID, options->datefilename);	
+	fprintf(stdout,"Generation time:            %f\n", options->generation_year);
+	for(pop=0;pop<data->numpop;pop++)
+	  {
+	    FGETS (input, LINESIZE, datefile); //first populations 
+	    for (locus=0; locus < data->loci; locus++)
+	      {
+		fprintf(stdout,"Mutation rate of Locus %li: %g\n", locus, options->mutationrate_year[locus]);
+		for(ind=0; ind < data->numind[pop][locus]; ind++)
+		  {
+		    FGETS (input, LINESIZE, datefile);
+		    sscanf (input+nmlength, "%lf",&temp);
+		    fprintf(stdout,"Locus %li: Tipdate %*.*s %f %f\n", locus, (int) nmlength, (int) nmlength, input, temp, temp * options->mutationrate_year[locus] * options->generation_year);
+		    data->sampledates[pop][locus][ind].date = temp;
+		    if(oldest < temp)
+		      oldest = temp;
+		  }
+	      }
+	  }
+    }
+    return oldest;
+}
+
+///
+/// read the file with the tip dates
+void
+read_datefile (data_fmt * data, option_fmt * options, long numpop)
+{
+  long locus, pop;
+  data->sampledates = (tipdate_fmt ***) mycalloc (data->numpop, sizeof (tipdate_fmt **));
+  data->sampledates[0] = (tipdate_fmt **) mycalloc (data->numpop * data->loci, sizeof (tipdate_fmt *));
+  for (pop = 1; pop < data->numpop; pop++)
+    {
+      data->sampledates[pop] = data->sampledates[0] + data->loci * pop;
+    }
+  for(locus=0;locus<data->loci;locus++)
+    {
+      for(pop=0;pop < data->numpop; pop++)
+	{
+	  data->sampledates[pop][locus] = (tipdate_fmt*) mycalloc(data->numind[pop][locus],sizeof(tipdate_fmt));
+	}
+    }
+
+  if (options->has_datefile)
+    {
+      data->maxsampledate = read_date_fromfile (data->datefile, data, options, options->nmlength);
+      //      printf("%i> in data section maxsampledate=%f\n",myID, data->maxsampledate);
+    }
+  else
+    {
+      data->maxsampledate=0.0;
+    }
+}
+
 #ifdef UEP
 void
 read_uepfile (data_fmt * data, option_fmt * options, long numpop)
