@@ -9,7 +9,7 @@
   School of Computational Science and Biological Sciences
   Florida State University
   Tallahassee, FL 32306-4120
-  beerli@csit.fsu.edu
+  beerli@scs.fsu.edu
  
   With help of Joe Felsenstein (joe@genetics.washington.edu),
   Mary Kuhner and Jon Yamato (mkkuhner@genetics.washington.edu)
@@ -17,30 +17,30 @@
   
 Copyright 2002 Peter Beerli and Joseph Felsenstein
  
-$Id: definitions.h 230 2006-01-22 15:18:01Z beerli $
+$Id: definitions.h 762 2007-07-12 22:25:28Z beerli $
  
   *----------------------------------------------------------------
   */
 #ifndef MIGRATEVERSION
-#define MIGRATEVERSION "2.1.0"
+#define MIGRATEVERSION "2.3"
 #else
 #ifdef SGI
 #undef MIGRATEVERSION
-#define MIGRATEVERSION "2.1.0"
+#define MIGRATEVERSION "2.3"
 #endif
 #endif
 #ifndef MIGRATESUBVERSION
 #define MIGRATESUBVERSION "x"
 #endif
 
-#define MAINTAINER "Peter Beerli <beerli@csit.fsu.edu>"
+#define MAINTAINER "Peter Beerli <beerli@scs.fsu.edu>"
 /*------------------------------------------------
   Adjusting the source to the system configuration
 */
 #define USE_MYREAL_DOUBLE 1
 #define MYREAL double
 // use this when using floats 
-// #define USE_MYREAL_FLOAT 0
+//#define USE_MYREAL_FLOAT 0
 //#define MYREAL float
 
 #ifdef HAVE_CONFIG_H
@@ -59,6 +59,15 @@ $Id: definitions.h 230 2006-01-22 15:18:01Z beerli $
 #define FPRINTF fprintf
 #endif
 
+/* random number system */
+#define LCG
+#ifdef MERSENNETWISTER
+#undef LCG
+#endif
+#ifdef SPRNG
+#undef LCG
+#endif
+
 #ifdef __MWERKS__
     #include "conf.h"
     #undef HAVE_LGAMMA
@@ -72,6 +81,9 @@ $Id: definitions.h 230 2006-01-22 15:18:01Z beerli $
     #define DOS
     #undef HAVE_STRINGS_H
     #undef HAVE_LGAMMA
+    #define HAVE_STRFTIME 1
+    #define HIGHBITS
+    #define LAMARC_MALLOC
     #endif /*windows */
 #endif /*MWERKS*/
 
@@ -79,6 +91,8 @@ $Id: definitions.h 230 2006-01-22 15:18:01Z beerli $
     #define DOS
     #undef HAVE_STRINGS_H
     #undef HAVE_LGAMMA
+    #define HAVE_STRFTIME
+    #undef HAVE_STRSEP
 #endif
 
 /* we have found no strftime() and replace the time
@@ -132,6 +146,13 @@ $Id: definitions.h 230 2006-01-22 15:18:01Z beerli $
 #define BAYESMDIMFILENUM 22
 #define PDFOUTFILE "outfile.pdf"
 #define PDFOUTFILENUM 21
+#define SKYLINEFILE "skylinefile"
+#define SKYLINEFILENUM 23
+#define MYMAXFILENUM 24
+#define DIVFILE "divfile"
+#define DIVFILENUM 24
+#define TIPDATEFILE "datefile"
+#define TIPDATEFILENUM 25
 #define BAYESNUMBIN 200
 #define PRETTY_MAX 0
 #define PRETTY_P99 1
@@ -140,6 +161,7 @@ $Id: definitions.h 230 2006-01-22 15:18:01Z beerli $
 #define THETAPRIOR 0
 #define MIGPRIOR   1
 #define RATEPRIOR  2
+#define PRIOR_SIZE 3 /*used because of slice sampler*/
 #ifdef __STDIO_LOADED
 #define VMS
 #define MACHINE_TYPE "VAX/VMS C"
@@ -222,6 +244,8 @@ typedef int boolean;
 #define FALSE   (boolean) 0
 #endif
 
+#define INVALID -999
+
 #define SETBITS 32
 /* for snp data with panel see sequence.c*/
 #define FIRST 0
@@ -236,14 +260,22 @@ typedef int boolean;
 #ifdef COMPAQ
 #define EXP(a)  (((a)< -100) ? 0.0 : exp ((a)))
 #else
+#ifdef FAST_EXP
+#define EXP(a)  fast_exp ((a))
+#else
 #define EXP(a)  exp ((a))
 #endif
+#endif
 
 // I was experimenting with a fast log implementation
 // but it is
 //not fast enough #define LOG(a) ((MYREAL) fast_log(((float) a)))
 // so back to standard log
+#ifdef FAST_EXP
+#define LOG(a) ((MYREAL) fast_log(((float) a)))
+#else
 #define LOG(a) log((a))
+#endif
 
 /* defines for speedier calc_like when we do NOT use undefined SLOWNET*/
 #ifdef SLOWNET
@@ -289,6 +321,7 @@ typedef int boolean;
 #ifndef HUGE
 #define HUGE 1e200
 #endif 
+#define SICK_VALUE              -1
 #define ONE                     1L
 #define TWO                     2L
 #define THREE                   3L
@@ -302,6 +335,7 @@ typedef int boolean;
 #define LONGCHAINEPSILON  10e100 /* an unreasonable big number,
 so that the given number of
 long chains is used */
+#define CHAINVARIANCEDELTA 0.01 /*difference of variances of parts of a chain [see burnin_autostop]*/
 #define GELMAN_MYSTIC_VALUE 1.2
 #define PLUSCHAIN 10
 #ifndef MAXLONG
@@ -314,6 +348,9 @@ long chains is used */
 #define DBL_EPSILON 2.2204460492503131e-16
 #endif
 #endif
+#ifndef FLT_EPSILON
+#define FLT_EPSILON 1.19209290e-07F
+#endif
 /* some math constants */
 #define HALF 0.5
 #define QUARTER 0.25
@@ -341,6 +378,7 @@ long chains is used */
 #define ATOF(a) strtod((a),(char **) NULL, 10)
 
 /* prior setting */
+#define SLICE        5
 #define GAMMAPRIOR   4
 #define MULTPRIOR    3
 #define WEXPPRIOR    2
@@ -352,9 +390,11 @@ long chains is used */
 #define SMALLEST_THETA    1e-10
 #define DNA_GUESS_THETA    0.01
 #define BIGGEST_THETA      1e4
-#define SMALLEST_MIGRATION 0.000001  
+#define SMALLEST_MIGRATION 0.0  
 #define DNA_GUESS_MIG      100
-#define BIGGEST_MIGRATION   10e9 
+#define BIGGEST_MIGRATION   10e9
+#define SMALLEST_RATE 1e-20
+#define BIGGEST_RATE  1e20 
 // defines the value that is used as a minimum summary statistic for migration events on all trees
 // version 1.2.4 had a value of 0.00000001, it seems that values of 0.1 are too large.
 // in version -1.6.9 this is set to zero, in 1.7 trial with 0.01 PB Dec 11 2002  [this was correct for Nm]
@@ -362,7 +402,7 @@ long chains is used */
 #define MINMIGSUMSTAT 0.000001
 /* the migration limit is per population, should be plenty
    for moderate sample sizes */
-#define MIGRATION_LIMIT    100
+#define MIGRATION_LIMIT     1000
 #define SMALLEST_PROB     1e-100
 #define SMALLEST_GAMMA      1e-3
 #define BIGGEST_GAMMA        1e9
@@ -379,8 +419,9 @@ long chains is used */
 #define WATTERSON              1 /*- start values are Watterson estimate*/
 #define EWENS                  2 /*- start values are Ewens estimate*/
 #define FST                    3 /*+ start values are FST estimate */
-#define RANDOMESTIMATE         4 /*+ start values are RANDOM estimate */
-#define PARAMGRID              5 /*+ use a range of values + replicate=yes:# */
+#define NRANDOMESTIMATE         4 /*+ start values are RANDOM estimate */
+#define URANDOMESTIMATE         5 /*+ start values are RANDOM estimate */
+#define PARAMGRID              6 /*+ use a range of values + replicate=yes:# */
 /* defines for the migration0 values */
 /* define OWN                  0   + start values in parmfile */
 /* define FST                  3   + start values are FST estimates*/
@@ -472,7 +513,7 @@ long chains is used */
 /* stop while loops after some time*/
 #define PANIC_MAX           1000
 /* microsatellite stuff*/
-#define MICRO_THRESHOLD       10
+#define MICRO_THRESHOLD      200
 #define MAX_MICROSTEPNUM     200
 #define XBROWN_SIZE            3
 /* sequence stuff*/
@@ -487,9 +528,4 @@ long chains is used */
 #ifdef powerpc
 #include <Accelerate/Accelerate.h>
 #endif
-
-#ifdef MEMDEBUG
-//#include <memdebug.h>
-#include "CMemLeak.h"
-#endif
 #endif /*definitions */
