@@ -6,7 +6,7 @@ using a Metropolis-Hastings Monte Carlo algorithm
  Gammalike and derivatives  R O U T I N E S
  
  Peter Beerli 2001, Seattle
- beerli@csit.fsu.edu
+ beerli@scs.fsu.edu
  
  Copyright 2001-2003 Peter Beerli and Joseph Felsenstein
  Copyright 2004-2005 Peter Beerli
@@ -15,7 +15,7 @@ using a Metropolis-Hastings Monte Carlo algorithm
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
-$Id: gammalike.c 175 2005-12-02 14:51:23Z beerli $
+$Id: gammalike.c 514 2006-11-09 20:14:08Z beerli $
 */
 /*! \file gammalike.c 
 
@@ -252,14 +252,14 @@ gamma_locus_like (nr_fmt * nr, MYREAL *oparam, MYREAL *olparam, MYREAL denom,
     //  printf("\n");
     for (r = 0; r < GAMMA_INTERVALS; r++)
         summ += EXP (temp[r] - tempmax);
-    free (temp);
-    return log (summ) + tempmax;
+    myfree(temp);
+    return LOG (summ) + tempmax;
 }
 
 MYREAL
 loggf (MYREAL r, MYREAL alpha, MYREAL theta1)
 {
-    return -r * alpha / theta1 + log (r) * (alpha - 1.);
+    return -r * alpha / theta1 + LOG (r) * (alpha - 1.);
 }
 
 void
@@ -284,7 +284,7 @@ gamma_loci_difference (helper_fmt * helper)
         memcpy (param, expxv, sizeof (MYREAL) * size);
         memcpy (lparam, xv, sizeof (MYREAL) * size);
         param1 = (param[i] += epsilon);
-        lparam[i] = log (param[i]);
+        lparam[i] = LOG (param[i]);
         high = gamma_loci_like (helper, param, lparam, helper->weight);
         param[i] -= 2. * epsilon;
         if (param[i] < 0.0)
@@ -294,13 +294,13 @@ gamma_loci_difference (helper_fmt * helper)
         }
         else
         {
-            lparam[i] = log (param[i]);
+            lparam[i] = LOG (param[i]);
             low = gamma_loci_like (helper, param, lparam, helper->weight);
         }
         nr->d[i] = (high - low) / (param1 - param[i]);
     }
-    free (lparam);
-    free (param);
+    myfree(lparam);
+    myfree(param);
 }
 
 void
@@ -354,7 +354,7 @@ gamma_locus_derivative (helper_fmt * helper, long locus)
     for (r = 0; r < GAMMA_INTERVALS; r++)
     {
         rate = nr->rate[r];
-        lograte = log (rate);
+        lograte = LOG (rate);
         set_gamma_param (nr->param, helper->expxv,
                          nr->lparam, helper->xv, rate, nr);
         ll = nr->probcat[r] + calc_locus_like (nr, nr->param, nr->lparam, locus);
@@ -366,14 +366,14 @@ gamma_locus_derivative (helper_fmt * helper, long locus)
         tmp = rate * alpha / (theta1 * theta1) + nr->parts[0];
 
         stemp[0][r] = tmp < 0. ? -1 : 1;
-        temp[0][r] = ll + log (fabs (tmp));
+        temp[0][r] = ll + LOG (fabs (tmp));
         if (tempmax[0] < temp[0][r])
             tempmax[0] = temp[0][r];
         //all others except alpha
         for (i = 1; i < nr->numpop2; i++)
         {
             stemp[i][r] = (nr->parts[i] < 0.) ? -1 : 1;
-            temp[i][r] = ll + log (fabs (nr->parts[i]));
+            temp[i][r] = ll + LOG (fabs (nr->parts[i]));
             if (tempmax[i] < temp[i][r])
                 tempmax[i] = temp[i][r];
         }
@@ -381,7 +381,7 @@ gamma_locus_derivative (helper_fmt * helper, long locus)
         tmp = -rate/theta1 + lograte;
         //tmp = - theta1*lograte + rate;
         stemp[i][r] = tmp < 0. ? -1 : 1;
-        temp[i][r] = ll + log (fabs (tmp));
+        temp[i][r] = ll + LOG (fabs (tmp));
         if (tempmax[i] < temp[i][r])
             tempmax[i] = temp[i][r];
     }
@@ -407,18 +407,18 @@ gamma_locus_derivative (helper_fmt * helper, long locus)
             if (i == nr->numpop2)
             {
                 sumi = sumi *(EXP (tempmax[i] - nr->locilikes[locus]))
-                       - polygamma (0, alpha) - log (theta1 / alpha) +1.;
+                       - polygamma (0, alpha) - LOG (theta1 / alpha) +1.;
             }
             else
                 sumi = sumi * EXP (tempmax[i] - nr->locilikes[locus]);
         }
         nr->d[ii] += sumi;
     }
-    free (temp[0]);
-    free (temp);
-    free (stemp[0]);
-    free (stemp);
-    //free(summll);
+    myfree(temp[0]);
+    myfree(temp);
+    myfree(stemp[0]);
+    myfree(stemp);
+    //myfree(summll);
 }
 
 void
@@ -496,5 +496,5 @@ gamma_parts_gradient (MYREAL x, MYREAL *param, nr_fmt * nr, long locus)
     for (pop = 0; pop < nr->world->numpop2; pop++)
         nr->parts[pop] /= nr->PGC[locus];
     // printf("nr->PGC[%li]=%f == %f\n",locus,nr->PGC[locus],summm);
-    free (part);
+    myfree(part);
 }
