@@ -24,7 +24,7 @@ Copyright 2003-2005 Peter Beerli, Tallahassee FL
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
-$Id: laguerre.c 175 2005-12-02 14:51:23Z beerli $
+$Id: laguerre.c 356 2006-05-17 18:22:36Z beerli $
 */
 /* \file laguerre.c
 Calculates the Laguerre Quadrature points, used for gamma deviated site rate variation
@@ -307,9 +307,9 @@ inithermitcat (long categs, MYREAL alpha, MYREAL theta1,
     for (i = 0; i < categs; i++) // set rates
     {
         rate[i] = theta1 + std * hroot[i];
-        probcat[i] = log (probcat[i]);
+        probcat[i] = LOG (probcat[i]);
     }
-    free (hroot);
+    myfree(hroot);
 }
 
 ///
@@ -432,11 +432,11 @@ initlaguerrecat (long categs, MYREAL alpha, MYREAL theta1, MYREAL *rate,
     }
     for (i = 0; i < categs; i++)
     {
-        probcat[i] = log (probcat[i]);
+        probcat[i] = LOG (probcat[i]);
         rate[i] *= theta1;
     }
-    free (lgroot[0]);
-    free (lgroot);
+    myfree(lgroot[0]);
+    myfree(lgroot);
 }    /* initgammacat */
 
 
@@ -465,8 +465,8 @@ integrate_laguerre (long categs, MYREAL *rate,
     {
         summ += ((MYREAL) stemp[i]) * probcat[i] * EXP (temp[i] - *rmax);
     }
-    free (temp);
-    free (stemp);
+    myfree(temp);
+    myfree(stemp);
     *result = summ;
 }
 
