@@ -1,3 +1,4 @@
+
 /*------------------------------------------------------
  Maximum likelihood estimation 
  of migration rate  and effectice population size
@@ -7,7 +8,7 @@
  
  moved out from world.c                                                                                                               
  Peter Beerli 2000, Seattle
- beerli@csit.fsu.edu
+ beerli@scs.fsu.edu
  
  Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
 Copyright 2003-2004 Peter Beerli, Tallahassee FL
@@ -16,13 +17,12 @@ Copyright 2003-2004 Peter Beerli, Tallahassee FL
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
-$Id: lrt.c 175 2005-12-02 14:51:23Z beerli $
+$Id: lrt.c 514 2006-11-09 20:14:08Z beerli $
  
 -------------------------------------------------------*/
 /* \file lrt.c 
 Calculates likelihood ratio tests
 */
-#define SICK_VALUE    -1
 #include "migration.h"
 #include "sighandler.h"
 #include "tools.h"
@@ -64,7 +64,7 @@ void parse_h0string(char ***box, long *size, long *tempsize,
 void print_box(world_fmt* world,char **box,long size,
                MYREAL like0, MYREAL like1,MYREAL lrt, MYREAL chiprob, MYREAL chiprob2, MYREAL aic, long df, long aicparamnum);
 
-long simplify_lrtvalues(char * in, char*** out);
+long simplify_lrtvalues(char * in, char** out);
 
 boolean
 mywhitespace (char ch)
@@ -142,9 +142,9 @@ print_lratio_test (world_fmt * world, long *Gmax)
         }
     }
     fflush (world->outfile);
-    free (param0);
-    free (param1);
-    free (maxwhich);
+    myfree(param0);
+    myfree(param1);
+    myfree(maxwhich);
     destroy_nr (nr, world);
 }
 
@@ -169,7 +169,7 @@ test_loci_like (nr_fmt * nr, MYREAL *param0, MYREAL *param1, long df,
     //long zi;
     long z = 0, w = 0;
     // long pop;
-    MYREAL normd = 0;
+    MYREAL normd = 0.0;
     long *which;
     MYREAL **hess;
     MYREAL /**values,*/ *saveparam0;
@@ -270,12 +270,12 @@ test_loci_like (nr_fmt * nr, MYREAL *param0, MYREAL *param1, long df,
 
       }
     print_lrt_box(world,param0, param1, zeros, elem, this_string1, like0, like1, df);
-    free (lparam0);
-    free (lparam1);
-    free (saveparam0);
-    free (which);    
-    free(hess[0]);
-    free(hess);
+    myfree(lparam0);
+    myfree(lparam1);
+    myfree(saveparam0);
+    myfree(which);    
+    myfree(hess[0]);
+    myfree(hess);
 }
 
 void print_lrt_box(world_fmt *world, MYREAL *param0, MYREAL *param1, long zeros, long elem,
@@ -319,8 +319,8 @@ void print_lrt_box(world_fmt *world, MYREAL *param0, MYREAL *param1, long zeros,
         print_line(stdout,'-',79,CONT);
     print_line(world->outfile,'-',79,CONT);
     for(i=0; i<size; i++)
-        free(box[i]);
-    free(box);
+        myfree(box[i]);
+    myfree(box);
 }
 
 long parse_h0(char ***box, long *size, char *thisString, MYREAL *param0, MYREAL *param1 , long elem, world_fmt *world)
@@ -457,11 +457,10 @@ set_test_param (MYREAL *param, lr_data_fmt *lrtdata, world_fmt * world, long lrl
     
     paramtype = (char *) mycalloc (1, sizeof (char) * elem);
     ss = (char *) mycalloc (LONGLINESIZE, sizeof (char));
-    custm = (char **) mycalloc (LONGLINESIZE, sizeof (char *));
-
+    charvec2d(&custm,world->numpop2 + 1, LONGLINESIZE);
     strcpy (ss, lrtdata->value1);
 
-    elements = simplify_lrtvalues(ss,&custm);
+    elements = simplify_lrtvalues(ss,custm);
 
     *zeros = 0;
     
@@ -608,31 +607,51 @@ set_test_param (MYREAL *param, lr_data_fmt *lrtdata, world_fmt * world, long lrl
         }
         
     }
-for (zi = world->numpop; zi < world->numpop2; zi++)
-{
-    if (paramtype[zi] == '+')
+    for (zi = world->numpop; zi < world->numpop2; zi++)
       {
-        zz = (zi - world->numpop) / (world->numpop - 1);
-        if(!world->options->usem)
-            param[zi] /= param[zz];
+	if (paramtype[zi] == '+')
+	  {
+	    zz = (zi - world->numpop) / (world->numpop - 1);
+	    if(!world->options->usem)
+	      param[zi] /= param[zz];
+	  }
       }
-}
-free (paramtype);
-free (ss);
-free(custm);
-return df;
+    myfree(paramtype);
+    myfree(ss);
+    free_charvec2d(custm);
+    return df;
 }
 
 
+long simplify_lrtvalues(char *in, char **out)
+{
+  char *ptr;
+  long elements=0;
+  while(in != NULL)
+    {
+      
+      ptr = strsep(&in,"\n\r\t; {,}");
+      if(strlen(ptr)>0)
+	{
+	  if(ptr[0]=='0' && ((atof(ptr) - SMALLEPSILON) <= 0.0))
+	    sprintf(out[elements++], "0");
+	  else
+	    sprintf(out[elements++],"%s", ptr);
+	}
+    }
+  return elements;
+}
 
 // in is the string from the l-ratio option, custom is a database array that
 // gets allocated while its filled
 // maximal event numbers is LONGLINESIZE [search for use of simplify_lrtvalues()]
-long simplify_lrtvalues(char * in, char*** out)
+long simplify_lrtvalues_old(char * in, char*** out)
 {
     long elements=0;
     long count = 0;
-    (*out)[0] = (char *) mycalloc(LONGLINESIZE,sizeof(char));
+    boolean white=FALSE;
+    boolean element_ready = TRUE;
+    //    (*out)[0] = (char *) mycalloc(LONGLINESIZE,sizeof(char));
     while(*in!='\0')
       {
         switch(*in)
@@ -641,20 +660,53 @@ long simplify_lrtvalues(char * in, char*** out)
             case '\n':
             case '\t':
             case ' ':
-                break;
             case ',':
             case ';':
-                count=0;
-                elements++;
-                (*out)[elements] = (char *) mycalloc(LONGLINESIZE,sizeof(char));
-                break;
-            default:
-                (*out)[elements][count++] = *in;
-                break;
+	      white=TRUE;
+	      break; // jump over "whitespace", 
+	      // if elements are from the list {*, m, M, s, S} read one character and then restart a new element
+	  case 't':
+	  case 'm':
+	  case 'M':
+	  case 's':
+	  case 'S':
+	  case '*':
+	  case 'x':
+	    white = FALSE;
+	    // fill element and reset counter within element
+	    count=0;
+	    (*out)[elements][0] = *in;
+	    // make new element ready
+	    elements++;
+	    //    (*out)[elements] = (char *) mycalloc(LONGLINESIZE,sizeof(char));
+	    element_ready = TRUE;
+	    break;
+	  default:
+	    if(!element_ready)
+	      {
+		(*out)[elements][count++] = *in;
+	      }
+	    else
+	      {
+		if(white)
+		  {
+		    count=0;
+		    elements++;
+		    //    (*out)[elements] = (char *) mycalloc(LONGLINESIZE,sizeof(char));
+		    element_ready = FALSE;
+		    (*out)[elements][count++] = *in;
+		  }
+		else
+		  {
+		    (*out)[elements][count++] = *in;
+		  }
+	      }
+	    white = FALSE;
+	    break;
           }
         in++;
     }
-    *out = (char**) myrealloc(*out, elements * sizeof(char*));
+    //*out = (char**) myrealloc(*out, elements * sizeof(char*));
     return elements;
 }
 
@@ -865,10 +917,10 @@ set_test_param (MYREAL *param, lr_data_fmt *lrtdata, world_fmt * world, long lrl
                 param[zi] /= param[pop];
         }
     }
-    free (paramtype);
-    free (ss);
-    free (tmp);
-    free(custm);
+    myfree(paramtype);
+    myfree(ss);
+    myfree(tmp);
+    myfree(custm);
     return df;
 }
 
