@@ -11,14 +11,15 @@
  
  
  Peter Beerli 1996, Seattle
- beerli@csit.fsu.edu
+ beerli@scs.fsu.edu
  
 Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
 Copyright 2003-2004 Peter Beerli, Tallahassee FL
  
-$Id: mcmc.h 175 2005-12-02 14:51:23Z beerli $
+$Id: mcmc.h 514 2006-11-09 20:14:08Z beerli $
 -------------------------------------------------------*/
 
 extern long tree_update (world_fmt * world, long g);
 extern void free_timevector (timelist_fmt * timevector);
+extern void count_migrations (node * p, long *count);
 #endif
