@@ -9,7 +9,7 @@
  
  
  Peter Beerli 1996, Seattle
- beerli@csit.fsu.edu
+ beerli@scs.fsu.edu
  
 Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
 Copyright 2003-2004 Peter Beerli, Tallahassee FL
@@ -18,7 +18,7 @@ Copyright 2003-2004 Peter Beerli, Tallahassee FL
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
-$Id: mcmc2.c 213 2005-12-29 20:27:05Z beerli $
+$Id: mcmc2.c 733 2007-05-31 02:28:34Z beerli $
 -------------------------------------------------------*/
 /* \file mcmc2.c
 Contains low-level routines to manipulate the treechanger
@@ -89,6 +89,7 @@ void
 coalesce1p (proposal_fmt * proposal)
 {
     node *obb = NULL;
+    //traverse_check(crawlback (proposal->root->next));
     erase_migr_nodes (proposal->origin);
     if (proposal->migr_table_counter2 > 0) //|| proposal->mig_removed)
         erase_migr_nodes (proposal->realtarget);
@@ -176,6 +177,7 @@ coalesce1p (proposal_fmt * proposal)
     }
     /*    if (proposal->oback == crawlback(proposal->root->next)) */
     fix_root_pop (crawlback (proposal->root->next));
+    //traverse_check(crawlback (proposal->root->next));
 }
 
 
@@ -520,7 +522,7 @@ target_stancoa (proposal_fmt * proposal)
 {
     node *obb, *nn, *oldnn, *d1, *d2, *proposalack, *uppern = NULL; /* o R */
     node **double_line;
-        double_line = (node **) mycalloc (2, sizeof (node *));
+    double_line = (node **) mycalloc (2, sizeof (node *));
     if (proposal->ocousin != NULL)
     {
         obb = showtop (crawlback (proposal->oback));
@@ -629,7 +631,7 @@ target_stancoa (proposal_fmt * proposal)
             }
         }
     }
-  free (double_line);
+  myfree(double_line);
 }
 
 void
@@ -783,8 +785,8 @@ erase_migr_nodes (node * up)
         theNode = theNode->next->back;
         if (oldNode->type == 'm')
         {
-            free (oldNode->next);
-            free (oldNode);
+            myfree(oldNode->next);
+            myfree(oldNode);
             deleted++;
         }
     }
@@ -867,6 +869,7 @@ gotoroot (node * origin, node ** ptrlist)
         ptrlist[i++] = theNode;
     }
     ptrlist[i] = theNode;  /* adds root->back to the list */
+    ptrlist[i+1] = NULL;   /* guarantees an end for the array)*/
 }
 
 void
@@ -889,7 +892,9 @@ adjust (node * theNode, MYREAL tyme, long level)
     }
     else if (theNode->type == 't')
     {
-        theNode->tyme = 0.0;
+#ifndef TESTINGDATE
+      theNode->tyme = 0.0;
+#endif
         theNode->length = lengthof (theNode);
         ltov (theNode);
         return;
@@ -945,36 +950,42 @@ localevaluate (node * mother)
     }
 }
 
+
+///
+/// copy the sequence data
 void
 copy_x (proposal_fmt * proposal, xarray_fmt xx1, xarray_fmt xx2)
 {
+    long locus        = proposal->world->locus;
+    long endsite      = proposal->endsite;
+    long rcategs      = proposal->world->options->rcategs;
+    long *maxalleles = proposal->world->data->maxalleles;
+    
 #ifdef ALTIVEC            
-    const size_t sitelikesize = sizeof (FloatVec);
+    const size_t sitelikesize = sizeof (FloatVec) * rcategs;
 #else
-    const size_t sitelikesize = sizeof (sitelike);//*proposal->world->options->rcategs;
+    const size_t sitelikesize = sizeof (sitelike) * rcategs;
 #endif    
         long i;
-        long j;
+	//        long j;
 
     switch (proposal->datatype)
     {
     case 'a':
     case 'b':
     case 'm':
-        memcpy (xx1.a, xx2.a,
-                sizeof (MYREAL) *
-                proposal->world->data->maxalleles[proposal->world->locus]);
+        memcpy (xx1.a, xx2.a, sizeof (MYREAL) * maxalleles[locus]);
         break;
     case 's':
     case 'n':
     case 'u':
     case 'f':
-        for (i = 0; i < proposal->endsite; i++)
+        for (i = 0; i < endsite; i++)
         {
-            for (j = 0; j < proposal->world->options->rcategs; j++)
-            {
-                memcpy (xx1.s[i], xx2.s[i], sitelikesize*proposal->world->options->rcategs);
-            }
+            //for (j = 0; j < rcategs; j++)
+            //{
+                memcpy (xx1.s[i], xx2.s[i], sitelikesize);
+            //}
         }
         break;
     }
@@ -1035,7 +1046,7 @@ findcrossing (node ** ptrl1, node ** ptrl2)
     /* assumes that there is an NULL element at the end */
     for (i = 0; ptrl1[i] != NULL; j = 0, i++)
     {
-        while ((ptrl1[i] != ptrl2[j]) && (ptrl2[j] != NULL))
+        while ((ptrl2[j] != NULL) && (ptrl1[i] != ptrl2[j]))
             j++;
         if (ptrl2[j] != NULL)
         {
