@@ -12,6 +12,12 @@
 
 #include "bayes.h"
 
+#define MYINT  int
+#define REPTAG 90
+#define PRINTTAG 800
+#define TEMPTAG 7000
+#define BURNTAG 6000
+
 typedef struct _filedb_fmt {
 	FILE *file;
 	long handle;
@@ -42,6 +48,8 @@ extern int profiledone;
 #define MIGMPI_SUMFILE 4
 #define MIGMPI_MIGHIST 5
 #define MIGMPI_BAYESHIST 6
+#define MIGMPI_SKYLINE 7
+#define MIGMPI_TREESPACE 8
 #ifdef MPI
 
 #ifndef PTHREADS
@@ -67,35 +75,44 @@ extern void broadcast_options_worker (option_fmt * options);
 
 extern void broadcast_data_master (data_fmt * data, option_fmt * options);
 extern void broadcast_data_worker (data_fmt * data, option_fmt * options);
-extern void mpi_results_master (MYREAL sendtype, world_fmt * world,
+extern void mpi_results_master (long sendtype, world_fmt * world,
                                     long maxrep,
-                                    void (*unpack) (char *buffer,
+                                    void (*unpack) (MYREAL *buffer,
                                                     world_fmt * world, long locus,
                                                     long maxrep, long numpop));
 extern void mpi_results_worker (long bufsize, world_fmt * world, long maxrep,
-                                    long (*pack) (char **buffer,
+                                    long (*pack) (MYREAL **buffer,
                                                   world_fmt * world, long locus,
                                                   long maxrep, long numpop));
 
-extern long pack_result_buffer (char **buffer, world_fmt * world,
+extern long pack_result_buffer (MYREAL **buffer, world_fmt * world,
                                     long locus, long maxrep, long numpop);
-extern void unpack_result_buffer (char *buffer, world_fmt * world,
+extern void unpack_result_buffer (MYREAL *buffer, world_fmt * world,
                                       long locus, long maxrep, long numpop);
-extern long pack_sumfile_buffer (char **buffer, world_fmt * world,
+extern long pack_sumfile_buffer (MYREAL **buffer, world_fmt * world,
                                      long locus, long maxrep, long numpop);
-extern void unpack_sumfile_buffer (char *buffer, world_fmt * world,
+extern void unpack_sumfile_buffer (MYREAL *buffer, world_fmt * world,
                                        long locus, long maxrep, long numpop);
-extern long pack_mighist_buffer (char **buffer, world_fmt * world,
+extern long pack_treespace_buffer (MYREAL **buffer, world_fmt * world,
+				   long locus, long maxrep, long numpop);
+extern void unpack_treespace_buffer (MYREAL *buffer, world_fmt * world,
+				     long locus, long maxrep, long numpop);
+
+extern long pack_mighist_buffer (MYREAL **buffer, world_fmt * world,
                                      long locus, long maxrep, long numpop);
-extern void unpack_mighist_buffer (char *buffer, world_fmt * world,
+extern void unpack_mighist_buffer (MYREAL *buffer, world_fmt * world,
                                        long locus, long maxrep, long numpop);
+extern long pack_skyline_buffer (MYREAL **buffer, world_fmt * world,
+				 long locus, long maxrep, long numpop);
+extern void unpack_skyline_buffer (MYREAL *buffer, world_fmt * world,
+				   long locus, long maxrep, long numpop);
 
-extern void unpack_single_bayes_buffer(char *buffer,bayes_fmt * bayes, world_fmt * world,long locus);
+extern long unpack_single_bayes_buffer(MYREAL *buffer,bayes_fmt * bayes, world_fmt * world,long locus);
 
-extern long pack_bayes_buffer (char **buffer, world_fmt * world,
+extern long pack_bayes_buffer (MYREAL **buffer, world_fmt * world,
                                long locus, long maxrep, long numpop);
 
-extern void unpack_bayes_buffer (char *buffer, world_fmt * world,
+extern void unpack_bayes_buffer (MYREAL *buffer, world_fmt * world,
                                  long locus, long maxrep, long numpop);
 
 extern void mpi_gradient_master (nr_fmt * nr, world_fmt * world, int *who);
@@ -122,9 +139,9 @@ extern long get_filehandle2(FILE *file, world_fmt *world);
 #ifdef SLOWNET
 extern void
 mpi_broadcast_results (world_fmt * world, long loci,
-                       long (*pack) (char **buffer, world_fmt * world,
+                       long (*pack) (MYREAL **buffer, world_fmt * world,
                                      long locus, long maxrep, long numpop),
-                       void (*unpack) (char *buffer, world_fmt * world,
+                       void (*unpack) (MYREAL *buffer, world_fmt * world,
                                        long locus, long maxrep, long numpop));
 extern void mpi_profiles_master (world_fmt * world, long nparam, int *profilewho);
 extern void mpi_profiles_worker (world_fmt * world, long *gmaxptr);
