@@ -12,11 +12,11 @@
   Department of Biological Sciences
   Florida State University
   Tallahassee FL 32306-4120
-  beerli@csit.fsu.edu
+  beerli@scs.fsu.edu
  
   Copyright 2002 Peter Beerli and Joseph Felsenstein, Seattle WA
   Copyright 2003 Peter Beerli, Tallahassee FL
-  $Id: migration.h 215 2006-01-03 00:03:51Z beerli $
+  $Id: migration.h 756 2007-06-27 02:45:26Z beerli $
   *----------------------------------------------------------------
 */
 /**
@@ -36,7 +36,10 @@ typedef char allele_type[DEFAULT_ALLELENMLENGTH]; //!< allele type string
 
 typedef long twin_fmt[2];
 typedef long quad_fmt[4];
-typedef MYREAL pair[2]; 
+typedef MYREAL pair[2];
+typedef long longpair[2];
+typedef float tetra[5];
+typedef float duo[2];
 #ifdef ALTIVEC
 typedef FloatVec sitelike;
 #else
@@ -50,6 +53,25 @@ typedef short seqval[MAXCATEGS];
 
 typedef char allele_fmt[DEFAULT_ALLELENMLENGTH];
 
+/// tipdate format
+typedef struct tipdate_fmt 
+{
+  MYREAL date;
+  long   id;
+} 
+tipdate_fmt;
+
+/// records when the automatic stop of burnin happened
+typedef struct _burnin_record_fmt
+{
+  long locus;
+  long replicate;
+  long stopstep;
+  MYREAL variance;
+  MYREAL oldvariance;
+  long worker;
+} burnin_record_fmt;
+
 /// likelihood ratio test data structure
 typedef struct lr_data_fmt
 {
@@ -138,6 +160,8 @@ typedef struct _data
     FILE *sumfile;          //!< intermediate output summary file
     FILE *distfile;         //!< distance file among individuals, instead of treefile
     FILE *geofile;          //!< geographic distance among sampling locations
+    FILE *divfile;          //!< fixed divergence time among populations
+    FILE *datefile;         //!< sample date for each individual
 #ifdef UEP
     FILE *uepfile;          //!< unique event polymorphism file
     int **uep;              //!< uep data holder
@@ -155,6 +179,8 @@ typedef struct _data
     char **popnames;        //!< array of the population names
     long **numind;          //!< array of the number of individuals
     long **numalleles;      //
+  tipdate_fmt ***sampledates;
+  MYREAL maxsampledate;
     char ***indnames;       //!< array of individual names per population
     long numpop;            //!< number of populations
     long loci;              //!< number of loci
@@ -197,24 +223,30 @@ typedef struct _bayes
 {
   long count;
   long numpop2;
+  longpair *map; //mapping of parameter using custom migration matrix
   MYREAL *datastore; //data store for
   // pointers into datastore
   MYREAL *priormean; // holds the means of priors
+  MYREAL *priorstd; // holds the standard deviation of priors [not always filled]
+  MYREAL *prioralpha; // holds alpha of prior distribution [not always filled]
   MYREAL *delta; // change scale [how big can this be
   MYREAL *minparam;// minimal allowed parameter value
   MYREAL *maxparam; // maximal allowed parameter value
   MYREAL *meanparam; // mean values for expprior
+  MYREAL *alphaparam; // alpha value for gamma
   // record all changes of parameters
-    MYREAL **params;  // save for parameter vectors
-    long allocparams; //number of allocated parameter vectors
-    long numparams;  //number of saved parameter vectors
-    long paramnum;  // which param one is working with
-    MYREAL oldval;    //holds prob(G|param)
-    long *datastore2; // datastore for long integer variables
-    // pointers into datastore2
+  MYREAL *params;  // save for parameter vectors
+  long allocparams; //number of allocated parameter vectors
+  long numparams;  //number of saved parameter vectors
+  long paramnum;  // which param one is working with
+  MYREAL starttime; // start time that was used to change the lineage on tree
+  MYREAL stoptime; // stop time that was used to change the lineage on the tree
+  MYREAL oldval;    //holds prob(G|param)
+  long *datastore2; // datastore for long integer variables
+  // pointers into datastore2
   long *accept;  //holds numbers of accepted changes for each parameter AND the tree
   long *trials; //holds how many time was tried to change a parameter
-    // holds histogram for each locus and summary statistics
+  // holds histogram for each locus and summary statistics
   bayeshistogram_fmt *histogram;
   MYREAL *deltahist; // delta for all histograms per parameters [important for multilocus case 
   char * custm2; //pointer to the world->options->custm2;
@@ -227,11 +259,12 @@ bayes_fmt;
 /// storage for bayesian prior related parameters
 typedef struct prior_fmt
 {
-    MYREAL min;
-    MYREAL mean;
-    MYREAL max;
-    MYREAL delta;
-    long   bins;
+  MYREAL min;
+  MYREAL mean;
+  MYREAL std;
+  MYREAL max;
+  MYREAL delta;
+  long   bins;
 } prior_fmt;
 
 
@@ -298,34 +331,41 @@ typedef struct _option
     boolean simulation;
     boolean mighist;
   boolean mighist_all;
-    boolean mixplot;
-    boolean dist;
-    boolean geo;
-    char *parmfilename;
-    char *infilename;
-    char *outfilename;
+  long mighist_counter;
+  long mighist_increment;
+  boolean skyline;
+  float eventbinsize;
+  boolean mixplot;
+  boolean dist;
+  boolean geo;
+  boolean div;
+  char *parmfilename;
+  char *infilename;
+  char *outfilename;
   char *pdfoutfilename;
-    char *logfilename;
-    char *mathfilename;
-    char *treefilename;
-    char *utreefilename;
-    char *catfilename;
-    char *weightfilename;
-    char *sumfilename;
-    char *mighistfilename;
-    char *distfilename;
-    char *geofilename;
-    char *bootfilename;
-    char *seedfilename;
-    char *mixfilename;
+  char *logfilename;
+  char *mathfilename;
+  char *treefilename;
+  char *utreefilename;
+  char *catfilename;
+  char *weightfilename;
+  char *sumfilename;
+  char *mighistfilename;
+  char *skylinefilename;
+  char *distfilename;
+  char *geofilename;
+  char *divfilename;
+  char *bootfilename;
+  char *seedfilename;
+  char *mixfilename;
 #ifdef UEP
-
-    char *uepfilename;
+  char *uepfilename;
 #endif
-
-    char *bayesfilename;
+  char *datefilename;
+  char *bayesfilename;
   char *bayesmdimfilename;
 
+
     char title[LINESIZE + 1];
     lratio_fmt *lratio;
     boolean aic;
@@ -362,6 +402,8 @@ typedef struct _option
     long rcategs;
     MYREAL *rrate;
     MYREAL *probcat;
+  long seqrate_gamma_num; /*number of loci for gamma values*/
+  MYREAL *seqrate_gamma; /* for gamma values for many loci*/
     MYREAL probsum;
     boolean gammarates;
     boolean autocorr;
@@ -377,6 +419,8 @@ typedef struct _option
     long saveseed;
     /* mcmc options */
     boolean bayes_infer;
+  boolean integrated_like;
+  boolean slice_sampling[PRIOR_SIZE];
     MYREAL updateratio;
     int bayesprior[3];
   int bayespretty;
@@ -391,47 +435,55 @@ typedef struct _option
     short migrguess;
     boolean gamma;
   boolean murates;
+  boolean murates_fromdata;
   boolean bayesmurates;
     MYREAL alphavalue;
     MYREAL *mu_rates;
     long muloci;
+  boolean has_datefile;
 #ifdef LONGSUM
 
     boolean fluctuate;
     MYREAL *flucrates;
 #endif
+  long burn_in;
+  char burnin_autostop;
+  short heating;
+  MYREAL *thetag;
+  long numthetag;
+  MYREAL *mg;
+  long nummg;
+  long schains;
+  long sincrement;
+  long ssteps;
+  long lchains;
+  long lincrement;
+  long lsteps;
+  MYREAL lcepsilon;
+  boolean gelman;
+  boolean gelmanpairs;
+  long pluschain;  //how many chains are allowed , currently HARDCODED
+  boolean replicate;
+  long replicatenum;
+  long gridpoints;
+  long numpop;
+  MYREAL heat[50];
+  boolean adaptiveheat;
+  long heating_interval;
+  long heated_chains;
+  /* save genealogy summary options */
+  boolean readsum;
+  boolean writesum;
+  /* threading over loci */
+  int cpu;
+  //
+  // fatal attraction of zero resistance
+  MYREAL minmigsumstat;
+  //
+  MYREAL *mutationrate_year; // keeps mutation rate per year for each locus 
+  long mutationrate_year_numalloc; // 
+  MYREAL generation_year;
 
-    long burn_in;
-    short heating;
-    MYREAL *thetag;
-    long numthetag;
-    MYREAL *mg;
-    long nummg;
-    long schains;
-    long sincrement;
-    long ssteps;
-    long lchains;
-    long lincrement;
-    long lsteps;
-    MYREAL lcepsilon;
-    boolean gelman;
-    long pluschain;  //how many chains are allowed , currently HARDCODED
-    boolean replicate;
-    long replicatenum;
-    long gridpoints;
-    long numpop;
-    MYREAL heat[50];
-    boolean adaptiveheat;
-    long heating_interval;
-    long heated_chains;
-    /* save genealogy summary options */
-    boolean readsum;
-    boolean writesum;
-    /* threading over loci */
-    int cpu;
-    //
-    // fatal attraction of zero resistance
-    MYREAL minmigsumstat;
 }
 option_fmt;
 
@@ -500,7 +552,8 @@ typedef struct timelist_fmt
     long allocT;
     long T;
     long oldT;
-    vtlist *tl;
+  vtlist *tl;
+  long *lineages;
 }
 timelist_fmt;
 
@@ -599,25 +652,43 @@ quantile_fmt;
 
 /// format for migration events
 /// holds time, from, to, and sumtips
-typedef MYREAL migevent_fmt[4];
+///typedef MYREAL migevent_fmt[4];
+typedef struct _migevent_fmt
+{
+  float age;
+  int from;
+  int to;
+  long sumlines;
+} migevent_fmt;
 
 /// holds migration events for migration event time histogram
 typedef struct _mighist_fmt
 {
-    long allocsize;
-    long copies;
-    long weight;
-    long migeventsize;
-    migevent_fmt *migevents;
+  long allocsize;
+  long copies;
+  long weight;
+  long migeventsize;
+  migevent_fmt *migevents;
 }
 mighist_fmt;
 
 /// holds all migration histograms for all loci
 typedef struct _mighistloci_fmt
 {
-    mighist_fmt *mighist;
-    long mighistnum;
-    long allocsize;
+  // records migration events into mighist container
+  mighist_fmt *mighist;
+  long mighistnum;
+  long allocsize;
+
+  // records all events  parameters per bin derived from timelist
+  // should make communication over MPI much smaller
+  duo **migeventbins;
+  long *migeventbinnum;
+  // records all expected  parameters per bin derived from timelist
+  tetra **eventbins; 
+  long *eventbinnum;
+  MYREAL eventbinsize; // used for eventbins and migeventbins
+
 }
 mighistloci_fmt;
 
@@ -636,9 +707,11 @@ typedef struct _worldoption
     boolean gamma;
     MYREAL alphavalue;
     boolean murates;
+  boolean murates_fromdata;
     long muloci;
     MYREAL *mu_rates;
     MYREAL *lmu_rates;
+  MYREAL *meanmu;
 #ifdef LONGSUM
 
     boolean fluctuate;
@@ -668,6 +741,7 @@ typedef struct _worldoption
     boolean verbose;
     boolean replicate;
     boolean gelman;
+  boolean gelmanpairs;
     MYREAL lcepsilon;
     boolean simulation;
     char datatype;
@@ -690,16 +764,21 @@ typedef struct _worldoption
     long df;
     long lchains;
     long replicatenum;
-    long micro_threshold;
+    long *micro_threshold;
     long micro_stepnum;
-    MYREAL **steps;
+    MYREAL ***steps;
     MYREAL *rrate;
     MYREAL *rate;
     MYREAL *probcat;
     long pluschain;
     boolean mighist;
   boolean mighist_all;
+  long mighist_counter;
+  long mighist_increment;
+  boolean skyline;
+  float eventbinsize;
     long burn_in;
+  char burnin_autostop;
     boolean usem;
     short migvar;
     boolean plot;
@@ -730,11 +809,17 @@ typedef struct _worldoption
 
     boolean fastlike;
     boolean bayes_infer;
+  boolean slice_sampling[PRIOR_SIZE];
   MYREAL updateratio;
   boolean has_bayesfile;
   boolean has_bayesmdimfile;
   long bayesmdiminterval;
     MYREAL minmigsumstat;
+  boolean has_datefile;
+  MYREAL *mutationrate_year; // keeps mutation rate per year for each locus 
+  long mutationrate_year_numalloc; // 
+  MYREAL generation_year;
+  boolean treeinmemory;
 }
 worldoption_fmt;
 
@@ -751,14 +836,17 @@ ueptime_fmt;
 /// reduced set of parameters related to the data used to run MCMC
 typedef struct _worlddata
 {
-    boolean *skiploci;
-    MYREAL *geo;
-    MYREAL *lgeo;
-    long *maxalleles;
-    seqmodel_fmt *seq;
-    FILE *sumfile;
-    MYREAL freq;
-    MYREAL freqlast;
+  boolean *skiploci;
+  MYREAL *geo;
+  MYREAL *lgeo;
+  long *maxalleles;
+  seqmodel_fmt *seq;
+  FILE *sumfile;
+  MYREAL freq;
+  MYREAL freqlast;
+  tipdate_fmt ***sampledates;
+  MYREAL maxsampledate;
+  long **numind;
 #ifdef UEP
 
     long uepsites;
@@ -769,88 +857,110 @@ worlddata_fmt;
 
 //typedef struct _proposal_fmt proposal_fmt;
 
+typedef struct _convergence
+{
+  MYREAL *gelmanmeanmaxR;
+  MYREAL gelmanmeanRall;
+  MYREAL gelmanmaxRall;
+  MYREAL *chain_s;
+  MYREAL *chain_means;
+  long *chain_counts;
+} convergence_fmt;
+
+#ifdef MPI
+typedef struct _mpirequest_fmt 
+{
+  char *tempstr;
+  int sender;
+  int tag;
+} mpirequest_fmt;
+#endif
 /// holds all the parameters etc to run the program
 typedef struct _world
 {
-    /* generalities */
-    char *name;
-    char *worldname;
-    worldoption_fmt *options;
-    worlddata_fmt *data;
-
-    //proposal_fmt  *proposal;  
+  /* generalities */
+  char *name;
+  char *worldname;
+  worldoption_fmt *options;
+  worlddata_fmt *data;
+  
 #ifdef MPI
-
-    int *who;
+  int *who;
+  int *mpistack;
+  mpirequest_fmt *mpistack_request;
+  long mpistack_requestnum;
+  long mpistacknum;
 #ifdef SLOWNET
-
-    int *profilewho;
+  
+  int *profilewho;
 #endif
 #endif
-
-    char *buffer;   // buffer for profiles, not needed before profiletables()
-    long loci;
-    long skipped;   /*loci with no data */
-    long locus;   /* the current locus, if single then set to 0 */
-    long thislocus;  /* the real current locus, the whole locus scheme
-               needs revision */
-    long numpop;
-    long numpop2;
-    long sumtips;
-    /* migration parameter array  starts and ends */
-    int *mstart;
-    int *mend;
-
-    /* time archives, contains the data/results for summarizing */
-    timearchive_fmt **atl;
-
-    /* migration histogram reporter */
-    mighistloci_fmt *mighistloci;
-    long mighistlocinum;
-
-    /*tree material */
-    node **nodep;
-    node *root;
-    MYREAL treelen;
-    long unique_id;
-    tbl_fmt tbl;
-    contribarr *contribution;
-    /* parameter */
-    MYREAL *param0;
-    MYREAL *param00;
-    MYREAL **fstparam;
-    /*allows for 3 different rate changes at specific times*/
+  
+  char *buffer;   // buffer for profiles, not needed before profiletables()
+  long loci;
+  long skipped;   /*loci with no data */
+  long locus;   /* the current locus, if single then set to 0 */
+  long thislocus;  /* the real current locus, the whole locus scheme
+			needs revision */
+  long numpop;
+  long numpop2;
+  long sumtips;
+  /* migration parameter array  starts and ends */
+  int *mstart;
+  int *mend;
+  
+  /* time archives, contains the data/results for summarizing */
+  timearchive_fmt **atl;
+  
+  /* migration histogram reporter */
+  mighistloci_fmt *mighistloci;
+  long mighistlocinum;
+  
+  /*tree material */
+  node **nodep;
+  node *root;
+  MYREAL treelen;
+  long unique_id;
+  tbl_fmt tbl;
+  contribarr *contribution;
+  /* parameter */
+  MYREAL *param0;
+  MYREAL *param00;
+  MYREAL **fstparam;
+  /*allows for 3 different rate changes at specific times*/
 #ifdef LONGSUM
-
-    MYREAL *flucrates;
-    MYREAL *lflucrates;
+  MYREAL *flucrates;
+  MYREAL *lflucrates;
 #endif
-    /* mcmc related */
-    long *lineages;
-    timelist_fmt *treetimes;
-    MYREAL *mig0list;
-    MYREAL **migproblist;
-    long *design0list;
-
-    /*nr related */
-    MYREAL ***apg0;  /* part-loglikelihoods of param0 */
-    MYREAL ***apg;  /* part-loglikelihoods of param */
-
-    /*heating scheme */
-    long actualinc;
-    long increment;
-    //  MYREAL heatratio;
+  /* mcmc related */
+  long *lineages;
+  timelist_fmt *treetimes;
+  MYREAL *mig0list;
+  MYREAL **migproblist;
+  long *design0list;
+  
+  /*nr related */
+  MYREAL ***apg0;  /* part-loglikelihoods of param0 */
+  MYREAL ***apg;  /* part-loglikelihoods of param */
+  
+  /*heating scheme */
+  long actualinc;
+  long increment;
+  //  MYREAL heatratio;
+  boolean cold; /* true for cold chain, false for all others*/
     MYREAL heat;
     MYREAL averageheat;
+  MYREAL varheat;
     long treeswapcount;
     /* reporting time */
     time_t starttime;
     long treesdone;
     long treestotal;
-
+  boolean in_burnin;
+  burnin_record_fmt *burnin_stops;
     /* gelman R reporting/checking "gelman-convergence option" */
-    MYREAL gelmanmeanR;
-    MYREAL gelmanmaxR;
+  convergence_fmt *convergence;
+
     long chains;
     boolean start;  //I need this in estimateParameter()
     /* reporting */
@@ -860,6 +970,7 @@ typedef struct _world
     plotmax_fmt **plotmax;
     quantile_fmt *quantiles;
     boolean **percentile_failed; /*indicator whether the percentile calculation failed*/
+    boolean percentile_some_failed; /*some precentiles failed*/
     MYREAL maxdatallike;  /* the maximum log likelihood of a chain */
     MYREAL allikemax;  /* the maximum log likelihood  the best tree */
     boolean in_last_chain; /*for last CHAIN option in print-tree */
@@ -890,16 +1001,30 @@ typedef struct _world
     MYREAL accept_freq;
     long swapped;
     long G;
-	long bayesaccept;
-    bayes_fmt *bayes;
-    FILE *bayesfile;
+  long bayesaccept;
+  bayes_fmt *bayes;
+  FILE *bayesfile;
   FILE *bayesmdimfile;
-    FILE *outfile;
+  FILE *outfile;
   FILE *pdfoutfile;
-    FILE *treefile;
-    FILE *mathfile;
-    FILE *mighistfile;
-    FILE *bootfile;
+  FILE *treefile;
+  FILE *mathfile;
+  FILE *mighistfile;
+  FILE *skylinefile;
+  char **treespace;
+  long *treespacenum;
+  long *treespacealloc;
+  MYREAL *besttreelike;
+  // Bayes factor
+  MYREAL *bf;
+  MYREAL bfscale;
+  MYREAL hm;
+  MYREAL am;
+  // autocorrelation and ESS
+  MYREAL *autocorrelation;
+  MYREAL *effective_sample;
+  MYREAL *auto_archive;
+  MYREAL *ess_archive;
 }
 world_fmt;
 
@@ -939,6 +1064,8 @@ typedef struct _nr_fmt
     MYREAL ***apg0;  /* part-loglikelihoods of param0 */
     MYREAL ***apg;  /* part-loglikelihoods of param */
     MYREAL *apg_max;  /* maxvalue of apg */
+  long seqrate_gamma_num; /*number of loci for gamma values*/
+  MYREAL *seqrate_gamma; /* for gamma values for many loci*/
     MYREAL *rate;   /* rates for gamma rates */
     MYREAL *probcat;  /* probabilities for gamma rates */
     long categs;   /* #categories for gamma rates */
@@ -1116,7 +1243,7 @@ typedef struct _plotfield_fmt
     char yaxis[255];  //yaxis label
     char yfaxis[255];  //frequency yaxis label
     char title[255];
-    MYREAL *yfreq;
+    float *yfreq;
     long *y;
     char **data;   //the plotplane
 }
@@ -1135,7 +1262,9 @@ extern void (*setup_param0) (world_fmt *, nr_fmt *, long, long, long, long,
                                  long, boolean);
 
 #endif
-
+//#ifdef MEMDEBUG
+//#include <memcheck.h>
+//#endif
 #include "sort.h"
 //#include "migrate_mpi.h"
 #endif
