@@ -6,7 +6,7 @@
  P R O F I L E    L I K E L I H O O D    R O U T I N E S 
  
  Peter Beerli 1997, Seattle
- beerli@csit.fsu.edu
+ beerli@scs.fsu.edu
  
 Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
 Copyright 2003-2004 Peter Beerli, Tallahassee FL
@@ -16,7 +16,7 @@ Copyright 2003-2004 Peter Beerli, Tallahassee FL
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: profile.c 175 2005-12-02 14:51:23Z beerli $
+$Id: profile.c 753 2007-06-15 18:17:57Z beerli $
  
 -------------------------------------------------------*/
 /*! \file profile.c
@@ -36,6 +36,11 @@ Calculation of profile likelihoods and printing of profile likelihood tables
 
 #include "migrate_mpi.h"
 
+#include "profile.h"
+#ifdef PRETTY
+#include "pretty.h"
+#endif
+
 #ifdef DMALLOC_FUNC_CHECK
 #include <dmalloc.h>
 #endif
@@ -52,7 +57,7 @@ void calc_profile_likelihood (char method, long which, MYREAL *likes,
 void print_profile_likelihood (long which, world_fmt * world, long *gmaxptr);
 void prepare_header (char *var, long which, world_fmt * world);
 void print_profile_title (world_fmt * world);
-void print_profile_table (char method, long which, char *var, MYREAL likes[],
+long print_profile_table (char method, long which, char *var, MYREAL likes[],
                           MYREAL **param, world_fmt * world);
 void print_profile_percentile (world_fmt * world);
 void sprint_percentile_header (char *outfile, boolean first);
@@ -113,16 +118,6 @@ void calc_grid(FILE *alldim, world_fmt * world, MYREAL *testparam, MYREAL *ltest
 #endif
 
 
-#define GRIDSIZE 9
-#define GRIDMIDDLE 5
-#define GRID    {0.01,0.05,0.10,0.50,0.99,0.95,0.90,0.50,1.0}
-#define SHOWGRID {0.005,0.025,0.05,0.25,0.995,0.975,0.95,0.75,0.50}
-#define GRID2   {0.005,0.025,0.05,0.25,0.5, 0.75,0.95,0.975,0.995}
-#define INDEX {0,1,2,3,8,7,6,5,4}
-#define DEVIATE {0.02,0.10,0.20, 0.5, 50.,10., 5., 2., 1.}
-#define DEVIATE2 {0.02,0.10,0.20, 0.5, 1., 2., 5., 10., 50. }
-//#define DEVIATE {0.002,0.010,0.020, 0.05, 5000.,1000., 500., 200., 1.}
-#define ABSOLUTE {1e-100,1e100}
 
 boolean
 print_profile_likelihood_driver (long which, world_fmt * world, long *gmaxptr)
@@ -206,6 +201,7 @@ void
 print_profile_likelihood (long which, world_fmt * world, long *gmaxptr)
 {
     long i, ii;
+    long bufsize = 0;
     nr_fmt *nr;
     char *var;
     char method = world->options->profilemethod;
@@ -222,25 +218,30 @@ print_profile_likelihood (long which, world_fmt * world, long *gmaxptr)
     create_nr (nr, world, *gmaxptr, which, world->loci,
                world->repkind, world->rep);
     
-    var = (char *) calloc(LINESIZE, sizeof(char));
+    var = (char *) mycalloc(LINESIZE, sizeof(char));
     doublevec1d(&likes, GRIDSIZE);
     //doublevec2d(&param,GRIDSIZE, elem);
-    param = (MYREAL **) calloc(GRIDSIZE,sizeof(MYREAL));
+    param = (MYREAL **) mycalloc(GRIDSIZE,sizeof(MYREAL));
     for(i=0;i<GRIDSIZE;i++)
-        param[i] = (MYREAL*) calloc(elem+1,sizeof(MYREAL));
+        param[i] = (MYREAL*) mycalloc(elem+1,sizeof(MYREAL));
     
     if (world->options->progress)
     {
         print_menu_profile (which,
                             world->numpop2 + (long) world->options->gamma);
     }
+    //#ifndef INTEGRATEDLIKE
     SETUPPARAM0 (world, nr, world->repkind, nr->repstart, nr->repstop,
                  world->loci, PROFILE, world->loci>1 ?  TRUE : FALSE);
+    //#endif
     calc_profile_likelihood (method, which, likes, param, world, GRIDSIZE, nr);
     prepare_header (var, which, world);
     if (world->options->printprofile)
     {
-        print_profile_table (method, which, var, likes, param, world);
+      bufsize = print_profile_table (method, which, var, likes, param, world);
+#ifdef PRETTY
+      prepare_profile_table(method, which, likes, param, world, bufsize);
+#endif
     }
     if (world->options->printprofsummary)
     {
@@ -252,10 +253,10 @@ print_profile_likelihood (long which, world_fmt * world, long *gmaxptr)
         }
     }
     destroy_nr (nr, world);
-    free (var);
-    free(likes);
-    //free (param[0]);
-    free (param);
+    myfree(var);
+    myfree(likes);
+    //myfree(param[0]);
+    myfree(param);
 }
 
 
@@ -264,7 +265,8 @@ print_profile_likelihood (long which, world_fmt * world, long *gmaxptr)
 /// depending on the profile option
 /// prints a mark to the precentiles that did not succeed to converge to the 
 /// proper percentile
-void
+/// \return bufsize size of buffer
+long
 print_profile_table (char method, long which, char *var, MYREAL likes[],
                      MYREAL **param, world_fmt * world)
 {
@@ -443,8 +445,9 @@ print_profile_table (char method, long which, char *var, MYREAL likes[],
 	*buffer = (char *) myrealloc(*buffer, sizeof(char) * bufsize);
 	strcat (*buffer, fp);
       }
-    free (paramstr);
-    free (var2);
+    myfree(paramstr);
+    myfree(var2);
+    return bufsize;
 }
 
 
@@ -456,6 +459,147 @@ print_profile_title (world_fmt * world)
     print_line (world->outfile, '=', 79, CONT);
 }
 
+
+///
+/// set the profile method for ascii output
+void ascii_method_set(FILE *outfile, char method)
+{
+    switch (method)
+    {
+    case 'p':
+      fprintf(outfile, "Parameters are evaluated at percentiles using bisection method (slow, but exact).\n");
+      break;
+    case 's':
+      fprintf(outfile,"Parameters are evaluated at percentiles\n");
+      fprintf(outfile,"using cubic splines of profiled parameter (faster, but not so exact).\n");
+      break;
+    case 'd':
+    fprintf(outfile,"Parameters are evaluated at pre-defined values\n");
+    break;
+    case 'q':
+     fprintf(outfile, "Parameters are evaluated assuming complete independence\n");
+      break;
+    case 'f':
+      fprintf(outfile, "Parameters are evaluated assuming complete independence\n");
+      fprintf(outfile,"and then once maximized at the found profiled parameter value\n");
+      break;
+    }
+    fprintf(outfile,"\n");
+}
+
+/*++++++++++++++++++++++++++ replacement for ascii table, not finished *************/
+#if 0
+///
+/// generic ascii table generator
+/// \param float left_margin left edge of table
+/// \param float * page_height page height 
+/// \param int cols number of columns in table if there are too many columns then new line
+/// \param int rows number of rows in table, if a new page is needed then header is repeated
+/// \param char *** elements all table elements, currently no formatting of these
+/// \param char ** header   header row
+/// \param char *position position of all elements: L=left, R=right, C=center
+/// \param int col_overflow when there are too many columns this is the column to restart
+long ascii_table(int cols, int rows, char *buffer, char *position, int col_overflow, float separator)
+{
+  boolean new_page=FALSE;
+
+  int row;
+  int col;
+  long location=0;
+
+  char pos;
+
+  float *col_widths;
+  float left_margin = 0.;
+  float page_width = 130.;
+  float right_margin = 130.;
+  float *col_starts;
+  float oldcol = HUGE;
+
+  char **header;
+  char ***elements;
+
+  col_widths = mycalloc(cols,sizeof(float));
+  col_starts = mycalloc(cols,sizeof(float));
+
+  charvec2d(&header, cols, LINESIZE);
+  elements = (char ***) mycalloc(rows, sizeof(char **));
+  for(row=0; row < rows; row++)
+    {
+      charvec2d(&(elements[row]),cols, LINESIZE);
+    }
+  
+  location = translate_buffer_table(cols, rows, buffer, header, elements);
+  ascii_find_col_width(cols, rows, elements, header, col_widths);
+  define_col_start(cols, col_widths, col_overflow, position, left_margin, right_margin, separator, col_starts);
+  
+  ascii_print_table_header(page_height, position, cols, col_starts,header);
+  sprint_line (fp, '-', 130, CONT);
+
+  for(row=0; row< rows; row++)
+    {
+      for(col=0;col < cols; col++)
+	{
+	  if(col_starts[col] < oldcol)
+	    {
+	      fprintf(outfile,"\n");
+	    }
+	  if(position==NULL)
+	    pos='L';
+	  else
+	    pos=position[col];
+	  
+	  fprintf(outfilecol_starts[col],*page_height, pos, "%s",elements[row][col]);
+	  oldcol = col_starts[col];
+	}
+    }
+  pdf_advance(page_height);
+  myfree(col_widths);
+  myfree(col_starts);
+  free_charvec2d(header);
+  for(row=0; row < rows; row++)
+    free_charvec2d(elements[row]);
+  myfree(elements);
+  return location;
+}
+
+
+///
+/// Printing PROFILE table for ASCII
+/// \param method       profile method
+/// \param world        master parameter structure
+void ascii_print_profile_table(char method, world_fmt *world)
+{
+  char *thisparam;
+  char *savedparam;
+  char *ptr;
+  char *buffer = world->buffer;
+  long position = 0;
+  FILE *outfile = world->outfile;
+
+  boolean failed = world->percentile_some_failed;
+
+  thisparam = (char *) mycalloc(LINESIZE,sizeof(char));
+  savedparam = thisparam;
+  while(buffer[position] != '\0')
+    {
+      strncpy(thisparam, buffer, LINESIZE-1);
+      strsep(&thisparam,"&"); // remove first element
+      strsep(&thisparam,"&"); // remove second  element
+      ptr = strsep(&thisparam,"&"); // keep third 
+      
+      fprintf(outfile,"\n\n");
+      fprintf(outfile, "Profile likelihood table for parameter %s\n", ptr);
+      ascii_method_set(outfile, method);
+      position += pdf_table(left_margin, page_height,world->numpop2 + 3, 9, buffer + position, 
+			    NULL, 4, 6.);
+      pdf_table_footnote(left_margin, page_height, failed);
+    }
+  myfree(savedparam);
+}
+#endif /*++++++++++++++++++++++++++ replacement for ascii table, not finished *************/
+
+
 long
 warp (long ii)
 {
@@ -777,7 +921,7 @@ prognose_profile_end (time_t starttime, long numpop2, long nn)
     }
     else
         tally++;
-    free(nowstr);
+    myfree(nowstr);
 #endif
 }
 
@@ -1022,14 +1166,14 @@ profile_max_spline (char method, long which, MYREAL *likes, MYREAL **param,
             likes[i] = nr->llike;
             normd = nr->normd;
             memcpy (param[i], world->param0, sizeof (MYREAL) * nr->partsize);
-            x = log (xx);
+            x = LOG (xx);
             prognose_profile_end (world->starttime,
                                   world->numpop2 + (long) world->options->gamma,
                                   nn);
         }
         destroy_spline (spline);
-	free(hess[0]);
-	free(hess);
+	myfree(hess[0]);
+	myfree(hess);
     }
 }
 
@@ -1057,13 +1201,13 @@ setup_spline (spline_fmt * spline)
 void
 destroy_spline (spline_fmt * spline)
 {
-    free (spline->param);
-    free (spline->like);
-    free (spline->diff);
-    free (spline->diff2);
-    free (spline->constr);
-    free (spline->diagn);
-    free (spline->work);
+    myfree(spline->param);
+    myfree(spline->like);
+    myfree(spline->diff);
+    myfree(spline->diff2);
+    myfree(spline->constr);
+    myfree(spline->diagn);
+    myfree(spline->work);
 }
 
 /* calc_spline() calculates the spline function derivatives,
@@ -1103,7 +1247,7 @@ prepare_spline_nodes (long n, long which, MYREAL **rawparam, MYREAL *rawlikes,
     long i;
     for (i = 0; i < n; i++)
     {
-        param[i + 1] = log (rawparam[indeks[i]][which]);
+        param[i + 1] = LOG (rawparam[indeks[i]][which]);
         likes[i + 1] = rawlikes[indeks[i]];
     }
 }
@@ -1113,7 +1257,7 @@ prepare_spline_first (MYREAL *param, MYREAL *likes)
 {
     const long i = 2;
     const long i0 = 1;
-    param[0] = log (SMALLEST_THETA);
+    param[0] = LOG (SMALLEST_THETA);
 
     likes[0] =
         ((likes[i] - likes[i0]) / (param[i] - param[i0]) * param[0]) +
@@ -1186,9 +1330,9 @@ recalc_profilelike (MYREAL testparam, MYREAL *param, MYREAL *like, long nn,
     //exit(0);
 
     tmp = y0tab[0];
-    free (y0tab);
-    free (y1tab);
-    free (y2tab);
+    myfree(y0tab);
+    myfree(y1tab);
+    myfree(y2tab);
     return tmp;
 
 }
@@ -1242,7 +1386,7 @@ set_profile_param (MYREAL *param, long which, long *allwhich,
             zzz = 0;
             addinvar (param, fixed, allwhich, xvals, 1, zz)
                 ;
-            free (fixed)
+            myfree(fixed)
             ;
         }
         profnum = 1 + zz;
@@ -1277,7 +1421,7 @@ set_profile_param (MYREAL *param, long which, long *allwhich,
         {
             addinvar (param, fixed, allwhich, xvals, 2, zz)
                 ;
-            free (fixed)
+            myfree(fixed)
             ;
         }
         profnum = 2 + zz;
@@ -1307,7 +1451,7 @@ set_profile_param (MYREAL *param, long which, long *allwhich,
         {
             addinvar (param, fixed, allwhich, xvals, 2, zz)
                 ;
-            free (fixed)
+            myfree(fixed)
             ;
         }
         profnum = 2 + zz;
@@ -1325,7 +1469,7 @@ set_profile_param (MYREAL *param, long which, long *allwhich,
             {
                 addinvar (param, fixed, allwhich, xvals, numpop, zz)
                     ;
-                free (fixed)
+                myfree(fixed)
                 ;
             }
             profnum = numpop + zz;
@@ -1534,9 +1678,9 @@ find_profilelike (MYREAL testlike, MYREAL prob, long whichprob, MYREAL minparam,
             memcpy (*testparam, world->param0, sizeof (MYREAL) * nr->partsize);
         }
     }
-    free (ltestparam);
-    free(hess[0]);
-    free(hess);
+    myfree(ltestparam);
+    myfree(hess[0]);
+    myfree(hess);
     return panic;
 }
 
@@ -1609,12 +1753,12 @@ destroy_profile_percentiles (world_fmt * world)
 
     for (i = 0; i < len; i++)
     {
-        free (world->quantiles[i].name);
-        free (world->quantiles[i].param);
-        free (world->percentile_failed[i]);
+        myfree(world->quantiles[i].name);
+        myfree(world->quantiles[i].param);
+        myfree(world->percentile_failed[i]);
     }
-    free (world->quantiles);
-    free (world->percentile_failed);
+    myfree(world->quantiles);
+    myfree(world->percentile_failed);
 }
 
 
@@ -1628,7 +1772,7 @@ print_menu_profile (long which, long nn)
     FPRINTF (stdout,
              "%s   Calculating profile likelihoods for parameter %2li out of %2li\n",
              nowstr, 1 + which, nn);
-    free(nowstr);
+    myfree(nowstr);
 }
 
 
@@ -1701,8 +1845,8 @@ void master_gridder(world_fmt *world, long *gmaxptr)
     alldim = fopen("dimensions","w+");
     calc_grid(alldim, world, testparam, ltestparam, mlparam, nr, 0);
     fclose(alldim);
-    free(testparam);
-    free(ltestparam);
+    myfree(testparam);
+    myfree(ltestparam);
     destroy_nr(nr, world);
 }
 
