@@ -12,17 +12,17 @@
  and finally helps to destroy itself.
                                                                                                                
  Peter Beerli 1996, Seattle
- beerli@csit.fsu.edu
+ beerli@scs.fsu.edu
  
 Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
-Copyright 2003-2004 Peter Beerli, Tallahassee FL
+Copyright 2003-2007 Peter Beerli, Tallahassee FL
  
  This software is distributed free of charge for non-commercial use
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: random.c 204 2005-12-24 15:12:21Z beerli $
+$Id: random.c 741 2007-06-06 18:40:58Z beerli $
 -------------------------------------------------------*/
 /* \file random.c */
 #include "sighandler.h"
@@ -30,19 +30,15 @@ $Id: random.c 204 2005-12-24 15:12:21Z beerli $
 #include "heating.h"
 #include "random.h"
 #include "migrate_mpi.h"
+#ifdef WIN32
+#include <sys/timeb.h>
+#else /* WIN32 */
+#include <sys/time.h>
+#endif /* WIN32 */
 
-#ifdef SPRNG   /*NCSA random number generator library, slower than ours */
-    #define SIMPLE_SPRNG
-    #ifdef MPI
-        #ifndef USE_MPI
-            #define USE_MPI
-        #endif
-    #endif
-    #include "sprng.h"
-#endif
 
 #ifdef MERSENNE_TWISTER
-#include "mtwist.h"
+#include "SFMT-src-1.2/SFMT.h"
 #endif
 
 #ifdef DMALLOC_FUNC_CHECK
@@ -67,9 +63,7 @@ long random_integer(long low, long high);
 #define MYINDEX 0
 
 #ifdef CUD
-
 int  nextn=1, kk=1,ii=0,jj=0;
-
 void setup_cud(int thiskk)
 {
   strncpy(generator,"Quasi-random number generator: Completely Uniform Distributed numbers", 79);
@@ -79,6 +73,7 @@ void setup_cud(int thiskk)
 MYINLINE MYREAL get_cud()
 {
   MYREAL rr;
+  MYREAL value;
   const MYREAL  magicnumbers[]={2.,2.,4.,15.,65.,315.,1586.,8036.,40448.,200401.,972536.,
 	       4609846.,21310545.,96017492., 421606654.,1804551131. };
   //double ps[]={2.,3.,5.,7.,11.,13.,17.,19.,23.,29.,31.,37.,41.,47.,53.,59.,61.,67.,71.,73.,79.,83.,89.,97.,101.};
@@ -97,6 +92,12 @@ MYINLINE MYREAL get_cud()
    4.4886363697321398383, 4.5747109785033828221, 
    4.6151205168412594509} ; 
   //rr=kk*log(ps[jj]);
+#ifdef PTHREADS
+
+    if ((pthread_mutex_lock (&(heating_pool->random_lock))) != 0)
+        error ("pthread_mutex_lock failed in get_cud()");
+#endif
+
   rr = kk * lps[jj];
   
   if (jj >= ii)
@@ -112,7 +113,14 @@ MYINLINE MYREAL get_cud()
       kk = 1; 
       ii++;
     }
-  return   rr - floor(rr);
+  value = rr - floor(rr);
+#ifdef PTHREADS
+
+    if ((pthread_mutex_unlock (&(heating_pool->random_lock))) != 0)
+        error ("pthread_mutex_unlock failed in get_cud()");
+#endif
+
+    return   value;
   //printf("%f\n",rr);
 }
 
@@ -382,20 +390,13 @@ MYINLINE MYREAL get_quasi()
 
 #endif
 
-#ifdef MERSENNETWISTER
+#ifdef MERSENNE_TWISTER
 void setup_mersennetwister()
 {
   strncpy(generator,"Pseudo-random number generator: Mersenne twister", 79);
 }
 #endif
 
-#ifdef SPRNG
-void setup_sprng()
-{
-  strncpy(generator,"Pseudo-random number generator: SPRNG library", 79);
-}
-#endif
-
 /// random integer
 //=============================================
 MYINLINE long random_integer(long low, long high)
@@ -405,153 +406,172 @@ MYINLINE long random_integer(long low, long high)
     long tt = high-low+1;
     r =  low + (long) (RANDUM() * tt);
     if(r<low || r>high)
-        error("random integer is out of bounds\n");
+      {
+        // warning("Random %li integer is out of bounds (%li, %li)\n",r, low, high);
+	if(r>high)
+	  return high;
+	else
+	  return low;
+	//	error("Stop because this should never happen");
+      }
     return r;
 }
 
-void
-getseed (option_fmt * options)
+
+///
+/// better random_seed function taken from a discussion on 
+/// http://sourceware.org/ml/gsl-discuss/2004-q1/msg00071.html
+/// March 30, 2007, Robert G. Brown (http://www.phy.duke.edu/~rgb/)
+/// at Duke University Dept. of Physics, Box 90305  Durham, N.C. 27708-030
+/// suggested the code 
+unsigned long int random_seed()
 {
-#ifdef MPI
-    long test=0;
-#endif
-    long timeseed;
+  
+  unsigned int seed;
+  struct timeval tv;
+  FILE *devrandom;
+  
+  if ((devrandom = fopen("/dev/random","r")) == NULL) 
+    {
+      gettimeofday(&tv,0);
+      seed = tv.tv_sec + tv.tv_usec;
+    }
+  else 
+    {
+      fread(&seed,sizeof(seed),1,devrandom);
+      fclose(devrandom);
+    }
+  return(seed);
+}
 
-#ifndef SPRNG
-#ifndef MERSENNE_TWISTER
-    long i;
-#ifndef QUASIRANDOM
-    strncpy(generator,"Pseudo-random number generator: Least Congruental Generator", 79);
-#endif
-#endif
-#endif
-
-    switch (options->autoseed)
+void set_seed(long autoseed, long * inseed)
+{
+  unsigned long timeseed = 0;
+   switch (autoseed)
     {
     case AUTO:
-        timeseed = abs ((long) time (NULL));
-#ifndef MERSENNE_TWISTER
-        switch (timeseed % 4)
+      timeseed = random_seed();
+#ifdef LCG
+      switch (timeseed % 4)
         {
         case 0:
-            ++timeseed;
-            break;
-        case 1:
-            break;
+	  ++timeseed; break;
         case 2:
             ++timeseed;
             break;
+        case 1:
         case 3:
             break;
-        default:
-            error ("Problem with automatic random number seed assignment");
         }
-        options->inseed = abs ((long) timeseed);
-#endif /*Mersenne-Twister*/
-        break;
+#endif
+      *inseed = abs((long) timeseed);
+      break;
     case NOAUTO:
-        break;
     case NOAUTOSELF:
         break;
     default:
         error ("Error: Seed value not defined");
         break;
     }
-#ifndef SPRNG
-#ifdef MPI
-    MYMPIBCAST (&options->inseed, 1, MPI_LONG, MASTER, comm_world);
-    if (myID != MASTER)
+}
+
+void init_lcg(option_fmt *options)
+{
+  long i;
+  for (i = 0; i <= 2; i++)
+    seed[i] = 0;
+  i = 0;
+  do
     {
-#ifndef MERSENNE_TWISTER    
-      // for (i = 1; i <= 1000; i++) /* clear the random number generator */
-      //      RANDUM ();
-
-        for (i = 0; i <= 2; i++)
-            seed[i] = 0;
-        i = 0;
-        do
-        {
-            seed[i] = options->inseed & 2047;
-            options->inseed /= 2048;
-            i++;
-        }
-        while (options->inseed != 0);
-        for (i = 0; i <= myID; i++)
-          {
-            test = RANDINT (1, MAXLONG-1);
-            //printf("     %i %li> %li\n", myID, i, test);
-          }
-        switch (test % 4)
-          {
-            case 0:
-                test++;
-                break;
-            case 1:
-                break;
-            case 2:
-                test--;
-                break;
-            case 3:
-                test--;
-                test--;
-                break;
-          }
-        options->inseed = test;
+      seed[i] = options->inseed & 2047;
+      printf("RANDOM SEEDING: i=%li seed[i]=%li inseed=%li\n",i,seed[i], options->inseed);
+      options->inseed /= 2048;
+      i++;
     }
-#ifdef DEBUG_MPI    
-    FPRINTF (stdout, "%i> random number seed = %li\n", myID, options->inseed);
-#endif    
-#else
-        mt_seed32new((unsigned long)options->inseed);
-#endif /*MERSENNE_TWISTER*/
-#endif /*MPI*/
-#endif /*SPRNG*/
+  while (options->inseed != 0);
+  if(i>3)
+    error("lcg random number generator is broken! random.c:507");
+}
 
+void getseed_lcg (option_fmt *options)
+{
+    strncpy(generator,"Pseudo-random number generator: Least Congruental Generator", 79);
+    set_seed(options->autoseed, &options->inseed);
     options->saveseed = options->inseed;
-#ifdef SPRNG
+    init_lcg(options);
+}
 
-    init_sprng (options->inseed, SPRNG_DEFAULT);
-#else /* if we do not have SPRNG */
+void getseed_mt (option_fmt *options)
+{
+    strncpy(generator,"Pseudo-random number generator: Mersenne-Twister", 79);
+    set_seed(options->autoseed, &options->inseed);
+    options->saveseed = options->inseed;
+    init_gen_rand((unsigned long)options->inseed);
+}
 
-#ifndef MERSENNE_TWISTER
-    //   for (i = 1; i <= 1000; i++) /* clear the random number generator */
-    //        RANDUM ();
-    for (i = 0; i <= 2; i++)
-        seed[i] = 0;
-    i = 0;
-    do
+///
+/// set up the quasi random number material
+/// this function is dependent on running the standard MT or LCG random number generator first.
+void getseed_quasi (option_fmt *options)
+{
+#ifdef CUD
+  setup_cud((int) RANDINT (1, MAXLONG-1));
+#endif
+#ifdef WEYL
+  setup_weyl();
+#endif
+#ifdef KOROBOV
+  setup_korobov();
+#endif
+#ifdef HALTON
+  setup_halton();
+#endif
+
+}
+
+
+#ifdef MPI
+void getseed(option_fmt *options)
+{
+  long test = -1;
+  long i;
+#ifdef MERSENNE_TWISTER
+  getseed_mt(options);
+#else
+  getseed_lcg(options);
+#endif
+  MYMPIBCAST (&options->inseed, 1, MPI_LONG, MASTER, comm_world);
+  if(myID != MASTER)
     {
-        seed[i] = options->inseed & 2047;
-        options->inseed /= 2048;
-        i++;
-    }
-    while (options->inseed != 0);
+      for (i = 0; i <= myID; i++)
+	{
+	  test = RANDINT (1, MAXLONG-1); 
+	}
+      options->inseed = test;
+#ifdef MERSENNE_TWISTER
+      getseed_mt(options);
 #else
-    mt_seed32new((unsigned long) options->inseed);
-#endif /*MERSENNE_TWISTER*/
-#endif /* not SPRNG */
+      getseed_lcd(options);
+#endif
 #ifdef QUASIRANDOM
-   #ifdef CUD
-    setup_cud((int) RANDINT (1, MAXLONG-1));
-   #endif
-   #ifdef WEYL
-    setup_weyl();
-   #endif
-   #ifdef KOROBOV
-     setup_korobov();
-   #endif
-   #ifdef HALTON
-     setup_halton();
-   #endif
+      getseed_quasi(options);
+#endif
+    }
+}
 #else
-   #ifdef MERSENNETWISTER
-    setup_mersennetwister(();
-   #endif
-   #ifdef SPRNG
-    setup_sprng();
-   #endif
+void getseed(option_fmt *options)
+{
+#ifdef MERSENNE_TWISTER
+  getseed_mt(options);
+#else
+  getseed_lcg(options);
+#endif
+#ifdef QUASIRANDOM
+  getseed_quasi(options);
 #endif
 }
+#endif
+
 
 void
 swap_ptr (long **ptr, long **newptr)
@@ -567,18 +587,12 @@ MYREAL
 randum_thread (void)
 /* thread save random number generator */
 {
-#ifdef SPRNG
-    return sprng ();
-#else
-
     MYREAL value;
-    //longer newseed;
-#ifdef PTHREADS
-
     if ((pthread_mutex_lock (&(heating_pool->random_lock))) != 0)
         error ("pthread_mutex_lock failed in random_thread()");
-#endif
-
+#ifdef MERSENNE_TWISTER
+    value = genrand_res53();
+#else
     newseed[0] = 1549 * seed[0];
     newseed[1] = newseed[0] / 2048;
     newseed[0] &= 2047;
@@ -589,31 +603,22 @@ randum_thread (void)
     newseed[1] &= 2047;
     newseed[2] += 1549 * seed[2] + 812 * seed[1];
     swap_ptr (&newseed, &seed);
-    //  memcpy (seed, newseed, sizeof (longer));
     seed[2] &= 1023;
     value = (((seed[0] / 2048.0 + seed[1]) / 2048.0 + seed[2]) / 1024.0);
-#ifdef PTHREADS
-
+#endif
     if ((pthread_mutex_unlock (&(heating_pool->random_lock))) != 0)
         error ("pthread_mutex_unlock failed in randum_thread()");
-#endif
-
     return value;
-#endif /*SPRNG*/
-}    /* randum */
+}    /* threaded randum */
 #else
 MYREAL
 randum (void)
 /* Non thread-safe random number generator (faster) */
 {
-#ifdef SPRNG
-    return sprng ();
-#else
 #ifdef MERSENNE_TWISTER
-    return mt_drand();
+    return genrand_res53();
 #else
 MYREAL value;
-//  longer newseed;
 newseed[0] = 1549 * seed[0];
 newseed[1] = newseed[0] / 2048;
 newseed[0] &= 2047;
@@ -623,12 +628,10 @@ newseed[1] += 1549 * seed[1] + 812 * seed[0];
 newseed[2] += newseed[1] / 2048;
 newseed[1] &= 2047;
 newseed[2] += 1549 * seed[2] + 812 * seed[1];
-//memcpy (seed, newseed, sizeof (longer));
 swap_ptr (&newseed, &seed);
 seed[2] &= 1023;
 value = (((seed[0] / 2048.0 + seed[1]) / 2048.0 + seed[2]) / 1024.0);
 return value;
-#endif /*MERSENNE_TWISTER*/
-#endif /*SPRNG*/
+#endif /*mersenne_twister*/
 }    /* randum */
-#endif
+#endif /*phtreads*/
