@@ -7,7 +7,7 @@
  
  
  Peter Beerli 1996, Seattle
- beerli@csit.fsu.edu
+ beerli@scs.fsu.edu
  
  Copyright 2002 Peter Beerli and Joseph Felsenstein
  
@@ -16,7 +16,7 @@
  works and are not responsible for any damage you may cause or have.
  
  
- $Id: sequence.c 175 2005-12-02 14:51:23Z beerli $
+ $Id: sequence.c 701 2007-05-08 01:57:47Z beerli $
  
 -------------------------------------------------------*/
 /* \file sequence.c
@@ -26,6 +26,7 @@
 #include "sighandler.h"
 #include "tools.h"
 #include "migrate_mpi.h"
+#include "watterson.h"
 
 #ifdef DMALLOC_FUNC_CHECK
 #include <dmalloc.h>
@@ -228,7 +229,7 @@ initcategs (long categs, MYREAL *rate, MYREAL *probcat)
     for (;;)
     {
         printf
-        ("Either enter the Shape parameter alpha for Gamma deviated rates\n*OR* enter the rates for each category (use a space to separate)\n");
+	  ("Either enter the Shape parameter alpha for Gamma deviated rates\n*OR* enter the rates for each category (use a space to separate)\n===>");fflush(stdout);
         FGETS (input, LINESIZE, stdin);
         done = TRUE;
         if (count_words (input) == 1)
@@ -256,7 +257,6 @@ initcategs (long categs, MYREAL *rate, MYREAL *probcat)
             break;
     }
     return FALSE;
-
 }
 
 void
@@ -309,6 +309,28 @@ initprobcat (long categs, MYREAL *probsum, MYREAL *probcat)
     while (!done);
 }
 
+///
+/// constrains the arbitrary site rate variation to an average of 1.0
+void constrain_rates(long categs, MYREAL *rate, MYREAL *probcat)
+{
+  char input[LINESIZE];
+  long i;
+  MYREAL mean;
+  printf("By default the rates will be constrained to an have an average rate of 1.0\nPress return if that is OK (preferred option) or enter the word RAW\n===>"); fflush(stdout);
+  FGETS (input, LINESIZE, stdin);
+  if(input[0] == '\0')
+    {
+      mean = 0.0;
+      for (i = 0; i < categs; i++)
+	{
+	  mean += probcat[i] * rate[i];
+	}
+      for (i = 0; i < categs; i++)
+	{
+	  rate[i] /= mean;
+	}
+    }
+}
 
 /*data read material ===================================== */
 
@@ -387,15 +409,16 @@ getbasefreqs (option_fmt * options, seqmodel_fmt * seq, long locus)
     seq->xv = 1.0 - seq->xi;
     if (seq->xi <= 0.0)
     {
-        printf ("\n WARNING: This transition/transversion ratio\n");
-        printf (" is impossible with these base frequencies!\n");
-        seq->xi = 0.0;
-        seq->xv = 1.0;
-        seq->ttratio =
-            (seq->freqa * seq->freqg +
-             seq->freqc * seq->freqt) / ((seq->freqr) * (seq->freqy));
-        printf (" Transition/transversion parameter reset\n");
-        printf ("  so transition/transversion ratio is %10.6f\n\n",
+        warning ("This transition/transversion ratio (%f)\n",seq->ttratio);
+        warning ("is impossible with these base frequencies (%f, %f, %f, %f)!\n",seq->freqa,seq->freqc,seq->freqg,seq->freqt);
+        seq->xi = 0.00001; // do not set this to zero because of the 1/(fracchange=xi*(...))
+        seq->xv = 0.99999;
+	seq->ttratio =
+	  (seq->freqa * seq->freqg +
+	   seq->freqc * seq->freqt) / ((seq->freqr) * (seq->freqy));
+
+        warning (" Transition/transversion parameter reset\n");
+        warning ("  so transition/transversion ratio is %10.6f\n\n",
                 (seq->ttratio));
     }
     // use 1/frac as precomputation speed up
@@ -631,6 +654,12 @@ makevalues_seq (world_fmt * world, option_fmt * options, data_fmt * data,
                 if (k==0 && !options->usertree)
                 {
                     strcpy (treenode[i]->nayme, data->indnames[pop][ii]);
+#ifdef TESTINGDATE
+		    treenode[i]->tyme = world->data->sampledates[pop][locus][ii].date 
+		      * world->options->generation_year 
+		      * world->options->mutationrate_year[locus];
+		    world->data->sampledates[pop][locus][ii].id = treenode[i]->id;
+#endif
                 }
                 for (l = 0; l < world->options->rcategs; l++)
                 {                    
@@ -1166,8 +1195,8 @@ sitesort2 (world_fmt * world, data_fmt * data, long sites, long locus)
         }
         gap /= 2;
     }
-    free (tempsum);
-    free (temppop);
+    myfree(tempsum);
+    myfree(temppop);
 }    /* sitesort2 */
 
 
@@ -1240,8 +1269,8 @@ sitecombine2 (world_fmt * world, data_fmt * data, long sites, long locus)
         }
         i = j;
     }
-    free (temppop);
-    free (tempsum);
+    myfree(temppop);
+    myfree(tempsum);
 }    /* sitecombine2 */
 
 
@@ -1469,7 +1498,7 @@ initlambda (option_fmt * options)
         printf
         ("Mean block length of sites having the same rate\n (needs to be greater than 1)?\n");
 #ifdef USE_MYREAL_FLOAT
-        scanf ("%lf%*[^\n]", &options->lambda);
+        scanf ("%f%*[^\n]", &options->lambda);
 #else
         scanf ("%lf%*[^\n]", &options->lambda);
 #endif
@@ -1508,7 +1537,7 @@ init_tbl (world_fmt * world, long locus)
         }
     }
     sumrates = 0.0;
-    for (i = 0; i < world->data->seq->endsite; i++)
+    for (i = 0; i < world->data->seq->endsite - world->data->seq->addon; i++)
     {
         for (j = 0; j < rcategs; j++)
             sumrates +=
@@ -1670,7 +1699,7 @@ treelike_seq (world_fmt * world, long locus)
             x1 = &(p->x.s[i][0].v);
             scale = p->scale[i];
             stterm =  vector_multiply(&freq.v, x1);
-            summ += seq->aliasweight[i] * (log (stterm) + scale);
+            summ += seq->aliasweight[i] * (LOG (stterm) + scale);
             if(MYISNAN(summ))
                 error("F..... ahem Fehler\n");
         }
@@ -1682,7 +1711,7 @@ treelike_seq (world_fmt * world, long locus)
             tterm[0] =
                 freqa * (*x1)[0] + freqc * (*x1)[1] +
                 freqg * (*x1)[2] + freqt * (*x1)[3];
-            summ += seq->aliasweight[i] * (log (tterm[0]) + scale);
+            summ += seq->aliasweight[i] * (LOG (tterm[0]) + scale);
         }
 #endif
     }
@@ -1707,14 +1736,17 @@ treelike_seq (world_fmt * world, long locus)
             sumterm = 0.0;
             for (j = 0; j < opt->rcategs; j++)
                 sumterm += opt->probcat[j] * tterm[j];
-            lterm = log (sumterm) + scale;
+            lterm = LOG (sumterm) + scale;
             for (j = 0; j < opt->rcategs; j++)
                 clai[j] = tterm[j] / sumterm;
             swap (clai, world->contribution[i]);
             //      memcpy (world->contribution[i], clai, size);
             summ += seq->aliasweight[i] * lterm;
-            if(MYISNAN(summ))
-                error("F..... ahem Fehler\n");
+            if(MYISNAN((float)summ))
+	      {
+		// error("summ is not a number, should not happen\n");
+		summ = -HUGE;
+	      }
 
         }
         for (j = 0; j < opt->rcategs; j++)
@@ -1744,7 +1776,7 @@ treelike_seq (world_fmt * world, long locus)
         sum2 = 0.0;
         for (i = 0; i < opt->rcategs; i++)
             sum2 += opt->probcat[i] * like[i];
-        summ += log (sum2);
+        summ += LOG (sum2);
     }
     return summ;
 }    /* treelikelihood */
@@ -1814,7 +1846,7 @@ treelike_snp (world_fmt * world, long locus)
         sumterm = 0.0;
         for (j = 0; j < opt->rcategs; j++)
             sumterm += opt->probcat[j] * tterm[j];
-        lterm = log (sumterm) + scale;
+        lterm = LOG (sumterm) + scale;
         for (j = 0; j < opt->rcategs; j++)
             world->contribution[i][j] = tterm[j] / sumterm;
 
@@ -1847,7 +1879,7 @@ treelike_snp (world_fmt * world, long locus)
     sum2 = 0.0;
     for (i = 0; i < opt->rcategs; i++)
         sum2 += opt->probcat[i] * like[i];
-    summ += log (sum2);
+    summ += LOG (sum2);
     return summ;
 }    /* treelike_snp */
 
@@ -1891,7 +1923,7 @@ treelike_snp_unlinked (world_fmt * world, long locus)
 #endif
         if (i % 5 == 0)
         {
-            lterm = log (tterm[0]) + scale;
+            lterm = LOG (tterm[0]) + scale;
             summ = 0;
             datasum = seq->aliasweight[ii] * lterm;
         }
@@ -1902,7 +1934,7 @@ treelike_snp_unlinked (world_fmt * world, long locus)
 	    if(summ > 0.)
 	      {
 		result +=
-		  datasum + log ((1 - EXP (log (summ) - datasum)) / invariants[0]);
+		  datasum + LOG ((1 - EXP (LOG (summ) - datasum)) / invariants[0]);
 	      }
 	  }
     }
@@ -1963,3 +1995,104 @@ copy_seq (world_fmt * original, world_fmt * kopie)
     memcpy (kseq->location, oseq->location, sizeof (long) * sites);
     kseq->addon = oseq->addon;
 }
+
+void find_rates_fromdata(data_fmt * data, option_fmt * options)
+{
+  long locus;
+  long pop;
+  long ind;
+  long sites;
+  long i;
+  long n;
+  long s;
+  long **v;
+  long maxsites = 1;
+  long *numind;
+  char *reference;
+  char *indseq;
+  long loci = data->loci;
+  long numpop = data->numpop;
+  long segreg;
+  MYREAL mean=0.;
+  MYREAL delta = 0.;
+
+  numind = (long *) mycalloc (data->loci, sizeof (long));
+
+  for (locus=0; locus < loci; locus++)
+    {
+      maxsites = (data->seq->sites[locus] > maxsites ? data->seq->sites[locus] : maxsites);
+      for(pop=0;pop < data->numpop; pop++)
+	{
+	  numind[locus] += data->numind[pop][locus];
+	}
+    }
+
+  v = (long **) mycalloc (loci, sizeof (long *));
+  v[0] = (long *) mycalloc (loci * maxsites, sizeof (long));
+  for (i = 1; i < loci; i++)
+    {
+      v[i] = v[0] + maxsites * i;
+    }
+  
+  for (locus=0; locus< loci; locus++)
+    {
+      reference = data->yy[0][0][locus][0];
+      for(pop=0;pop < numpop; pop++)
+	{
+	  for(ind=0; ind < data->numind[pop][locus];ind++)
+	    {
+	      indseq=data->yy[pop][ind][locus][0];
+	      if(strcmp(indseq,reference)!=0)
+		{
+		  for(sites=0; sites < data->seq->sites[locus]; sites++)
+		    {
+		      if(v[locus][sites] == 0)
+			{
+			  s = (long)(indseq[sites] != reference[sites]);
+			  v[locus][sites] += s;
+			}
+		    }
+		}
+	    }
+	}
+    }
+  n = 0;
+  mean = 0.;
+  if(options->mu_rates == NULL)
+    options->mu_rates = (MYREAL *) mycalloc( loci, sizeof(MYREAL));
+  n = 0;
+  mean = 0. ;
+  options->muloci = loci;
+  for (locus=0; locus < loci; locus++)
+    {
+      segreg = 0;
+      for(sites=0; sites < data->seq->sites[locus]; sites++)
+	{
+	  segreg += (long) (v[locus][sites] > 0); 
+	}
+      options->mu_rates[locus] = watterson(segreg,numind[locus]) + 0.0000001;
+      n = n + 1;
+      delta = options->mu_rates[locus] - mean;
+      mean += delta/n;
+    }
+  if(mean > 0.0)
+    {
+      for (locus=0; locus < loci; locus++)
+	{
+	  options->mu_rates[locus] /= mean;
+	  if(options->progress)
+	    {
+	      fprintf(stdout,"Locus %2li: rate=%f (Watterson's overall Theta = %f)\n",locus+1, options->mu_rates[locus],options->mu_rates[locus] * mean / data->seq->sites[locus]);
+	    }
+	}
+    }
+  else
+    {
+      options->murates = FALSE;
+      options->murates_fromdata = FALSE;
+      warning("Relative mutation rates estimation from data was requested but failed, reset to all-equal rates");
+    }
+  myfree(v[0]);
+  myfree(v);
+  myfree(numind);
+}
