@@ -9,24 +9,25 @@
  string and file manipulation routines
  
  
- Peter Beerli 1996, Seattle
- beerli@csit.fsu.edu
+ Peter Beerli started 1996, Seattle
+ beerli@scs.fsu.edu
  
 Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
-Copyright 2003-2004 Peter Beerli, Tallahassee FL
+Copyright 2003-2006 Peter Beerli, Tallahassee FL
  
   This software is distributed free of charge for non-commercial use
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: tools.c 220 2006-01-10 19:00:59Z beerli $
+$Id: tools.c 758 2007-07-03 14:12:40Z beerli $
 -------------------------------------------------------*/
 /* \file tools.c */
 #include "migration.h"
 #include "sighandler.h"
 #include "data.h"
 #include "world.h"
+#include "tree.h"
 #include "random.h"
 #include "broyden.h"
 #include "migrate_mpi.h"
@@ -43,24 +44,24 @@ extern int myID;
 
 /* prototypes ------------------------------------------- */
 MYREAL lengthof (node * p);
-node *crawlback (const node * theNode);
+MYINLINE node *crawlback (const node * theNode);
 /*node *crawl(node * theNode); */
-node *showtop (node * theNode);
+MYINLINE node *showtop (node * theNode);
 void adjust_time (node * theNode, MYREAL tyme);
+void adjust_time_all (node * theNode, MYREAL tyme);
 void insert_migr_node (world_fmt * world, node * up, node * down,
                        migr_table_fmt * migr_table, long *migr_table_counter);
 void children (node * mother, node ** brother, node ** sister);
 /* math tools */
-MYREAL incompletegamma (MYREAL x, MYREAL alpha);
+MYREAL incompletegamma (MYREAL tx, MYREAL talpha);
 MYREAL polygamma (long n, MYREAL z);
 void invert_matrix (MYREAL **a, long nsize);
 boolean nrcheck (MYREAL **m, MYREAL **tm, MYREAL *v, long nrows, MYREAL *r1,
                  MYREAL *r2, boolean do_newton);
 MYREAL rannor (MYREAL mean, MYREAL sd);
-char lowercase (char c);
-char uppercase (char c);
+char lowercase (int c);
+char uppercase (int c);
 MYREAL calc_sum (MYREAL *vector, long n);
-void bootstrap (world_fmt * world);
 
 void gamma_rates (MYREAL *rate, MYREAL *probcat, long categs, char *input);
 void calc_gamma (MYREAL alpha, MYREAL *gama, long categs);
@@ -69,10 +70,20 @@ MYREAL mylgamma (MYREAL z);
 
 MYREAL logfac (long n);
 
+void onepass_mean_std_start(MYREAL *mean, MYREAL *std, long *n);
+void onepass_mean_std_calc(MYREAL *mean, MYREAL *std, long *n, MYREAL x);
+void onepass_mean_std_end(MYREAL *mean, MYREAL *std, long *n);
+
+//MYINLINE double fast_exp(double y) ;
+//MYINLINE float fast_log (float vval);
+
 /* vector initialization */
 void doublevec1d (MYREAL **v, long size);
 void doublevec2d (MYREAL ***v, long size1, long size2);
+void charvec1d (char **v, long size1);
+void charvec2d (char ***v, long size1, long size2);
 void free_doublevec2d (MYREAL **v);
+void free_charvec2d (char **v);
 void setdoublevec1d (MYREAL **v, MYREAL *w, long size);
 void add_vector (MYREAL *result, MYREAL *v, long size);
 
@@ -120,7 +131,7 @@ long i1mach (long i);
 int dpsifn (MYREAL *x, long *n, long kode, long m, MYREAL *ans, long *nz,
             long *ierr);
 MYREAL find_chi (long df, MYREAL prob);
-MYREAL probchi (long df, MYREAL chi);
+MYREAL probchi (long df, double chi);
 MYREAL chisquare (long df, MYREAL alpha);
 
 MYREAL chiboundary (long zeros, long nonzeros, MYREAL alpha);
@@ -144,11 +155,32 @@ doublevec2d (MYREAL ***v, long size1, long size2)
     }
 }
 
+///
+/// allocates columns and rows for a matrix of strings
+/// use freecharvec2d() to free this.
+void
+charvec2d (char ***v, long size1, long size2)
+{
+    long i;
+    *v = (char **) mycalloc (size1, sizeof (char *));
+    (*v)[0] = (char *) mycalloc (size1 * size2, sizeof (char));
+    for (i = 1; i < size1; i++)
+    {
+        (*v)[i] = (*v)[0] + size2 * i;
+    }
+}
+
 void
 free_doublevec2d (MYREAL **v)
 {
-    free (v[0]);
-    free (v);
+    myfree(v[0]);
+    myfree(v);
+}
+void
+free_charvec2d (char **v)
+{
+    myfree(v[0]);
+    myfree(v);
 }
 
 void
@@ -241,6 +273,39 @@ logfac (long n)
     }
 }
 
+///
+/// initialize the mean and standard deviation calculations 
+/// for the one-pass calculations that allows to throw away 
+/// the intermediate results
+void onepass_mean_std_start(MYREAL *mean, MYREAL *std, long *n)
+{
+  (*mean) = 0.0;
+  (*std)  = 0.0;
+  (*n)    = 0;
+}
+
+///
+/// calculates the mean and standard deviation using a one-pass filter 
+/// that allows to throw away the intermediate results
+void onepass_mean_std_calc(MYREAL *mean, MYREAL *std, long *n, MYREAL x)
+{
+  
+  long   nn    = (*n);
+  MYREAL mm    = (*mean);
+  MYREAL delta = x - mm;
+  mm           = mm + delta / nn;
+  (*std)       = (*std) + delta * (x - mm);
+  (*mean)      = mm;
+  (*n)         = nn;
+}
+
+///
+/// finishes one-pass mean and std calculations
+void onepass_mean_std_end(MYREAL *mean, MYREAL *std, long *n)
+{
+  (*std) = sqrt((*std) / ((*n) - 1));
+}
+
 
 /*FILEMANIPULATION======================================================= */
 void
@@ -250,7 +315,7 @@ init_files (world_fmt * world, data_fmt * data, option_fmt * options)
 #ifdef MPI
 	long i,j;
 #endif
-    long maxfilenum = 43; //see definitions.h all files have a number and a filename;
+    long maxfilenum = MYMAXFILENUM; //see definitions.h all files have a number and a filename;
 	tempdb = mycalloc(maxfilenum,sizeof(long));
 	
     if (myID == MASTER)
@@ -286,6 +351,11 @@ init_files (world_fmt * world, data_fmt * data, option_fmt * options)
 			openfile (&world->mighistfile, options->mighistfilename, "w+", NULL);
 #ifdef MPI
 			setup_filehandle_db((void *) world->mighistfile, world, options, data);
+#endif			
+			if(options->skyline)
+			  openfile (&world->skylinefile, options->skylinefilename, "w+", NULL);
+#ifdef MPI
+			setup_filehandle_db((void *) world->skylinefile, world, options, data);
 #endif			
 		  }
         
@@ -327,9 +397,13 @@ init_files (world_fmt * world, data_fmt * data, option_fmt * options)
             }
         }
         if (options->geo)
-		  {
+	  {
             openfile (&data->geofile, options->geofilename, "r+",  NULL);
-		  }
+	  }
+        if (options->has_datefile)
+	  {
+            openfile (&data->datefile, options->datefilename, "r+",  NULL);
+	  }
 #ifdef UEP
 
         if (options->uep)
@@ -374,6 +448,8 @@ init_files (world_fmt * world, data_fmt * data, option_fmt * options)
 		world->outfile = filedb[j++].file;
 		if (options->mighist)
 			world->mighistfile = filedb[j++].file;
+		if (options->skyline)
+			world->skylinefile = filedb[j++].file;
 		if (options->writelog)
 			options->logfile = filedb[j++].file;
 		if (options->aic)
@@ -389,7 +465,7 @@ init_files (world_fmt * world, data_fmt * data, option_fmt * options)
 	  }
 #endif /*MPI*/
 	  }
-    free(tempdb);
+    myfree(tempdb);
 }
 
 void
@@ -397,48 +473,50 @@ exit_files (world_fmt * world, data_fmt * data, option_fmt * options)
 {
     if(myID==MASTER)
     {
-		if (!options->readsum)
-		  {
-			FClose (data->infile);
-			if (options->treeprint > 0)
-				FClose (world->treefile);
-			if(options->usertree)
-				FClose(data->utreefile);
-			if (options->weights)
-				FClose (data->weightfile);
-			if (options->categs > 1)
-				FClose (data->catfile);
-			if (options->dist)
-				FClose (data->distfile);
-		  }
-
-			if (options->writesum || options->readsum)
-				FClose (data->sumfile);
-
-			FClose (world->outfile);
-
-			if (options->writelog)
-            FClose (options->logfile);
-        if (options->mighist)
-            FClose (world->mighistfile);
-        if (options->geo)
-            FClose (data->geofile);
-        if (options->aic)
-            FClose (options->aicfile);
-        if (options->plot && options->plotmethod == PLOTALL)
-            FClose (world->mathfile);
+      if (!options->readsum)
+	{
+	  FClose (data->infile);
+	  //  if (options->treeprint > 0)
+	  //  FClose (world->treefile);
+	  if(options->usertree)
+	    FClose(data->utreefile);
+	  if (options->weights)
+	    FClose (data->weightfile);
+	  if (options->categs > 1)
+	    FClose (data->catfile);
+	  if (options->dist)
+	    FClose (data->distfile);
+	}
+      
+      if (options->writesum || options->readsum)
+	FClose (data->sumfile);
+      
+      FClose (world->outfile);
+      
+      if (options->writelog)
+	FClose (options->logfile);
+      if (options->mighist)
+	FClose (world->mighistfile);
+      if (options->geo)
+	FClose (data->geofile);
+      if (options->has_datefile)
+	FClose (data->datefile);
+      if (options->aic)
+	FClose (options->aicfile);
+      if (options->plot && options->plotmethod == PLOTALL)
+	FClose (world->mathfile);
 #ifdef UEP
-
-        if (options->uep)
-            FClose (data->uepfile);
+      
+      if (options->uep)
+	FClose (data->uepfile);
 #endif
-        if (options->bayes_infer)
-	  {
-	    if(options->has_bayesfile)
-	      FClose (world->bayesfile);
-	    if(options->has_bayesmdimfile)
-	      FClose (world->bayesmdimfile);
-	  }
+      if (options->bayes_infer)
+	{
+	  if(options->has_bayesfile)
+	    FClose (world->bayesfile);
+	  if(options->has_bayesmdimfile)
+	    FClose (world->bayesmdimfile);
+	}
     }
 }
 
@@ -467,6 +545,23 @@ translate (char *text, char from, char to)
     text[j] = '\0';
 }
 
+///
+/// get next word from list with list of delimiters
+/// uses strsep
+void
+get_next_word(char **instring, char *delimiters, char **nextword)
+{
+  *nextword = strsep(instring, delimiters);
+  while(strchr(delimiters,(*nextword)[0]))
+  {
+    if((*instring)==NULL)
+      {
+	(*nextword) = NULL;
+	return;
+      }
+      *nextword = strsep(instring,delimiters);
+  }
+}
 /*===============================================
   count words in a string delimited by delimiter
 */
@@ -575,12 +670,13 @@ void read_word(FILE *infile, char *word)
 // text in the file
 void unread_word(FILE *infile, char *word)
 {
-    long count = (long) strlen(word);
-    ungetc(' ', infile);
-    while(count > 0)
+  char c = ' ';
+  long count = (long) strlen(word);
+  ungetc(c, infile);
+  while(count > 0)
     {
-        count--;
-        ungetc(word[count], infile);
+      count--;
+      ungetc(word[count], infile);
     }
 }
 
@@ -596,7 +692,7 @@ openfile (FILE ** fp, char *filename, char *mode, char *perm)
     FILE *of = NULL;
     char *file;
     char *p;
-    file = calloc(LINESIZE,sizeof(char));
+    file = (char *) mycalloc(LINESIZE,sizeof(char));
     if ((p = strpbrk (filename, CRLF)) != NULL)
         *p = '\0';
     remove_trailing_blanks(&filename);
@@ -670,7 +766,7 @@ openfile (FILE ** fp, char *filename, char *mode, char *perm)
     if (perm != NULL)
         strcpy (perm, file);
     strcpy (filename, file);
-    free(file);
+    myfree(file);
 }
 
 long
@@ -941,7 +1037,7 @@ Find the next non-migration node starting
 with the theNode, returns to backnode which is not 
 a migration, does NOT return always a top-node!
 */
-node *
+MYINLINE node *
 crawlback (const node * theNode)
 {
     node *tmp = theNode->back;
@@ -975,7 +1071,7 @@ node *crawl(node * theNode)
 */
 
  
-node *
+MYINLINE node *
 showtop (node * theNode)
 {
     if (theNode == NULL)
@@ -1009,17 +1105,45 @@ adjust_time (node * theNode, MYREAL tyme)
     {
     case 'm':
         theNode->tyme = theNode->next->tyme = tyme;
+	set_dirty(theNode);
         break;
     case 'i':
         theNode->tyme = theNode->next->tyme = theNode->next->next->tyme = tyme;
+	set_dirty(theNode);
         break;
     case 'r':
+      break;
     case 't':
-        break;
+      break;
     default:
-        error ("Wrong node type");
+      error ("Wrong node type");
+      break;
+    }
+}
+/* adjust the time in any node to time */
+void
+adjust_time_all (node * theNode, MYREAL tyme)
+{
+    switch (theNode->type)
+    {
+    case 'm':
+        theNode->tyme = theNode->next->tyme = tyme;
+	set_dirty(theNode);
         break;
-
+    case 'i':
+        theNode->tyme = theNode->next->tyme = theNode->next->next->tyme = tyme;
+	set_dirty(theNode);
+        break;
+    case 'r':
+      theNode->tyme = theNode->next->tyme = theNode->next->next->tyme = tyme;
+      set_dirty(theNode);
+      break;
+    case 't':
+      theNode->tyme = tyme;
+      break;
+    default:
+      error ("Wrong node type");
+      break;
     }
 }
 
@@ -1052,7 +1176,7 @@ insert_migr_node (world_fmt * world, node * up, node * down,
         panic = 0;
         while (theNode->tyme < migr_table[i].time /*&& panic++ < 10000*/)
         {
-            if (theNode->tip && theNode->type != 'r')
+            if (theNode->type == 't' )
             {
                 oldNode = theNode;
                 theNode = theNode->back;
@@ -1067,8 +1191,8 @@ insert_migr_node (world_fmt * world, node * up, node * down,
         oldNode->back = tmp;
         tmp->number = -999;
         tmp->nayme = NULL;
-        tmp->tip = 0;
-        tmp->top = 0;
+        tmp->tip = FALSE;
+        tmp->top = FALSE;
         tmp->dirty = TRUE;
         tmp->id = world->unique_id++;
         tmp->tyme = migr_table[i].time;
@@ -1149,7 +1273,7 @@ mylgamma (MYREAL z)
         tmp -= 1.;
     }
     result += a[0];
-    result = log (result) + lnsqrt2pi - (z + 6.5) + (z - 0.5) * log (z + 6.5);
+    result = LOG (result) + lnsqrt2pi - (z + 6.5) + (z - 0.5) * LOG (z + 6.5);
     return result;
 }    /* lgamma */
 
@@ -1160,20 +1284,23 @@ mylgamma (MYREAL z)
    in Mathematica this is GammaRegularized[a,0,x] === Gamma[a,0,x]/Gamma[a]
  */
 MYREAL
-incompletegamma (MYREAL x, MYREAL alpha)
+incompletegamma (MYREAL tx, MYREAL talpha)
 {
-    MYREAL gama, d_1, d_2, d_3;
-    /*static */
-    MYREAL a, b, c, an, rn;
-    /*static */
-    MYREAL pn1, pn2, pn3, pn4, pn5, pn6, arg;
+  const  MYREAL eps = DBL_EPSILON ;
+  double gama, d_1, d_2, d_3;
+  /*static */
+  double  a, b, c, an, rn;
+  /*static */
+  double  pn1, pn2, pn3, pn4, pn5, pn6, arg;
+  double x = (double) tx;
+  double alpha = (double) talpha;
 
-    gama = 0.;
-    /*  Check that we have valid values for X and P */
-    if (alpha <= 0. || x < 0.)
-        error ("failed in imcompletegamma(): wrong alpha or x\n");
-    if (fabs (x) < DBL_EPSILON)
-        return gama;
+  gama = 0.;
+  /*  Check that we have valid values for X and P */
+  if (alpha <= 0. || x < 0.)
+    error ("failed in imcompletegamma(): wrong alpha or x\n");
+  if (fabs (x) < eps)
+        return (MYREAL) gama;
 
     /*  Use a normal approximation if P > PLIMIT */
     if (alpha > 1e3)
@@ -1182,21 +1309,21 @@ incompletegamma (MYREAL x, MYREAL alpha)
             sqrt (alpha) * 3. * (pow (x / alpha, (1. / 3.)) + 1. / (alpha * 9.) -
                                  1.);
         gama = alnorm (pn1, FALSE);
-        return gama;
+        return (MYREAL) gama;
     }
 
-    /*  If X is extremely large compared to P then set GAMMAD = 1 */
+   /*  If X is extremely large compared to P then set GAMMAD = 1 */
     if (x > 1e8)
     {
         gama = 1.;
-        return gama;
+        return (MYREAL) gama;
     }
 
     if (x <= 1. || x < alpha)
     {
         /*  Use Pearson's series expansion. */
         /*  (Note that P is not large enough to force overflow in lgamma()). */
-        arg = alpha * log (x) - x - LGAMMA (alpha + 1.);
+        arg = alpha * LOG (x) - x - LGAMMA (alpha + 1.);
         c = 1.;
         gama = 1.;
         a = alpha;
@@ -1206,7 +1333,7 @@ incompletegamma (MYREAL x, MYREAL alpha)
             c = c * x / a;
             gama += c;
         }
-        arg += log (gama);
+        arg += LOG (gama);
         gama = 0.;
         if (arg >= -88.)
         {
@@ -1217,7 +1344,7 @@ incompletegamma (MYREAL x, MYREAL alpha)
     else
     {
         /*  Use a continued fraction expansion */
-        arg = alpha * log (x) - x - LGAMMA (alpha);
+        arg = alpha * LOG (x) - x - LGAMMA (alpha);
         a = 1. - alpha;
         b = a + x + 1.;
         c = 0.;
@@ -1241,13 +1368,13 @@ incompletegamma (MYREAL x, MYREAL alpha)
                 d_2 = 1e-14, d_3 = rn * 1e-14;
                 if ((d_1 = gama - rn, fabs (d_1)) <= MIN (d_2, d_3))
                 {
-                    arg += log (gama);
+                    arg += LOG (gama);
                     gama = 1.;
                     if (arg >= -88.)
                     {
                         gama = 1. - EXP (arg);
                     }
-                    return gama;
+                    return (MYREAL) gama;
                 }
                 gama = rn;
             }
@@ -1265,7 +1392,7 @@ incompletegamma (MYREAL x, MYREAL alpha)
             }
         }
     }
-    return gama;
+    return (MYREAL) gama;
 }    /* incompletegamma() */
 
 
@@ -1318,17 +1445,17 @@ nrcheck (MYREAL **m, MYREAL **tm, MYREAL *v, long nrows, MYREAL *r1,
         (*r2) += tmp[i] * v[i];
         tmp3 += tmp2[i] * v[i];
     }
-    tmp4 = log (fabs ((*r1)));
-    tmp4 = tmp4 + tmp4 - log (fabs ((*r2)));
+    tmp4 = LOG (fabs ((*r1)));
+    tmp4 = tmp4 + tmp4 - LOG (fabs ((*r2)));
     tmp4 = ((*r2) < 0 ? -1 : 1) * EXP (tmp4);
-    free (tmp);
+    myfree(tmp);
     if (do_newton && (tmp3 > (tmp4 > 0 ? tmp4 : 0)))
     {
         memcpy (v, tmp2, sizeof (MYREAL) * nrows);
-        free (tmp2);
+        myfree(tmp2);
         return TRUE;
     }
-    free (tmp2);
+    myfree(tmp2);
     return FALSE;
 }
 
@@ -1364,11 +1491,11 @@ invert_matrix (MYREAL **a, long nsize)
     for (i = 0; i < nsize; i++)
     {
         memcpy (a[i], result[i], sizeof (MYREAL) * nsize);
-        free (result[i]);
+        myfree(result[i]);
     }
-    free (result);
-    free (column);
-    free (indeks);
+    myfree(result);
+    myfree(column);
+    myfree(indeks);
 }
 
 /*=======================================================*/
@@ -1448,7 +1575,7 @@ lu_decomp (MYREAL **m, long *indeks, long nrows)
                 m[k][i] *= pivot;
         }
     }
-    free (max_row_vals);
+    myfree(max_row_vals);
 }    /* end of lu_decomp */
 
 /*-------------------------------------------------------*/
@@ -1701,16 +1828,26 @@ long fourteen = 14;
 MYREAL
 d1mach (long i)
 {
+  //#ifdef MYREAL == float
+  //const  MYREAL eps = FLT_EPSILON ;
+  //const  MYREAL numbermax = FLT_MAX;
+  //const  MYREAL numbermin = FLT_MIN;
+  //#else
+  const  MYREAL eps = DBL_EPSILON ;
+  const  MYREAL numbermax = DBL_MAX;
+  const  MYREAL numbermin = DBL_MIN;
+  //#endif
+
     switch (i)
     {
     case 1:
-        return DBL_MIN;
+        return numbermin;
     case 2:
-        return DBL_MAX;
+        return numbermax;
     case 3:
-        return DBL_EPSILON / FLT_RADIX;
+        return eps / FLT_RADIX;
     case 4:
-        return DBL_EPSILON;
+        return eps;
     case 5:
         return log10 ((MYREAL) FLT_RADIX);
     }
@@ -1848,7 +1985,7 @@ dpsifn (MYREAL *x, long *n, long kode, long m, MYREAL *ans, long *nz,
     /*     ELIM = APPROXIMATE EXPONENTIAL OVER AND UNDERFLOW LIMIT */
     /* ----------------------------------------------------------------------- */
     elim = (nx * r1m5 - 3.) * 2.302;
-    xln = log (*x);
+    xln = LOG (*x);
 L41:
     nn = *n + mm - 1;
     fn = nn;
@@ -1905,7 +2042,7 @@ L50:
     nx = (long) (*x);
     xinc = xmin - nx;
     xdmy = *x + xinc;
-    xdmln = log (xdmy);
+    xdmln = LOG (xdmy);
 L60:
     /* ----------------------------------------------------------------------- */
     /*     GENERATE W(N+MM-1,X) BY THE ASYMPTOTIC EXPANSION */
@@ -2078,7 +2215,7 @@ L190:
         return 0;
     }
     xq = xdmy / *x;
-    ans[1] = s - log (xq);
+    ans[1] = s - LOG (xq);
     return 0;
     /* ----------------------------------------------------------------------- */
     /*     COMPUTE BY SERIES (X+K)**(-(N+1)) , K=0,1,2,... */
@@ -2201,20 +2338,20 @@ rannor (MYREAL mean, MYREAL sd)
     MYREAL r1, r2;
     r1 = RANDUM ();
     r2 = RANDUM ();
-    return sd * sqrt (-2. * log (r1)) * cos (TWOPI * r2) + mean;
+    return sd * sqrt (-2. * LOG (r1)) * cos (TWOPI * r2) + mean;
 }
 
 
 char
-lowercase (char c)
+lowercase (int c)
 {
-    return (char) tolower ((int) c);
+    return (char) tolower (c);
 }
 
 char
-uppercase (char c)
+uppercase (int c)
 {
-    return (char) toupper ((int) c);
+    return (char) toupper (c);
 }
 
 void upper(char *from, char **to)
@@ -2231,16 +2368,17 @@ void upper(char *from, char **to)
 MYREAL
 find_chi (long df, MYREAL prob)
 {
-    MYREAL a, b, m;
-    MYREAL xb = 200.0;
-    MYREAL xa = 0.0;
-    MYREAL xm = 5.;
+    double a, b, m;
+    double xb = 200.0;
+    double xa = 0.0;
+    double xm = 5.;
+    double dprob = (double) prob;
     a = probchi (df, xa);
     m = probchi (df, xm);
     b = probchi (df, xb);
-    while (fabs (m - prob) > EPSILON)
+    while (fabs (m - dprob) > EPSILON)
     {
-        if (m < prob)
+        if (m < dprob)
         {
             b = m;
             xb = xm;
@@ -2250,21 +2388,22 @@ find_chi (long df, MYREAL prob)
             a = m;
             xa = xm;
         }
-        xm = (-(b * xa) + prob * xa + a * xb - prob * xb) / (a - b); //(xa + xb)/2.;
+        xm = (-(b * xa) + prob * xa + a * xb - dprob * xb) / (a - b); //(xa + xb)/2.;
 
         m = probchi (df, xm);
     }
-    return xm;
+    return (MYREAL) xm;
 }
 
 
 MYREAL
-probchi (long df, MYREAL chi)
+probchi (long df, double chi)
 {
-    MYREAL prob;
-    MYREAL v = ((MYREAL) df) / 2.;
+  const  MYREAL eps = DBL_EPSILON ;
+    double prob;
+    double v = ((MYREAL) df) / 2.;
     
-    if (chi > DBL_EPSILON && v > DBL_EPSILON)
+    if (chi > eps && v > eps)
     {
         //lg = EXP (LGAMMA (v));
         prob = 1. - incompletegamma (chi / 2., v);
@@ -2436,7 +2575,7 @@ mm2m (long frompop, long topop, long numpop)
 }
 
 ///
-/// Calulates the j and j from a linear abbreviated matrix
+/// Calulates the j and i from a linear abbreviated matrix
 /// position z in {d,d,d,...,d,a,a,...,a,b,b,...,b, c, ...} we know how many populations c (columns or rows) 
 /// exist and wnat to calculate the i,j coordinates in the c x c matrix {{d,a,a,...,a},{b,d,b,...},...}
 /// in the function z above is im and i = frompop, j is topop
@@ -2513,7 +2652,6 @@ m2mml2 (long i, long topop, long numpop)
     }
 }
 
-
 void
 gamma_rates (MYREAL *rate, MYREAL *probcat, long categs, char *input)
 {
@@ -2694,6 +2832,35 @@ void add_to_buffer(char *fp, long *bufsize, char **buffer)
 }
 
 
+/// sticks a text into the printing buffer
+long print_to_buffer(char **buffer, long *maxbufsize, long *pos, const char *fmt, ...)
+{
+  long mypos=0;
+  char *p;
+  va_list ap;
+  p = (char *) mycalloc(MAXBUFSIZE,sizeof(char));
+  va_start(ap, fmt);
+  mypos = vsprintf(p, fmt, ap);
+  va_end(ap);
+  if((*pos + mypos) < (*maxbufsize))
+      {
+	(*pos) += sprintf((*buffer) + (*pos), "%s",p);
+	//	if(*pos + mypos >= *maxbufsize )
+	//  {
+	//    printf("%i> pos=%li + mypos=%li >  maxbufsize=%li \n",myID, *pos, mypos, *maxbufsize); 
+	//  }
+      }
+    else
+      {
+	*maxbufsize = *pos + 4 * mypos; // add some extra space
+	(*buffer) = (char *) myrealloc ((*buffer), (*maxbufsize) * sizeof (char));
+	(*pos) += sprintf((*buffer) + (*pos), "%s",p);
+      }
+    myfree(p);
+    return (*pos);
+}
+
+
 
 // calculates a rough approximation to log(val)
 // max error = 0.0007
@@ -2706,7 +2873,8 @@ void add_to_buffer(char *fp, long *bufsize, char **buffer)
 // The function is based on floating point coding.
 // It's easy to get floor (log2(N)) by isolating exponent part.
 // We can refine the approximation by using the mantissa. This function returns log2(N)
-float fast_log2 (float vval)
+/*
+MYINLINE float fast_log2 (float vval)
 {
     float vv;
     int * const exp_ptr =  (int *) (&vval);
@@ -2729,10 +2897,10 @@ float fast_log2 (float vval)
 //Now we got log2(N), we have to multiply it by ln(2) to get the natural log :
 
 
-float fast_log (float vval)
+MYINLINE float fast_log (float vval)
 {
     float v = fast_log2 (vval) * 0.69314718f;
-    fprintf(stdout,"val= %f fast_log=%f log=%f\n", (float) vval, (float) v, log(vval));
+    //fprintf(stdout,"val= %f fast_log=%f log=%f\n", (float) vval, (float) v, log(vval));
     return v;
 }
 
@@ -2745,7 +2913,8 @@ MYINLINE double fast_exp(double y)
     union 
     { 
         double d; 
-#ifdef LITTLE_ENDIAN 
+      // weird the implementation says ifdef but that does no work at all
+#ifndef LITTLE_ENDIAN 
         struct { int j, i; } n; 
 #else 
         struct { int i, j; } n; 
@@ -2756,3 +2925,4 @@ MYINLINE double fast_exp(double y)
     return eco.d; 
 } 
 
+*/
