@@ -6,7 +6,7 @@
  Unique event polymorphism   R O U T I N E S
  
  Peter Beerli 2000, Seattle
- beerli@csit.fsu.edu
+ beerli@scs.fsu.edu
  
 Copyright 2002 Peter Beerli and Joseph Felsenstein
  
@@ -14,7 +14,7 @@ Copyright 2002 Peter Beerli and Joseph Felsenstein
         and is copyrighted. Of course, we do not guarantee that the software
         works and are not responsible for any damage you may cause or have.
  
-$Id: uep.c 266 2006-02-19 18:09:34Z beerli $
+$Id: uep.c 514 2006-11-09 20:14:08Z beerli $
 */
 /* \file uep.c 
 Unique event polymorphism manipulation
@@ -78,38 +78,38 @@ destroy_uep (world_fmt * world)
     long j;
   if (world->options->uep)
     {
-      free(world->uepanc[0]);
-      free(world->uepanc);
-      free(world->ueplike[0]);
-      free(world->ueplike);
+      myfree(world->uepanc[0]);
+      myfree(world->uepanc);
+      myfree(world->ueplike[0]);
+      myfree(world->ueplike);
       for (locus = 0; locus < loci; locus++)
 	{
 	  for (i = 0; i < storesize; ++i)
 	    {
 	      for (j = 0; j < size; ++j)
 		{
-		  free(world->ueplikestore[locus][i][j]);
-		  free(world->ueptimestore[j].populations);
-		  free(world->ueptimestore[j].ueptime);
+		  myfree(world->ueplikestore[locus][i][j]);
+		  myfree(world->ueptimestore[j].populations);
+		  myfree(world->ueptimestore[j].ueptime);
 		}
-	      free(world->ueplikestore[locus][i]);
-	      free(world->ueptimestore[locus][i]);
-	      free(world->ueprootstore[locus][i]);
+	      myfree(world->ueplikestore[locus][i]);
+	      myfree(world->ueptimestore[locus][i]);
+	      myfree(world->ueprootstore[locus][i]);
 	    }
-	  free(world->ueplikestore[locus]);
-	  free(world->ueptimestore[locus]);
-	  free(world->ueprootstore[locus]);
+	  myfree(world->ueplikestore[locus]);
+	  myfree(world->ueptimestore[locus]);
+	  myfree(world->ueprootstore[locus]);
 	}
-      free(world->ueplikestore);
-      free(world->ueptimestore);
-      free(world->ueprootstore);
+      myfree(world->ueplikestore);
+      myfree(world->ueptimestore);
+      myfree(world->ueprootstore);
       for (j = 0; j < size; ++j)
 	{
-	  free(world->ueptime[j].populations);
-	  free(world->ueptime[j].ueptime);
+	  myfree(world->ueptime[j].populations);
+	  myfree(world->ueptime[j].ueptime);
 	}
-      free(world->ueptime);
-      free(world->oldrootuep);    
+      myfree(world->ueptime);
+      myfree(world->oldrootuep);    
     }
 }
 
@@ -398,7 +398,7 @@ pseudo_tl_uep (ueparray_fmt * xx1, ueparray_fmt * xx2, MYREAL v1, MYREAL v2,
         x1 = &(xx1->s[i]);
         tterm =
             world->options->uepfreq0 * (*x1)[0] + world->options->uepfreq1 * (*x1)[1];
-        summ += (log (tterm) + proposal->mf[i]);
+        summ += (LOG (tterm) + proposal->mf[i]);
     }
     return summ;
 }
@@ -421,7 +421,7 @@ treelike_uep (world_fmt * world, long locus)
         //        scale = p->uepscale[i];
         tterm =
             world->options->uepfreq0 * (*x1)[0] + world->options->uepfreq1 * (*x1)[1];
-        summ += (log (tterm)); //+ scale);
+        summ += (LOG (tterm)); //+ scale);
     }
     return summ;
 }    /* treelikelihood */
@@ -632,9 +632,9 @@ print_uep (FILE * out, world_fmt * world, long copies)
     FPRINTF (out, "-------------------------------------------------\n");
     FPRINTF (out, "Probability=Prob(is in population | uep_anc=1)\n");
     FPRINTF (out, "(*), (**) see above\n");
-    free (prob);
-    free (tyme);
-    free (timep);
+    myfree(prob);
+    myfree(tyme);
+    myfree(timep);
 }
 
 MYREAL *
@@ -1027,11 +1027,11 @@ sum_ueplike (proposal_fmt * proposal)
     }
     if (proposal->world->options->ueprate > 0.0)
     {
-        return log (like) -
+        return LOG (like) -
                (proposal->world->options->ueprate * proposal->treelen);
     }
     else
-        return log (like);
+        return LOG (like);
 }
 
 
@@ -1452,10 +1452,10 @@ ueplikelihood (world_fmt * world)
     }
     if (world->options->ueprate > 0.0)
     {
-        return log (like) - (world->options->ueprate * world->treelen);
+        return LOG (like) - (world->options->ueprate * world->treelen);
     }
     else
-        return log (like);
+        return LOG (like);
 }
 
 void
